@@ -28,8 +28,10 @@
 #include "../src/gtkhtml.h"
 
 #define CUSTOM_KEYMAP_NAME "Custom"
+#define EMACS_KEYMAP_NAME "Emacs like"
+#define MS_KEYMAP_NAME "MS like"
 
-static GtkWidget *capplet, *check, *menu, *option, *bi;
+static GtkWidget *capplet, *check, *bi, *live_spell_check, *language, *live_spell_color;
 static gboolean active = FALSE;
 #ifdef GTKHTML_HAVE_GCONF
 static GConfError  *error  = NULL;
@@ -47,7 +49,7 @@ static gchar *home_rcfile;
 static void
 set_ui ()
 {
-	guint idx;
+	gchar *keymap_name;
 
 	active = FALSE;
 
@@ -55,12 +57,19 @@ set_ui ()
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), actual_prop->magic_links);
 
 	if (!strcmp (actual_prop->keybindings_theme, "emacs")) {
-		idx = 0;
+		keymap_name = EMACS_KEYMAP_NAME;
 	} else if (!strcmp (actual_prop->keybindings_theme, "ms")) {
-		idx = 1;
+		keymap_name = MS_KEYMAP_NAME;
 	} else
-		idx = 2;
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option), idx);
+		keymap_name = CUSTOM_KEYMAP_NAME;
+	gnome_bindings_properties_select_keymap (GNOME_BINDINGS_PROPERTIES (bi), keymap_name);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (live_spell_check), actual_prop->live_spell_check);
+	gtk_widget_set_sensitive (live_spell_color, actual_prop->live_spell_check);
+	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (live_spell_color),
+							actual_prop->spell_error_color.red,
+							actual_prop->spell_error_color.green,
+							actual_prop->spell_error_color.blue, 0);
+	gtk_entry_set_text (GTK_ENTRY (language), actual_prop->language);
 
 	active = TRUE;
 }
@@ -68,6 +77,8 @@ set_ui ()
 static void
 apply (void)
 {
+	gchar *keymap_id, *keymap_name;
+
 	/* bindings */
 	gnome_bindings_properties_save_keymap (GNOME_BINDINGS_PROPERTIES (bi), CUSTOM_KEYMAP_NAME, home_rcfile);
 	gnome_binding_entry_list_destroy (saved_bindings);
@@ -76,9 +87,25 @@ apply (void)
 
 	/* properties */
 	actual_prop->magic_links = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+	keymap_name = gnome_bindings_properties_get_keymap_name (GNOME_BINDINGS_PROPERTIES (bi));
+	if (!strcmp (keymap_name, EMACS_KEYMAP_NAME)) {
+		keymap_id = "emacs";
+	} else if (!strcmp (keymap_name, MS_KEYMAP_NAME)) {
+		keymap_id = "ms";
+	} else
+		keymap_id = "custom";
+
 	g_free (actual_prop->keybindings_theme);
-	actual_prop->keybindings_theme = g_strdup (gtk_object_get_data (GTK_OBJECT (gtk_menu_get_active (GTK_MENU (menu))),
-									"theme"));
+	actual_prop->keybindings_theme = g_strdup (keymap_id);
+
+	actual_prop->live_spell_check = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (live_spell_check));
+	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (live_spell_color),
+				    &actual_prop->spell_error_color.red,
+				    &actual_prop->spell_error_color.green,
+				    &actual_prop->spell_error_color.blue, NULL);
+	g_free (actual_prop->language);
+	actual_prop->language = g_strdup (gtk_entry_get_text (GTK_ENTRY (language)));
+
 #ifdef GTKHTML_HAVE_GCONF
 	gtk_html_class_properties_update (actual_prop, client, saved_prop);
 #else
@@ -108,31 +135,22 @@ revert (void)
 }
 
 static void
-changed (GtkWidget *widget, gpointer data)
+changed (GtkWidget *widget)
 {
 	if (active)
 		capplet_widget_state_changed (CAPPLET_WIDGET (capplet), TRUE);
 }
 
-static void
-setup(void)
+static GtkWidget *
+setup_bindings (void)
 {
-	GtkWidget *vbox, *hbox, *frame, *mi;
+	GtkWidget *vbox;
 	guchar *base, *rcfile;
 
-        capplet = capplet_widget_new();
-
 	vbox  = gtk_vbox_new (FALSE, 3);
-	hbox  = gtk_hbox_new (FALSE, 3);
-	frame = gtk_frame_new (_("Behaviour"));
-	check = gtk_check_button_new_with_label (_("magic links"));
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
 
-	gtk_signal_connect (GTK_OBJECT (check), "toggled", changed, NULL);
-
-	gtk_container_add (GTK_CONTAINER (frame), check);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), frame);
-
-	frame  = gtk_frame_new (_("Keybindings"));
+	/* hbox  = gtk_hbox_new (FALSE, 3);
 	option = gtk_option_menu_new ();
 	menu   = gtk_menu_new ();
 
@@ -142,15 +160,14 @@ setup(void)
         gtk_signal_connect (GTK_OBJECT (mi), "activate", changed, NULL); \
         gtk_object_set_data (GTK_OBJECT (mi), "theme", v);
 
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu);
+	gtk_option_menu_update_contents
 	ADD ("Emacs like", "emacs");
 	ADD ("MS like", "ms");
 	ADD (CUSTOM_KEYMAP_NAME, "custom");
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu);
-	gtk_container_add (GTK_CONTAINER (frame), option);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), frame);
-
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), option, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0); */
 
 #define LOAD(x) \
 	base = g_strconcat ("gtkhtml/keybindingsrc.", x, NULL); \
@@ -166,10 +183,10 @@ setup(void)
 
 	bi = gnome_bindings_properties_new ();
 	gnome_bindings_properties_add_keymap (GNOME_BINDINGS_PROPERTIES (bi),
-					      "Emacs like", "gtkhtml-bindings-emacs", "command",
+					      EMACS_KEYMAP_NAME, "gtkhtml-bindings-emacs", "command",
 					      GTK_TYPE_HTML_COMMAND, FALSE);
 	gnome_bindings_properties_add_keymap (GNOME_BINDINGS_PROPERTIES (bi),
-					      "MS like", "gtkhtml-bindings-ms", "command",
+					      MS_KEYMAP_NAME, "gtkhtml-bindings-ms", "command",
 					      GTK_TYPE_HTML_COMMAND, FALSE);
 	gnome_bindings_properties_add_keymap (GNOME_BINDINGS_PROPERTIES (bi),
 					      CUSTOM_KEYMAP_NAME, "gtkhtml-bindings-custom", "command",
@@ -182,7 +199,73 @@ setup(void)
 	gtk_signal_connect (GTK_OBJECT (bi), "changed", changed, NULL);
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), bi);
 
-        gtk_container_add (GTK_CONTAINER (capplet), vbox);
+	return vbox;
+}
+
+static void
+live_changed (GtkWidget *w)
+{
+	gtk_widget_set_sensitive (live_spell_color, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (live_spell_check)));
+	changed (w);
+}
+
+static GtkWidget *
+setup_behaviour (void)
+{
+	GtkWidget *frame, *vbox, *hbox, *vb1;
+
+	vbox  = gtk_vbox_new (FALSE, 2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+
+	frame = gtk_frame_new (_("Spell checking"));
+	hbox  = gtk_hbox_new (FALSE, 0);
+	vb1   = gtk_vbox_new (FALSE, 2);
+	live_spell_check = gtk_check_button_new_with_label (_("live spell checking"));
+	gtk_signal_connect (GTK_OBJECT (live_spell_check), "toggled", live_changed, NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (vb1), 3);
+	gtk_box_pack_start (GTK_BOX (hbox), live_spell_check, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vb1), hbox, FALSE, FALSE, 0);
+
+	live_spell_color = gnome_color_picker_new ();
+	hbox  = gtk_hbox_new (FALSE, 5);
+	gtk_signal_connect (GTK_OBJECT (live_spell_color), "color_set", changed, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("spell checking color")), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), live_spell_color, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vb1), hbox, FALSE, FALSE, 0);
+
+	language = gtk_entry_new ();
+	hbox  = gtk_hbox_new (FALSE, 5);
+	gtk_signal_connect (GTK_OBJECT (language), "changed", changed, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("language")), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), language, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vb1), hbox, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), vb1);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+
+	frame = gtk_frame_new (_("Miscellaneous"));
+	hbox  = gtk_hbox_new (FALSE, 0);
+	check = gtk_check_button_new_with_label (_("magic links"));
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+	gtk_signal_connect (GTK_OBJECT (check), "toggled", changed, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), check, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), hbox);
+
+	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+
+	return vbox;
+}
+
+static void
+setup(void)
+{
+	GtkWidget *notebook;
+
+        capplet  = capplet_widget_new ();
+	notebook = gtk_notebook_new ();
+
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), setup_bindings (), gtk_label_new (_("Bindings")));
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), setup_behaviour (), gtk_label_new (_("Behaviour")));
+        gtk_container_add (GTK_CONTAINER (capplet), notebook);
         gtk_widget_show_all (capplet);
 
 	set_ui ();
@@ -194,7 +277,7 @@ main (int argc, char **argv)
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
         textdomain (PACKAGE);
 
-        if (gnome_capplet_init ("gtkhtml-properties", VERSION, argc, argv, NULL, 0, NULL) < 0)
+        if (gnome_capplet_init ("gtkhtml-editor-properties", VERSION, argc, argv, NULL, 0, NULL) < 0)
 		return 1;
 
 #ifdef GTKHTML_HAVE_GCONF
