@@ -30,6 +30,9 @@ gtk_html_control_data_new (GtkHTML *html, GtkWidget *vbox)
 
 	ncd->html = html;
 	ncd->vbox = vbox;
+	ncd->properties_dialog = NULL;
+	ncd->properties_types  = NULL;
+	ncd->block_font_style_change = FALSE;
 
 	return ncd;
 }
@@ -43,12 +46,8 @@ gtk_html_control_data_destroy (GtkHTMLControlData *cd)
 	if (cd->replace_dialog) {
 		gtk_html_replace_dialog_destroy (cd->replace_dialog);
 	}
-	if (cd->image_dialog) {
-		gtk_html_image_dialog_destroy (cd->image_dialog);
+	if (cd->rule_dialog) {
+		gtk_html_rule_dialog_destroy (cd->rule_dialog);
 	}
-	if (cd->link_dialog) {
-		gtk_html_link_dialog_destroy (cd->link_dialog);
-	}
-
 	g_free (cd);
 }
