@@ -48,6 +48,7 @@
 #include "link.h"
 #include "body.h"
 #include "spell.h"
+#include "resolver-progressive-impl.h"
 
 #include "editor-control-factory.h"
 #include "gtkhtmldebug.h"
@@ -68,21 +69,24 @@ struct _SetFrameData {
 };
 typedef struct _SetFrameData SetFrameData;
 
+static BonoboGenericFactory *factory = NULL;
+static gint active_controls = 0;
+
 static void
 set_frame_cb (BonoboControl *control,
 	      gpointer data)
 {
-	Bonobo_UIHandler remote_uih;
-	BonoboUIHandler *uih;
+	Bonobo_UIContainer remote_ui_container;
+	BonoboUIComponent *ui_component;
 	GtkHTMLControlData *control_data;
 	GtkWidget *toolbar;
 	GtkWidget *scrolled_window;
 
 	control_data = (GtkHTMLControlData *) data;
 
-	remote_uih = bonobo_control_get_remote_ui_handler (control);
-	uih = bonobo_control_get_ui_handler (control);
-	bonobo_ui_handler_set_container (uih, remote_uih);
+	remote_ui_container = bonobo_control_get_remote_ui_container (control);
+	ui_component = bonobo_control_get_ui_component (control);
+	bonobo_ui_component_set_container (ui_component, remote_ui_container);
 
 	/* Setup the tool bar.  */
 
@@ -98,8 +102,8 @@ set_frame_cb (BonoboControl *control,
 
 	/* Setup the menu bar.  */
 
-	menubar_setup (uih, control_data);
-	toolbar_setup (uih, control_data);
+	menubar_setup (ui_component, control_data);
+	toolbar_setup (ui_component, control_data);
 }
 
 static gint
@@ -119,7 +123,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 		default:
 		}
 		if (run_dialog) {
-			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE);
+			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
 
 			html_cursor_jump_to (e->cursor, e, cd->obj, 0);
 			html_engine_disable_selection (e);
@@ -213,38 +217,43 @@ load_from_file (GtkHTML *html,
 
 static int
 load_from_corba (BonoboControl *control,
+		 GtkHTML *html,
 		 const char *url, 
 		 GtkHTMLStream *handle)
 {
 	HTMLEditor_Resolver resolver;
-	Bonobo_UIHandler remote_uih;
-	BonoboUIHandler *uih;
+	Bonobo_ControlFrame Frame;
 	CORBA_Environment ev;	
+	int ret_val = FALSE;
 
 	CORBA_exception_init (&ev);
-	
-	uih = gtk_type_new (bonobo_ui_handler_get_type());
-	remote_uih = bonobo_control_get_remote_ui_handler (control);
-	uih = BONOBO_UI_HANDLER (bonobo_object_construct (BONOBO_OBJECT(uih), remote_uih));
-	if (remote_uih != CORBA_OBJECT_NIL) {
-		resolver = bonobo_object_query_interface (BONOBO_OBJECT (uih), "IDL:HTMLEditor/Resolver:1.0");
+
+	Frame = bonobo_control_get_control_frame (control);
+	if (Frame != CORBA_OBJECT_NIL) {
+		resolver = Bonobo_Unknown_query_interface (Frame,
+							   "IDL:HTMLEditor/Resolver:1.0",
+							   &ev);
 		
-		if (resolver == CORBA_OBJECT_NIL) {
-			/* g_warning ("Unable to aquire resolver interface"); */
-		} else {
-			/* g_warning ("found resolver - rejoice the masses"); */
-			HTMLEditor_Resolver_loadURL (resolver, CORBA_OBJECT_NIL, url, &ev);
-			if (ev._major != CORBA_NO_EXCEPTION){
-				/* g_warning ("Got exception!!!"); */
-			} else {
-				/* g_warning ("No Exceptions made"); */
-			}		
+		if (resolver != CORBA_OBJECT_NIL) {
+			Bonobo_ProgressiveDataSink sink;
+			BonoboObject *object;
+
+			object = BONOBO_OBJECT (resolver_sink (html, url, handle));
+
+			sink = bonobo_object_corba_objref (object);
 			
-		}
+			HTMLEditor_Resolver_loadURL (resolver, sink, url, &ev);
+			if (ev._major != CORBA_NO_EXCEPTION){
+				g_warning ("Corba load exception");
+			} else {
+				/* g_print ("Corba Load successfull"); */
+				ret_val = TRUE;
+			}
+		} 
 	}
 	CORBA_exception_free (&ev);
 
-	return FALSE;
+	return ret_val;
 }
 
 static void
@@ -258,7 +267,7 @@ url_requested_cb (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointe
 
 	control = BONOBO_CONTROL (data);
 
-	if (load_from_corba (control, url, handle))
+	if (load_from_corba (control, html, url, handle))
 		;
 		/* g_warning ("valid corba reponse"); */
 	else if (load_from_file (html, url, handle))
@@ -278,15 +287,24 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 					     event->x + engine->x_offset,
 					     event->y + engine->y_offset,
 					     NULL, FALSE);
-	if (cd->obj && event->type == GDK_2BUTTON_PRESS && event->button == 1) {
-		cd->releaseId = gtk_signal_connect (GTK_OBJECT (html), "button_release_event", GTK_SIGNAL_FUNC (release), cd);
-	} else if (event->button == 2) {
+	switch (event->button) {
+	case 1:
+		if (event->type == GDK_2BUTTON_PRESS && cd->obj && event->state & GDK_CONTROL_MASK)
+			cd->releaseId = gtk_signal_connect (GTK_OBJECT (html), "button_release_event",
+							    GTK_SIGNAL_FUNC (release), cd);
+		else
+			return TRUE;
+		break;
+	case 2:
 		/* pass this for pasting */
 		return TRUE;
-	} else if (event->button == 3) {
+	case 3:
 		if (popup_show (cd, event))
 			gtk_signal_emit_stop_by_name (GTK_OBJECT (html), "button_press_event");
+		break;
+	default:
 	}
+
 	return FALSE;
 }
 
@@ -294,6 +312,16 @@ static void
 destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
 {
 	gtk_html_control_data_destroy (cd);
+
+	printf ("active--\n");
+	active_controls --;
+
+	printf ("control_destroy %d\n", active_controls);
+
+	if (active_controls)
+		return;
+	bonobo_object_unref (BONOBO_OBJECT (factory));
+	gtk_main_quit ();
 }
 
 static BonoboObject *
@@ -306,9 +334,8 @@ editor_control_factory (BonoboGenericFactory *factory,
 	GtkWidget *html_widget;
 	GtkWidget *vbox;
 	GtkHTMLControlData *control_data;
-		
-	html_widget = gtk_html_new ();
 
+	html_widget = gtk_html_new ();
 	gtk_html_load_empty (GTK_HTML (html_widget));
 	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
 
@@ -318,46 +345,51 @@ editor_control_factory (BonoboGenericFactory *factory,
 	/* g_warning ("Creating a new GtkHTML editor control."); */
 	control = bonobo_control_new (vbox);
 
-	/* Bonobo::PersistStream */
+	if (control) {
 
-	stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
+		active_controls++;
+		printf ("active++\n");
 
-	/* Bonobo::PersistFile */
+		/* Bonobo::PersistStream */
 
-	file_impl = persist_file_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
+		stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
+		bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
 
-	/* Part of the initialization must be done after the control is
-	   embedded in its control frame.  We use the "set_frame" signal to
-	   handle that.  */
+		/* Bonobo::PersistFile */
 
-	control_data = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
+		file_impl = persist_file_impl_new (GTK_HTML (html_widget));
+		bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
 
-	gtk_signal_connect (GTK_OBJECT (control), "set_frame",
-			    GTK_SIGNAL_FUNC (set_frame_cb), control_data);
+		/* Part of the initialization must be done after the control is
+		   embedded in its control frame.  We use the "set_frame" signal to
+		   handle that.  */
 
-	gtk_signal_connect (GTK_OBJECT (control), "destroy",
-			    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
+		control_data = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
 
-	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
-			    GTK_SIGNAL_FUNC (url_requested_cb), control);
+		gtk_signal_connect (GTK_OBJECT (control), "set_frame",
+				    GTK_SIGNAL_FUNC (set_frame_cb), control_data);
 
-	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
-			    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
+		gtk_signal_connect (GTK_OBJECT (control), "destroy",
+				    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
+
+		gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
+				    GTK_SIGNAL_FUNC (url_requested_cb), control);
+
+		gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
+				    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
 
 #ifdef GTKHTML_HAVE_PSPELL
-	gtk_signal_connect (GTK_OBJECT (html_widget), "spell_suggestion_request",
-			    GTK_SIGNAL_FUNC (spell_suggestion_request_cb), control_data);
+		gtk_signal_connect (GTK_OBJECT (html_widget), "spell_suggestion_request",
+				    GTK_SIGNAL_FUNC (spell_suggestion_request_cb), control_data);
 #endif
+	}
+
 	return BONOBO_OBJECT (control);
 }
 
 void
 editor_control_factory_init (void)
 {
-	static BonoboGenericFactory *factory = NULL;
-
 	if (factory != NULL)
 		return;
 
