@@ -87,7 +87,7 @@ paragraph_style_menu_item_activated_cb (GtkWidget *widget,
 	html = GTK_HTML (data);
 	style = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "paragraph_style_value"));
 
-	g_warning ("Setting paragraph style to %d.", style);
+	/* g_warning ("Setting paragraph style to %d.", style); */
 
 	gtk_html_set_paragraph_style (html, style);
 }
@@ -160,7 +160,7 @@ setup_font_size_option_menu (GtkHTMLControlData *cd)
 	menu = gtk_menu_new ();
 	size [2] = 0;
 
-	for (i = 0; i <= GTK_HTML_FONT_STYLE_SIZE_MAX; i++) {
+	for (i = 0; i < GTK_HTML_FONT_STYLE_SIZE_MAX; i++) {
 		GtkWidget *menu_item;
 
 		size [0] = (i>1) ? '+' : '-';
@@ -616,10 +616,11 @@ toolbar_style (GtkHTMLControlData *cd)
 }
 
 void
-toolbar_setup (BonoboUIHandler *uih,
+toolbar_setup (BonoboUIComponent  *uic,
 	       GtkHTMLControlData *cd)
 {
 	BonoboUIHandlerToolbarItem *tree;
+	BonoboUIHandler *uih = bonobo_ui_handler_new_from_component (uic);
 
 	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
