@@ -24,14 +24,34 @@
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
 
+#define DEFAULT_FONT_SIZE   12
+#define DEFAULT_FONT_SIZE_S "12"
+
+#define STRINGIZE(x) #x
+
 GtkHTMLClassProperties *
 gtk_html_class_properties_new (void)
 {
 	GtkHTMLClassProperties *p = g_new (GtkHTMLClassProperties, 1);
 
 	/* default values */
-	p->magic_links = TRUE;
-	p->keybindings_theme = g_strdup ("emacs");
+	p->magic_links             = TRUE;
+	p->keybindings_theme       = g_strdup ("emacs");
+	p->font_var_family         = g_strdup ("helvetica");
+	p->font_fix_family         = g_strdup ("courier");
+	p->font_var_size           = DEFAULT_FONT_SIZE;
+	p->font_fix_size           = DEFAULT_FONT_SIZE;
+	p->font_var_family_print   = g_strdup ("helvetica");
+	p->font_fix_family_print   = g_strdup ("courier");
+	p->font_var_size_print     = DEFAULT_FONT_SIZE;
+	p->font_fix_size_print     = DEFAULT_FONT_SIZE;
+	p->animations              = TRUE;
+
+	p->live_spell_check        = TRUE;
+	p->spell_error_color.red   = 0xffff;
+	p->spell_error_color.green = 0;
+	p->spell_error_color.blue  = 0;
+	p->language                = g_strdup ("en");
 
 	return p;
 }
@@ -61,8 +81,28 @@ gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
 	g_assert (client);
 
 	GET (bool, "/magic_links", magic_links,,);
+	GET (bool, "/animations", animations,,);
 	GET (string, "/keybindings_theme", keybindings_theme,
 	     g_free (p->keybindings_theme), g_strdup);
+	GET (string, "/font_variable_family", font_var_family,
+	     g_free (p->font_var_family), g_strdup);
+	GET (string, "/font_fixed_family", font_fix_family,
+	     g_free (p->font_fix_family), g_strdup);
+	GET (int, "/font_variable_size", font_var_size,,);
+	GET (int, "/font_fixed_size", font_fix_size,,);
+	GET (string, "/font_variable_family_print", font_var_family_print,
+	     g_free (p->font_var_family_print), g_strdup);
+	GET (string, "/font_fixed_family_print", font_fix_family_print,
+	     g_free (p->font_fix_family_print), g_strdup);
+	GET (int, "/font_variable_size_print", font_var_size_print,,);
+	GET (int, "/font_fixed_size_print", font_fix_size_print,,);
+
+	GET (bool, "/live_spell_check", live_spell_check,,);
+	GET (int, "/spell_error_color_red",   spell_error_color.red,,);
+	GET (int, "/spell_error_color_green", spell_error_color.green,,);
+	GET (int, "/spell_error_color_blue",  spell_error_color.blue,,);
+	GET (string, "/language", language,
+	     g_free (p->language), g_strdup);
 }
 
 #define SET(t,x,prop) \
@@ -76,20 +116,86 @@ gtk_html_class_properties_update (GtkHTMLClassProperties *p, GConfClient *client
 {
 	gchar *key;
 
+	if (p->animations != old->animations)
+		SET (bool, "/animations", animations);
 	if (p->magic_links != old->magic_links)
 		SET (bool, "/magic_links", magic_links);
 	SET (string, "/keybindings_theme", keybindings_theme);
+	if (strcmp (p->font_var_family, old->font_var_family))
+		SET (string, "/font_variable_family", font_var_family);
+	if (strcmp (p->font_fix_family, old->font_fix_family))
+		SET (string, "/font_fixed_family", font_fix_family);
+	if (p->font_var_size != old->font_var_size)
+		SET (int, "/font_variable_size", font_var_size);
+	if (p->font_fix_size != old->font_fix_size)
+		SET (int, "/font_fixed_size", font_fix_size);
+	if (strcmp (p->font_var_family_print, old->font_var_family_print))
+		SET (string, "/font_variable_family_print", font_var_family_print);
+	if (strcmp (p->font_fix_family_print, old->font_fix_family_print))
+		SET (string, "/font_fixed_family_print", font_fix_family_print);
+	if (p->font_var_size_print != old->font_var_size_print)
+		SET (int, "/font_variable_size_print", font_var_size_print);
+	if (p->font_fix_size_print != old->font_fix_size_print)
+		SET (int, "/font_fixed_size_print", font_fix_size_print);
+
+	if (p->live_spell_check != old->live_spell_check)
+		SET (bool, "/live_spell_check", live_spell_check);
+	if (!gdk_color_equal (&p->spell_error_color, &old->spell_error_color)) {
+		SET (int, "/spell_error_color_red",   spell_error_color_red);
+		SET (int, "/spell_error_color_green", spell_error_color_green);
+		SET (int, "/spell_error_color_blue",  spell_error_color_blue);
+	}
+	if (strcmp (p->language, old->language))
+		SET (string, "/language", language);
 }
 
 #else
+
+#undef GET
+#define GET(t,v,s) \
+	p-> ## v = gnome_config_get_ ## t (s)
+#define GETS(v,s) \
+        g_free (p-> ## v); \
+        GET(string,v,s)
+
+
 void
 gtk_html_class_properties_load (GtkHTMLClassProperties *p)
 {
+	char *s;
+
 	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
-	p->magic_links = gnome_config_get_bool ("magic_links=true");
-	p->keybindings_theme = gnome_config_get_string ("keybindings_theme=ms");
+	GET  (bool, magic_links, "magic_links=true");
+	GET  (bool, animations, "animations=true");
+	GETS (keybindings_theme, "keybindings_theme=ms");
+	GETS (font_var_family, "font_variable_family=helvetica");
+	GETS (font_fix_family, "font_fixed_family=courier");
+	GETS (font_var_family_print, "font_variable_family_print=helvetica");
+	GETS (font_fix_family_print, "font_fixed_family_print=courier");
+
+	s = g_strdup_printf ("font_variable_size=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_var_size, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_fixed_size=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_fix_size, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_variable_size_print=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_var_size_print, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_fixed_size_print=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_fix_size_print, s);
+	g_free (s);
+
+	GET  (bool, live_spell_check, "live_spell_check=true");
+	GET  (int, spell_error_color.red,   "spell_error_color_red=0xffff");
+	GET  (int, spell_error_color.green, "spell_error_color_green=0");
+	GET  (int, spell_error_color.blue,  "spell_error_color_blue=0");
+	GETS (language, "language=en");
+
 	gnome_config_pop_prefix ();
-	gnome_config_sync ();
 }
 
 void
@@ -97,19 +203,52 @@ gtk_html_class_properties_save (GtkHTMLClassProperties *p)
 {
 	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
 	gnome_config_set_bool ("magic_links", p->magic_links);
+	gnome_config_set_bool ("animations", p->animations);
 	gnome_config_set_string ("keybindings_theme", p->keybindings_theme);
+	gnome_config_set_string ("font_variable_family", p->font_var_family);
+	gnome_config_set_string ("font_fixed_family", p->font_fix_family);
+	gnome_config_set_int ("font_variable_size", p->font_var_size);
+	gnome_config_set_int ("font_fixed_size", p->font_fix_size);
+	gnome_config_set_string ("font_variable_family_print", p->font_var_family_print);
+	gnome_config_set_string ("font_fixed_family_print", p->font_fix_family_print);
+	gnome_config_set_int ("font_variable_size_print", p->font_var_size_print);
+	gnome_config_set_int ("font_fixed_size_print", p->font_fix_size_print);
+
+	gnome_config_set_bool ("live_spell_check", p->live_spell_check);
+	gnome_config_set_int  ("spell_error_color_red",   p->spell_error_color.red);
+	gnome_config_set_int  ("spell_error_color_green", p->spell_error_color.green);
+	gnome_config_set_int  ("spell_error_color_blue",  p->spell_error_color.blue);
+	gnome_config_set_string ("language", p->language);
+
 	gnome_config_pop_prefix ();
 	gnome_config_sync ();
 }
 #endif
 
+#define COPYS(v) \
+        g_free (p1-> ## v); \
+        p1->## v = g_strdup (p2-> ## v);
+#define COPY(v) \
+        p1-> ## v = p2-> ## v;
+
 void
 gtk_html_class_properties_copy (GtkHTMLClassProperties *p1,
 				GtkHTMLClassProperties *p2)
 {
-	p1->magic_links = p2->magic_links;
-	g_free (p1->keybindings_theme);
-	p1->keybindings_theme = g_strdup (p2->keybindings_theme);
+	COPY  (magic_links);
+	COPYS (keybindings_theme);
+	COPYS (font_var_family);
+	COPYS (font_fix_family);
+	COPY  (font_var_size);
+	COPY  (font_fix_size);
+	COPYS (font_var_family_print);
+	COPYS (font_fix_family_print);
+	COPY  (font_var_size_print);
+	COPY  (font_fix_size_print);
+
+	COPY  (live_spell_check);
+	COPY  (spell_error_color);
+	COPYS (language);
 }
 
 /* enums */
@@ -195,6 +334,12 @@ static GtkEnumValue _gtk_html_command_values[] = {
   { GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD, "GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD", "spell-personal-add" },
   { GTK_HTML_COMMAND_SPELL_SESSION_DICTIONARY_ADD, "GTK_HTML_COMMAND_SPELL_SESSION_DICTIONARY_ADD", "spell-session-add" },
 #endif
+  { GTK_HTML_COMMAND_SEARCH_INCREMENTAL_FORWARD, "GTK_HTML_COMMAND_SEARCH_INCREMENTAL_FORWARD", "isearch-forward" },
+  { GTK_HTML_COMMAND_SEARCH_INCREMENTAL_BACKWARD, "GTK_HTML_COMMAND_SEARCH_INCREMENTAL_BACKWARD", "isearch-backward" },
+  { GTK_HTML_COMMAND_SEARCH, "GTK_HTML_COMMAND_SEARCH", "search" },
+  { GTK_HTML_COMMAND_SEARCH_REGEX, "GTK_HTML_COMMAND_SEARCH_REGEX", "search-regex" },
+  { GTK_HTML_COMMAND_FOCUS_FORWARD, "GTK_HTML_COMMAND_FOCUS_FORWARD", "focus-forward" },
+  { GTK_HTML_COMMAND_FOCUS_BACKWARD, "GTK_HTML_COMMAND_FOCUS_BACKWARD", "focus-backward" },
   { 0, NULL, NULL }
 };
 
