@@ -26,7 +26,6 @@
 #define GTK_HTML_GCONF_DIR "/GNOME/Documents/HTML_Editor"
 #define GTK_HTML_GNOME_CONFIG_PREFIX "/gtkhtml/Settings/"
 
-typedef enum _GtkHTMLClassPropertiesItem GtkHTMLClassPropertiesItem;
 typedef struct _GtkHTMLClassProperties GtkHTMLClassProperties;
 
 #include <glib.h>
@@ -37,14 +36,25 @@ typedef struct _GtkHTMLClassProperties GtkHTMLClassProperties;
 #include <libgnome/gnome-config.h>
 #endif
 
-enum _GtkHTMLClassPropertiesItem {
-	GTK_HTML_CLASS_PROPERTIES_MAGIC_LINKS,
-	GTK_HTML_CLASS_PROPERTIES_KEYBINDINGS,
-};
-
 struct _GtkHTMLClassProperties {
+	/* viewer */
+	gboolean  animations;
+	gchar    *font_var_family;
+	guint     font_var_size;
+	gchar    *font_fix_family;
+	guint     font_fix_size;
+	gchar    *font_var_family_print;
+	guint     font_var_size_print;
+	gchar    *font_fix_family_print;
+	guint     font_fix_size_print;
+
+	/* editor */
 	gboolean  magic_links;
 	gchar    *keybindings_theme;
+
+	gboolean  live_spell_check;
+	GdkColor  spell_error_color;
+	gchar    *language;
 };
 
 
