@@ -1,3 +1,4 @@
+
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  This file is part of the GtkHTML library.
 
@@ -24,6 +25,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <gal/widgets/e-unicode.h>
 
 #include <gnome.h>
 
@@ -42,7 +44,10 @@
 #include "htmlcolorset.h"
 
 #include "gtkhtml-embedded.h"
+#include "gtkhtml-im.h"
+#include "gtkhtml-input.h"
 #include "gtkhtml-keybinding.h"
+#include "gtkhtml-search.h"
 #include "gtkhtml-stream.h"
 #include "gtkhtml-private.h"
 #include <libgnome/gnome-util.h>
@@ -76,22 +81,21 @@ enum {
 	SCROLL,
 	CURSOR_MOVE,
 	INSERT_PARAGRAPH,
-#ifdef GTKHTML_HAVE_PSPELL
 	/* spell checking */
 	SPELL_SUGGESTION_REQUEST,
-#endif
 	/* now only last signal */
 	LAST_SIGNAL
 };
 static guint signals [LAST_SIGNAL] = { 0 };
 
 /* keybindings signal hadlers */
-static void scroll              (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
-static void cursor_move         (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
-static void command             (GtkHTML *html, GtkHTMLCommandType com_type);
-static gint mouse_change_pos    (GtkWidget *widget, gint x, gint y);
+static void scroll                 (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
+static void cursor_move            (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
+static void command                (GtkHTML *html, GtkHTMLCommandType com_type);
+static gint mouse_change_pos       (GtkWidget *widget, gint x, gint y);
+static void load_keybindings       (GtkHTMLClass *klass);
+static void set_editor_keybindings (GtkHTML *html, gboolean editable);
 
-static void load_keybindings    (GtkHTMLClass *klass);
 
 /* Values for selection information.  FIXME: what about COMPOUND_STRING and
    TEXT?  */
@@ -219,21 +223,21 @@ update_styles (GtkHTML *html)
 	clueflow_style = html_engine_get_current_clueflow_style (engine);
 	paragraph_style = clueflow_style_to_paragraph_style (clueflow_style);
 
-	if (paragraph_style != html->paragraph_style) {
-		html->paragraph_style = paragraph_style;
+	if (paragraph_style != html->priv->paragraph_style) {
+		html->priv->paragraph_style = paragraph_style;
 		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_STYLE_CHANGED],
 				 paragraph_style);
 	}
 
 	indentation = html_engine_get_current_clueflow_indentation (engine);
-	if (indentation != html->paragraph_indentation) {
-		html->paragraph_style = paragraph_style;
+	if (indentation != html->priv->paragraph_indentation) {
+		html->priv->paragraph_style = paragraph_style;
 		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_STYLE_CHANGED], paragraph_style);
 	}
 
 	alignment = html_alignment_to_paragraph (html_engine_get_current_clueflow_alignment (engine));
-	if (alignment != html->paragraph_alignment) {
-		html->paragraph_alignment = alignment;
+ 	if (alignment != html->priv->paragraph_alignment) {
+		html->priv->paragraph_alignment = alignment;
 		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_ALIGNMENT_CHANGED], alignment);
 	}
 
@@ -267,15 +271,15 @@ idle_handler (gpointer data)
 
 	html_engine_flush_draw_queue (engine);
 
-	html->idle_handler_id = 0;
+ 	html->priv->idle_handler_id = 0;
 	return FALSE;
 }
 
 static void
 queue_draw (GtkHTML *html)
 {
-	if (html->idle_handler_id == 0)
-		html->idle_handler_id = gtk_idle_add (idle_handler, html);
+	if (html->priv->idle_handler_id == 0)
+		html->priv->idle_handler_id = gtk_idle_add (idle_handler, html);
 }
 
 
@@ -513,8 +517,7 @@ scroll_timeout_cb (gpointer data)
 		engine = html->engine;
 		html_engine_select_region (engine,
 					   html->selection_x1, html->selection_y1,
-					   x + engine->x_offset, y + engine->y_offset,
-					   TRUE);
+					   x + engine->x_offset, y + engine->y_offset);
 	}
 
 	layout = GTK_LAYOUT (widget);
@@ -532,10 +535,10 @@ scroll_timeout_cb (gpointer data)
 static void
 setup_scroll_timeout (GtkHTML *html)
 {
-	if (html->scroll_timeout_id != 0)
+	if (html->priv->scroll_timeout_id != 0)
 		return;
 
-	html->scroll_timeout_id = gtk_timeout_add (SCROLL_TIMEOUT_INTERVAL,
+	html->priv->scroll_timeout_id = gtk_timeout_add (SCROLL_TIMEOUT_INTERVAL,
 						   scroll_timeout_cb, html);
 
 	GDK_THREADS_LEAVE();
@@ -546,11 +549,11 @@ setup_scroll_timeout (GtkHTML *html)
 static void
 remove_scroll_timeout (GtkHTML *html)
 {
-	if (html->scroll_timeout_id == 0)
+	if (html->priv->scroll_timeout_id == 0)
 		return;
 
-	gtk_timeout_remove (html->scroll_timeout_id);
-	html->scroll_timeout_id = 0;
+	gtk_timeout_remove (html->priv->scroll_timeout_id);
+	html->priv->scroll_timeout_id = 0;
 }
 
 
@@ -564,20 +567,23 @@ destroy (GtkObject *object)
 	html = GTK_HTML (object);
 
 	g_free (html->pointer_url);
-
 	gdk_cursor_destroy (html->hand_cursor);
 	gdk_cursor_destroy (html->arrow_cursor);
 	gdk_cursor_destroy (html->ibeam_cursor);
 
 	connect_adjustments (html, NULL, NULL);
 
-	if (html->idle_handler_id != 0)
-		gtk_idle_remove (html->idle_handler_id);
+	if (html->priv->idle_handler_id != 0)
+		gtk_idle_remove (html->priv->idle_handler_id);
 
-	if (html->scroll_timeout_id != 0)
-		gtk_timeout_remove (html->scroll_timeout_id);
+	if (html->priv->scroll_timeout_id != 0)
+		gtk_timeout_remove (html->priv->scroll_timeout_id);
+	
+	g_free (html->priv);
+	html->priv = NULL;
 
 	gtk_object_destroy (GTK_OBJECT (html->engine));
+	gtk_html_input_line_destroy (html->input_line);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy != NULL)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -590,12 +596,17 @@ style_set (GtkWidget *widget,
 	   GtkStyle  *previous_style)
 {
 	HTMLEngine *engine = GTK_HTML (widget)->engine;
-	
+       
 	html_colorset_set_style (engine->defaultSettings->color_set,
 				 widget->style);
 	html_colorset_set_unchanged (engine->settings->color_set,
 				     engine->defaultSettings->color_set);
 	html_engine_schedule_update (engine);
+
+#ifdef GTKHTML_USE_XIM
+	if (previous_style)
+		gtk_html_im_style_set (widget);
+#endif
 }
 
 static gint
@@ -607,22 +618,26 @@ key_press_event (GtkWidget *widget,
 	gint position = html->engine->cursor->position;
 
 	html->binding_handled = FALSE;
-	gtk_bindings_activate (GTK_OBJECT (widget), event->keyval, event->state);
+	if (html->editor_bindings && html_engine_get_editable (html->engine))
+		gtk_binding_set_activate (html->editor_bindings, event->keyval, event->state, GTK_OBJECT (widget));
+	if (!html->binding_handled)
+		gtk_bindings_activate (GTK_OBJECT (widget), event->keyval, event->state);
 	retval = html->binding_handled;
 
-
 	if (! retval
 	    && html_engine_get_editable (html->engine)
 	    && ! (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
 	    && event->length > 0) {
+		gchar *str;
 		html_engine_delete_selection (html->engine, TRUE);
-		html_engine_insert (html->engine, event->string, event->length);
+		str = e_utf8_from_gtk_event_key (widget, event->keyval, event->string);
+		/* printf ("len: %d str: %s\n", unicode_strlen (str, -1), str); */
+		if (str)
+			html_engine_insert (html->engine, str, unicode_strlen (str, -1));
+		g_free (str);
 		retval = TRUE;
 	}
 
-	if (!retval && GTK_WIDGET_CLASS (parent_class)->key_press_event)
-		retval = GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
-
 	if (retval && html_engine_get_editable (html->engine))
 		html_engine_reset_blinking_cursor (html->engine);
 
@@ -652,6 +667,7 @@ realize (GtkWidget *widget)
 	gdk_window_set_events (html->layout.bin_window,
 			       (gdk_window_get_events (html->layout.bin_window)
 				| GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
+				| GDK_ENTER_NOTIFY_MASK
 				| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
 				| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK));
 
@@ -662,6 +678,10 @@ realize (GtkWidget *widget)
 	/* This sets the backing pixmap to None, so that scrolling does not
            erase the newly exposed area, thus making the thing smoother.  */
 	gdk_window_set_back_pixmap (html->layout.bin_window, NULL, FALSE);
+
+#ifdef GTKHTML_USE_XIM
+	gtk_html_im_realize (html);
+#endif /* GTKHTML_USE_XIM */
 }
 
 static void
@@ -671,6 +691,9 @@ unrealize (GtkWidget *widget)
 	
 	html_engine_unrealize (html->engine);
 
+#ifdef GTKHTML_USE_XIM	
+	gtk_html_im_unrealize (html);
+#endif
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
@@ -720,45 +743,59 @@ size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 	if (html->engine->width != allocation->width
 	    || html->engine->height != allocation->height) {
-		html->engine->width = allocation->width;
-		html->engine->height = allocation->height;
+		HTMLEngine *e = html->engine;
 
-		html_engine_calc_size (html->engine);
+		e->width  = allocation->width;
+		e->height = allocation->height;
+
+		if (e->clue && e->painter
+		    && html_object_calc_min_width (e->clue, e->painter) < e->width - e->leftBorder - e->rightBorder)
+			html_engine_calc_size (html->engine);
 
 		gtk_html_private_calc_scrollbars (html);
 	}
+
+#ifdef GTKHTML_USE_XIM
+	gtk_html_im_size_allocate (html);
+#endif
 }
 
 static void
-on_url (GtkWidget *widget, HTMLObject *obj)
+on_object (GtkWidget *widget, HTMLObject *obj)
 {
 	GtkHTML *html = GTK_HTML (widget);
 	const gchar *url;
 
-	g_return_if_fail (obj);
+	if (obj) {
+		if ((url = (obj) ? html_object_get_url (obj) : NULL)) {
+			if (html->pointer_url == NULL || strcmp (html->pointer_url, url) != 0) {
+				g_free (html->pointer_url);
+				html->pointer_url = g_strdup (url);
+				gtk_signal_emit (GTK_OBJECT (html), signals[ON_URL], url);
+			}
 
-	if ((url = (obj) ? html_object_get_url (obj) : NULL)) {
-		if (html->pointer_url == NULL || strcmp (html->pointer_url, url) != 0) {
-			g_free (html->pointer_url);
-			html->pointer_url = g_strdup (url);
-			gtk_signal_emit (GTK_OBJECT (html), signals[ON_URL], url);
+			if (html->engine->editable)
+				gdk_window_set_cursor (widget->window, html->ibeam_cursor);
+			else
+				gdk_window_set_cursor (widget->window, html->hand_cursor);
+		} else {
+			if (html->pointer_url != NULL) {
+				g_free (html->pointer_url);
+				html->pointer_url = NULL;
+				gtk_signal_emit (GTK_OBJECT (html), signals [ON_URL], NULL);
+			}
+
+			if (obj != NULL && html_object_is_text (obj) && html->allow_selection)
+				gdk_window_set_cursor (widget->window, html->ibeam_cursor);
+			else
+				gdk_window_set_cursor (widget->window, html->arrow_cursor);
 		}
-
-		if (html->engine->editable)
-			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
-		else
-			gdk_window_set_cursor (widget->window, html->hand_cursor);
 	} else {
-		if (html->pointer_url != NULL) {
-			g_free (html->pointer_url);
+		if (html->pointer_url) {
 			html->pointer_url = NULL;
-			gtk_signal_emit (GTK_OBJECT (html), signals [ON_URL], NULL);
+			g_free (html->pointer_url);
 		}
-
-		if (obj != NULL && html_object_is_text (obj) && html->allow_selection)
-			gdk_window_set_cursor (widget->window, html->ibeam_cursor);
-		else
-			gdk_window_set_cursor (widget->window, html->arrow_cursor);
+		gdk_window_set_cursor (widget->window, html->arrow_cursor);
 	}
 }
 
@@ -770,6 +807,9 @@ mouse_change_pos (GtkWidget *widget, gint x, gint y)
 	HTMLObject *obj;
 	HTMLType type;
 
+	if (!GTK_WIDGET_REALIZED (widget))
+		return FALSE;
+
 	html = GTK_HTML (widget);
 	engine = html->engine;
 
@@ -777,7 +817,9 @@ mouse_change_pos (GtkWidget *widget, gint x, gint y)
 					 x + engine->x_offset, y + engine->y_offset,
 					 NULL, FALSE);
 
-	if (html->button_pressed && html->allow_selection) {
+	if (html->button1_pressed && html->allow_selection) {
+		gboolean need_scroll;
+
 		if (obj) {
 			type = HTML_OBJECT_TYPE (obj);
 
@@ -798,8 +840,25 @@ mouse_change_pos (GtkWidget *widget, gint x, gint y)
 
 		html->in_selection = TRUE;
 
-		if (x < 0 || x >= widget->allocation.width
-		    || y < 0 || y >= widget->allocation.height)
+		need_scroll = FALSE;
+
+		if (x < 0) {
+			x = 0;
+			need_scroll = TRUE;
+		} else if (x >= widget->allocation.width) {
+			x = widget->allocation.width - 1;
+			need_scroll = TRUE;
+		}
+
+		if (y < 0) {
+			y = 0;
+			need_scroll = TRUE;
+		} else if (y >= widget->allocation.height) {
+			y = widget->allocation.height - 1;
+			need_scroll = TRUE;
+		}
+
+		if (need_scroll)
 			setup_scroll_timeout (html);
 		else
 			remove_scroll_timeout (html);
@@ -811,12 +870,10 @@ mouse_change_pos (GtkWidget *widget, gint x, gint y)
 
 		html_engine_select_region (engine,
 					   html->selection_x1, html->selection_y1,
-					   x + engine->x_offset, y + engine->y_offset,
-					   TRUE);
+					   x + engine->x_offset, y + engine->y_offset);
 	}
 
-	if (obj)
-		on_url (widget, obj);
+	on_object (widget, obj);
 
 	return TRUE;
 }
@@ -843,7 +900,7 @@ motion_notify_event (GtkWidget *widget,
 		return FALSE;
 
 	engine = GTK_HTML (widget)->engine;
-	if (GTK_HTML (widget)->button_pressed && html_engine_get_editable (engine))
+	if (GTK_HTML (widget)->button1_pressed && html_engine_get_editable (engine))
 		html_engine_jump_at (engine,
 				     event->x + engine->x_offset,
 				     event->y + engine->y_offset);
@@ -865,8 +922,8 @@ button_press_event (GtkWidget *widget,
 
 	if (event->type == GDK_BUTTON_PRESS) {
 		GtkAdjustment *vadj;
-			
-		vadj = GTK_LAYOUT (widget)->vadjustment;
+
+		vadj   = GTK_LAYOUT ((html->iframe_parent) ? html->iframe_parent : widget)->vadjustment;
 		
 		switch (event->button) {
 		case 4:
@@ -890,8 +947,13 @@ button_press_event (GtkWidget *widget,
 			return TRUE;
 			break;
 		case 2:
-			gtk_html_request_paste (widget, event->time);
-			return TRUE;
+			if (html_engine_get_editable (engine)) {
+				gtk_html_request_paste (widget, event->time);
+				return TRUE;
+			}
+			break;
+		case 1:
+			html->button1_pressed = TRUE;
 			break;
 		default:
 			break;
@@ -906,26 +968,33 @@ button_press_event (GtkWidget *widget,
 	}
 
 	if (html->allow_selection && !(event->state & GDK_SHIFT_MASK)) {
-		gtk_grab_add (widget);
-		gdk_pointer_grab (widget->window, TRUE,
-				  (GDK_BUTTON_RELEASE_MASK
-				   | GDK_BUTTON_MOTION_MASK
-				   | GDK_POINTER_MOTION_HINT_MASK),
-				  NULL, NULL, 0);
+		if (gdk_pointer_grab (GTK_LAYOUT (widget)->bin_window, FALSE,
+				      (GDK_BUTTON_RELEASE_MASK
+				       | GDK_BUTTON_MOTION_MASK
+				       | GDK_POINTER_MOTION_HINT_MASK),
+				      NULL, NULL, 0) == 0) {
+			html->selection_x1 = event->x + engine->x_offset;
+			html->selection_y1 = event->y + engine->y_offset;
 
-		html->selection_x1 = event->x + engine->x_offset;
-		html->selection_y1 = event->y + engine->y_offset;
+			if (event->button == 1) {
+				if (event->type == GDK_2BUTTON_PRESS && !event->state)
+					gtk_html_select_word (html);
+				else if (event->type == GDK_3BUTTON_PRESS && !event->state)
+					gtk_html_select_line (html);
+			}
+		}
 	}
 
-	html->button_pressed = TRUE;
-
-	if (!(event->state & GDK_SHIFT_MASK))
-		html_engine_disable_selection (engine);
-	else if (html->allow_selection)
-		html_engine_select_region (engine,
-					   html->selection_x1, html->selection_y1,
-					   event->x + engine->x_offset, event->y + engine->y_offset,
-					   TRUE);
+	if (!(event->button == 1
+	      && (event->type == GDK_2BUTTON_PRESS || event->type == GDK_3BUTTON_PRESS)
+	      && !event->state)) {
+		if (!(event->state & GDK_SHIFT_MASK))
+			html_engine_disable_selection (engine);
+		else if (html->allow_selection)
+			html_engine_select_region (engine,
+						   html->selection_x1, html->selection_y1,
+						   event->x + engine->x_offset, event->y + engine->y_offset);
+	}
 
 	return TRUE;
 }
@@ -941,10 +1010,11 @@ button_release_event (GtkWidget *widget,
 	gtk_grab_remove (widget);
 	gdk_pointer_ungrab (0);
 
-	if (event->button == 1 && html->pointer_url != NULL && ! html->in_selection)
-		gtk_signal_emit (GTK_OBJECT (widget), signals[LINK_CLICKED], html->pointer_url);
-
-	html->button_pressed = FALSE;
+	if (event->button == 1) {
+		html->button1_pressed = FALSE;
+		if (html->pointer_url != NULL && ! html->in_selection)
+			gtk_signal_emit (GTK_OBJECT (widget), signals[LINK_CLICKED], html->pointer_url);
+	}
 
 	if (html->in_selection) {
 		gtk_selection_owner_set (widget, GDK_SELECTION_PRIMARY, event->time);
@@ -962,11 +1032,19 @@ focus_in_event (GtkWidget *widget,
 		GdkEventFocus *event)
 {
 	GtkHTML *html = GTK_HTML (widget);
+
 	if (!html->iframe_parent) {
 		GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
 		html_engine_set_focus (html->engine, TRUE);
-	} else
-		gtk_window_set_focus (GTK_WINDOW (html->iframe_parent->window), html->iframe_parent);
+	} else {
+		GtkWidget *window = gtk_widget_get_ancestor (widget, gtk_window_get_type ());
+		if (window)
+			gtk_window_set_focus (GTK_WINDOW (window), html->iframe_parent);
+	}
+
+#ifdef GTKHTML_USE_XIM
+	gtk_html_im_focus_in (html);
+#endif
 
 	return FALSE;
 }
@@ -981,9 +1059,20 @@ focus_out_event (GtkWidget *widget,
 		html_engine_set_focus (html->engine, FALSE);
 	}
 
+#ifdef GTKHTML_USE_XIM
+	gtk_html_im_focus_out (html);
+#endif
+
 	return FALSE;
 }
 
+static gint
+enter_notify_event (GtkWidget *widget, GdkEventCrossing *event)
+{
+	mouse_change_pos (widget, event->x, event->y);
+	return TRUE;
+}
+
 
 /* X11 selection support.  */
 
@@ -1007,7 +1096,7 @@ selection_get (GtkWidget        *widget,
 			{
 				gtk_selection_data_set (selection_data_ptr,
 							GDK_SELECTION_TYPE_STRING, 8,
-							selection_string, 
+							(const guchar *) selection_string, 
 							strlen (selection_string));
 			}
 		else if ((info == TARGET_TEXT) || (info == TARGET_COMPOUND_TEXT))
@@ -1074,7 +1163,7 @@ selection_received (GtkWidget *widget,
 		html_engine_delete_selection (GTK_HTML (widget)->engine, TRUE);
 		html_engine_insert (GTK_HTML (widget)->engine, 
 				    selection_data->data,
-				    selection_data->length);
+				    (guint) selection_data->length);
 	}
 }  
 
@@ -1129,15 +1218,104 @@ set_adjustments (GtkLayout     *layout,
 
 /* Initialization.  */
 
+static gint
+set_fonts_idle (GtkHTML *html)
+{
+	GtkHTMLClassProperties *prop = GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties;
+
+	if (html->engine) {
+		html_font_manager_set_default (&html->engine->painter->font_manager,
+					       prop->font_var_family, prop->font_fix_family,
+					       prop->font_var_size,   prop->font_fix_size);
+
+		/* tables don't resize correctly :( who knows the solution? */
+		if (html->engine->clue) {
+			html_object_reset (html->engine->clue);
+			html_object_change_set_down (html->engine->clue, HTML_CHANGE_ALL);
+			html_engine_calc_size (html->engine);
+			html_engine_schedule_update (html->engine);
+		}
+	}
+#ifdef GTKHTML_HAVE_PSPELL
+	html->priv->set_font_id = 0;
+#endif
+
+	return FALSE;
+}
+
 #ifdef GTKHTML_HAVE_GCONF
 
 static void
-client_notify (GConfClient* client,
-	       guint cnxn_id,
-	       const gchar* key,
-	       GConfValue* value,
-	       gboolean is_default,
-	       gpointer user_data)
+set_fonts (GtkHTML *html)
+{
+#ifdef GTKHTML_HAVE_PSPELL
+	if (!html->priv->set_font_id)
+		html->priv->set_font_id = gtk_idle_add ((GtkFunction) set_fonts_idle, html);
+#else
+	set_fonts_idle (html);
+#endif
+}
+
+static void
+client_notify_widget (GConfClient* client,
+		      guint cnxn_id,
+		      const gchar* key,
+		      GConfValue* value,
+		      gboolean is_default,
+		      gpointer user_data)
+{
+	GtkHTML *html = (GtkHTML *) user_data;
+	GtkHTMLClass *klass = GTK_HTML_CLASS (GTK_OBJECT (html)->klass);
+	GtkHTMLClassProperties *prop = klass->properties;	
+	gchar *tkey;
+
+	/* printf ("notify widget\n"); */
+	g_assert (client == gconf_client);
+	g_assert (key);
+	tkey = strrchr (key, '/');
+	g_assert (tkey);
+
+	if (!strcmp (tkey, "/font_variable_family")) {
+		g_free (prop->font_var_family);
+		prop->font_var_family = gconf_client_get_string (client, key, NULL);
+		set_fonts (html);
+	} else if (!strcmp (tkey, "/font_fixed_family")) {
+		g_free (prop->font_fix_family);
+		prop->font_fix_family = gconf_client_get_string (client, key, NULL);
+		set_fonts (html);
+	} else if (!strcmp (tkey, "/font_variable_size")) {
+		prop->font_var_size = gconf_client_get_int (client, key, NULL);
+		set_fonts (html);
+	} else if (!strcmp (tkey, "/font_fixed_size")) {
+		prop->font_fix_size = gconf_client_get_int (client, key, NULL);
+		set_fonts (html);
+	} else if (!strcmp (tkey, "/spell_error_color_red") {
+		prop->spell_error_color.red = gconf_client_get_int (client, key, NULL);
+	} else if (!strcmp (tkey, "/spell_error_color_green") {
+		prop->spell_error_color.green = gconf_client_get_int (client, key, NULL);
+	} else if (!strcmp (tkey, "/spell_error_color_blue") {
+		prop->spell_error_color.blue = gconf_client_get_int (client, key, NULL);
+		html_colorset_set_color (html->engine->defaultSettings->color_set,
+					 &prop->spell_error_color, HTMLSpellErrorColor);
+		html_colorset_set_color (html->engine->settings->color_set,
+					 &prop->spell_error_color, HTMLSpellErrorColor);
+		if (html_engine_get_editable (html->engine) && !strcmp (tkey, "/spell_error_color_blue"))
+			gtk_widget_queue_draw (GTK_WIDGET (html));
+	} else if (!strcmp (tkey, "/live_spell_check")) {
+		prop->live_spell_check = gconf_client_get_bool (client, key, NULL);
+	} else if (!strcmp (tkey, "/language")) {
+		g_free (prop->language);
+		prop->language = g_strdup (gconf_client_get_string (client, key, NULL);
+	}
+}
+
+static void
+client_notify_class (GConfClient* client,
+		     guint cnxn_id,
+		     const gchar* key,
+		     GConfValue* value,
+		     gboolean is_default,
+		     gpointer user_data)
 {
 	GtkHTMLClass *klass = (GtkHTMLClass *) user_data;
 	GtkHTMLClassProperties *prop = klass->properties;	
@@ -1154,8 +1332,7 @@ client_notify (GConfClient* client,
 		g_free (prop->keybindings_theme);
 		prop->keybindings_theme = gconf_client_get_string (client, key, NULL);
 		load_keybindings (klass);
-	} else
-		g_assert_not_reached ();
+	}
 }
 
 #endif
@@ -1175,25 +1352,43 @@ init_properties (GtkHTMLClass *klass)
 		gtk_html_class_properties_load (klass->properties, gconf_client);
 	} else
 		g_error ("gconf is not initialized, please call gconf_init before using GtkHTML library\n");
+#else
+		gtk_html_class_properties_load (klass->properties);
 #endif
 	load_keybindings (klass);
 #ifdef GTKHTML_HAVE_GCONF
-	gconf_client_notify_add (gconf_client, GTK_HTML_GCONF_DIR, client_notify, klass, NULL, NULL);
+	gconf_client_notify_add (gconf_client, GTK_HTML_GCONF_DIR, client_notify_class, klass, NULL, &gconf_error);
+	if (gconf_error)
+		g_warning ("gconf error: %s\n", gconf_error->str);
 #endif
 }
 
+static gint
+focus (GtkContainer *c, GtkDirectionType direction)
+{
+	gint rv;
+
+	GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (c), GTK_CAN_FOCUS);
+	rv = (*GTK_CONTAINER_CLASS (parent_class)->focus) (c, direction);
+	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (c), GTK_CAN_FOCUS);
+
+	return rv;
+}
+
 static void
 class_init (GtkHTMLClass *klass)
 {
-	GtkHTMLClass   *html_class;
-	GtkWidgetClass *widget_class;
-	GtkObjectClass *object_class;
-	GtkLayoutClass *layout_class;
+	GtkHTMLClass      *html_class;
+	GtkWidgetClass    *widget_class;
+	GtkObjectClass    *object_class;
+	GtkLayoutClass    *layout_class;
+	GtkContainerClass *container_class;
 	
 	html_class = (GtkHTMLClass *)klass;
-	widget_class = (GtkWidgetClass *)klass;
-	object_class = (GtkObjectClass *)klass;
-	layout_class = (GtkLayoutClass *)klass;
+	widget_class = (GtkWidgetClass *) klass;
+	object_class = (GtkObjectClass *) klass;
+	layout_class = (GtkLayoutClass *) klass;
+	container_class = (GtkContainerClass *) klass;
 
 	object_class->destroy = destroy;
 
@@ -1366,7 +1561,7 @@ class_init (GtkHTMLClass *klass)
 				GTK_SIGNAL_OFFSET (GtkHTMLClass, command),
 				gtk_marshal_NONE__ENUM,
 				GTK_TYPE_NONE, 1, GTK_TYPE_HTML_COMMAND);
-#ifdef GTKHTML_HAVE_PSPELL
+
 	signals [SPELL_SUGGESTION_REQUEST] =
 		gtk_signal_new ("spell_suggestion_request",
 				GTK_RUN_FIRST,
@@ -1376,7 +1571,7 @@ class_init (GtkHTMLClass *klass)
 				GTK_TYPE_NONE, 2,
 				GTK_TYPE_POINTER,
 				GTK_TYPE_POINTER);
-#endif
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	object_class->destroy = destroy;
@@ -1393,10 +1588,13 @@ class_init (GtkHTMLClass *klass)
 	widget_class->button_release_event = button_release_event;
 	widget_class->focus_in_event = focus_in_event;
 	widget_class->focus_out_event = focus_out_event;
+	widget_class->enter_notify_event = enter_notify_event;
 	widget_class->selection_get = selection_get;
 	widget_class->selection_received = selection_received;
 	widget_class->selection_clear_event = selection_clear_event;
 
+	container_class->focus = focus;
+
 	layout_class->set_scroll_adjustments = set_adjustments;
 
 	html_class->scroll            = scroll;
@@ -1406,6 +1604,23 @@ class_init (GtkHTMLClass *klass)
 	init_properties (klass);
 }
 
+static void
+init_properties_widget (GtkHTML *html)
+{
+	GtkHTMLClassProperties *prop;
+
+	/* printf ("init_properties_widget\n"); */
+	prop = GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties;
+	set_fonts_idle (html);
+	html_colorset_set_color (html->engine->defaultSettings->color_set, &prop->spell_error_color, HTMLSpellErrorColor);
+
+#ifdef GTKHTML_HAVE_GCONF
+	gconf_client_notify_add (gconf_client, GTK_HTML_GCONF_DIR, client_notify_widget, html, NULL, &gconf_error);
+	if (gconf_error)
+		g_warning ("gconf error: %s\n", gconf_error->str);
+#endif
+}
+
 static void
 init (GtkHTML* html)
 {
@@ -1419,6 +1634,8 @@ init (GtkHTML* html)
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (html), GTK_CAN_FOCUS);
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (html), GTK_APP_PAINTABLE);
 
+	html->editor_bindings = NULL;
+
 	html->debug = FALSE;
 	html->allow_selection = TRUE;
 
@@ -1433,18 +1650,17 @@ init (GtkHTML* html)
 	html->selection_y1 = 0;
 
 	html->in_selection = FALSE;
-	html->button_pressed = FALSE;
+	html->button1_pressed = FALSE;
 
 	html->load_in_progress = TRUE;
 
-	html->idle_handler_id = 0;
-	html->scroll_timeout_id = 0;
-
-	html->paragraph_style = GTK_HTML_PARAGRAPH_STYLE_NORMAL;
-	html->paragraph_alignment = GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT;
-	html->paragraph_indentation = 0;
-
-	html->insertion_font_style = GTK_HTML_FONT_STYLE_DEFAULT;
+	html->priv = g_new0 (GtkHTMLPrivate, 1);
+	html->priv->idle_handler_id = 0;
+	html->priv->scroll_timeout_id = 0;
+	html->priv->paragraph_style = GTK_HTML_PARAGRAPH_STYLE_NORMAL;
+	html->priv->paragraph_alignment = GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT;
+	html->priv->paragraph_indentation = 0;
+	html->priv->insertion_font_style = GTK_HTML_FONT_STYLE_DEFAULT;
 
 	gtk_selection_add_targets (GTK_WIDGET (html),
 				   GDK_SELECTION_PRIMARY,
@@ -1495,9 +1711,10 @@ gtk_html_construct (GtkWidget *htmlw)
 
 	html = GTK_HTML (htmlw);
 
-	html->engine = html_engine_new (htmlw);
+	html->engine        = html_engine_new (htmlw);
+	html->input_line    = gtk_html_input_line_new (html);
 	html->iframe_parent = NULL;
-
+	
 	gtk_signal_connect (GTK_OBJECT (html->engine), "title_changed",
 			    GTK_SIGNAL_FUNC (html_engine_title_changed_cb), html);
 	gtk_signal_connect (GTK_OBJECT (html->engine), "set_base",
@@ -1516,6 +1733,8 @@ gtk_html_construct (GtkWidget *htmlw)
 			    GTK_SIGNAL_FUNC (html_engine_submit_cb), html);
 	gtk_signal_connect (GTK_OBJECT (html->engine), "object_requested",
 			    GTK_SIGNAL_FUNC (html_engine_object_requested_cb), html);
+
+	init_properties_widget (html);
 }
 
 
@@ -1541,11 +1760,20 @@ gtk_html_allow_selection (GtkHTML *html,
 }
 
 
+
+/* #define LOG_INPUT */
+#ifdef LOG_INPUT
+static FILE *log_file;
+#endif
+
 GtkHTMLStream *
 gtk_html_begin (GtkHTML *html)
 {
 	GtkHTMLStream *handle;
 
+#ifdef LOG_INPUT
+	log_file = fopen ("gtkhtml.log", "w+");
+#endif
 	g_return_val_if_fail (! gtk_html_get_editable (html), NULL);
 
 	handle = html_engine_begin (html->engine);
@@ -1565,6 +1793,11 @@ gtk_html_write (GtkHTML *html,
 		const gchar *buffer,
 		size_t size)
 {
+#ifdef LOG_INPUT
+	gint i;
+	for (i=0; i<size; i++)
+		fprintf (log_file, "%c", buffer [i]);
+#endif
 	gtk_html_stream_write (handle, buffer, size);
 }
 
@@ -1573,6 +1806,9 @@ gtk_html_end (GtkHTML *html,
 	      GtkHTMLStream *handle,
 	      GtkHTMLStreamStatus status)
 {
+#ifdef LOG_INPUT
+	fclose (log_file);
+#endif
 	gtk_html_stream_close (handle, status);
 
 	html->load_in_progress = FALSE;
@@ -1677,6 +1913,7 @@ gtk_html_set_editable (GtkHTML *html,
 	g_return_if_fail (GTK_IS_HTML (html));
 
 	html_engine_set_editable (html->engine, editable);
+	set_editor_keybindings (html, editable);
 
 	if (editable)
 		update_styles (html);
@@ -1713,6 +1950,18 @@ gtk_html_print (GtkHTML *html,
 	html_engine_print (html->engine, print_context);
 }
 
+void
+gtk_html_print_with_header_footer (GtkHTML *html, GnomePrintContext *print_context,
+				   gdouble header_height, gdouble footer_height,
+				   GtkHTMLPrintCallback header_print, GtkHTMLPrintCallback footer_print)
+{
+	g_return_if_fail (html != NULL);
+	g_return_if_fail (GTK_IS_HTML (html));
+
+	html_engine_print_with_header_footer (html->engine, print_context,
+					      header_height, footer_height, header_print, footer_print);
+}
+
 
 /* Editing.  */
 
@@ -1738,10 +1987,11 @@ gtk_html_set_paragraph_style (GtkHTML *html,
 					      HTML_ENGINE_SET_CLUEFLOW_STYLE, TRUE))
 		return;
 
-	html->paragraph_style = style;
+	html->priv->paragraph_style = style;
 
 	gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_STYLE_CHANGED],
 			 style);
+	queue_draw (html);
 }
 
 GtkHTMLParagraphStyle
@@ -1868,6 +2118,19 @@ gtk_html_set_default_background_color (GtkHTML *html, GdkColor *c)
 	html_colorset_set_color (html->engine->defaultSettings->color_set, c, HTMLBgColor);
 }
 
+gpointer
+gtk_html_get_image_by_imageid (GtkHTML *html, gchar *id)
+{
+	g_return_val_if_fail (html, NULL);
+	g_return_val_if_fail (id, NULL);
+	g_return_val_if_fail (GTK_IS_HTML (html), NULL);
+	g_return_val_if_fail (html->engine, NULL);
+
+	if (!html->engine->imageid_table) return NULL;
+
+	return g_hash_table_lookup (html->engine->imageid_table, id);
+}
+
 /*******************************************
 
    keybindings
@@ -1881,6 +2144,7 @@ scroll (GtkHTML *html,
 	gfloat         position)
 {
 	GtkAdjustment *adj;
+	gint line_height;
 	gfloat delta;
 
 	/* we dont want scroll in editable (move cursor instead) */
@@ -1891,6 +2155,10 @@ scroll (GtkHTML *html,
 		? gtk_layout_get_vadjustment (GTK_LAYOUT (html)) : gtk_layout_get_hadjustment (GTK_LAYOUT (html));
 
 
+	line_height = (html->engine && adj->page_increment > html->engine->painter->font_manager.var_size)
+		? html->engine->painter->font_manager.var_size
+		: 0;
+
 	switch (scroll_type) {
 	case GTK_SCROLL_STEP_FORWARD:
 		delta = adj->step_increment;
@@ -1899,10 +2167,10 @@ scroll (GtkHTML *html,
 		delta = -adj->step_increment;
 		break;
 	case GTK_SCROLL_PAGE_FORWARD:
-		delta = adj->page_increment;
+		delta = adj->page_increment - line_height;
 		break;
 	case GTK_SCROLL_PAGE_BACKWARD:
-		delta = -adj->page_increment;
+		delta = -adj->page_increment + line_height;
 		break;
 	default:
 		delta = 0.0;
@@ -2067,6 +2335,8 @@ move_selection (GtkHTML *html, GtkHTMLCommandType com_type)
 	default:
 		g_assert_not_reached ();
 	}
+
+	html->binding_handled = TRUE;
 }
 
 static void
@@ -2074,11 +2344,33 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 {
 	HTMLEngine *e = html->engine;
 
+	/* printf ("command %d\n", com_type); */
+	html->binding_handled = TRUE;
+
+	/* non-editable + editable commands */
+	switch (com_type) {
+	case GTK_HTML_COMMAND_SEARCH_INCREMENTAL_FORWARD:
+		gtk_html_isearch (html, TRUE);
+		break;
+	case GTK_HTML_COMMAND_SEARCH_INCREMENTAL_BACKWARD:
+		gtk_html_isearch (html, FALSE);
+		break;
+	case GTK_HTML_COMMAND_FOCUS_FORWARD:
+		html->binding_handled = gtk_container_focus (GTK_CONTAINER (html), GTK_DIR_TAB_FORWARD);
+		break;
+	case GTK_HTML_COMMAND_FOCUS_BACKWARD:
+		html->binding_handled = gtk_container_focus (GTK_CONTAINER (html), GTK_DIR_TAB_BACKWARD);
+		break;
+	default:
+		html->binding_handled = FALSE;
+	}
+
 	if (!html_engine_get_editable (e))
 		return;
 
-	/* printf ("command %d\n", com_type); */
+	html->binding_handled = TRUE;
 
+	/* editable commands only */
 	switch (com_type) {
 	case GTK_HTML_COMMAND_UNDO:
 		html_engine_undo (e);
@@ -2257,10 +2549,8 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	}
 #endif
 	default:
-		return;
+		html->binding_handled = FALSE;
 	}
-
-	html->binding_handled = TRUE;
 }
 
 /*
@@ -2312,12 +2602,38 @@ clean_bindings_set (GtkBindingSet *binding_set)
 	g_list_free (vals);
 }
 
+static void
+set_editor_keybindings (GtkHTML *html, gboolean editable)
+{
+	if (editable) {
+		gchar *name;
+
+		name = g_strconcat ("gtkhtml-bindings-",
+				    GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties->keybindings_theme, NULL);
+		html->editor_bindings = gtk_binding_set_find (name);
+		if (!html->editor_bindings)
+			g_warning ("cannot find %s bindings", name);
+		g_free (name);
+	} else
+		html->editor_bindings = NULL;
+}
+
+static void
+load_bindings_from_file (gboolean from_share, gchar *name)
+{
+	gchar *rcfile;
+
+	rcfile = g_strconcat ((from_share ? PREFIX "/share/gtkhtml/" : gnome_util_user_home ()),
+			      (from_share ? "" : "/.gnome/"), name, NULL);
+	if (g_file_exists (rcfile))
+		gtk_rc_parse (rcfile);
+	g_free (rcfile);
+}
+
 static void
 load_keybindings (GtkHTMLClass *klass)
 {
 	GtkBindingSet *binding_set;
-	gchar *base = NULL;
-	gchar *rcfile, *name;
 
 	/* FIXME add to gtk gtk_binding_set_clear & gtk_binding_set_remove_path */
 	clean_bindings_set (gtk_binding_set_by_class (klass));
@@ -2329,32 +2645,9 @@ load_keybindings (GtkHTMLClass *klass)
 	gtk_html_cursor_skip_get_type ();
 	gtk_html_command_get_type ();
 
-	if (strcmp (klass->properties->keybindings_theme, "custom")) {
-		base = g_strconcat ("keybindingsrc.", klass->properties->keybindings_theme, NULL);
-		rcfile = g_concat_dir_and_file (PREFIX "/share/gtkhtml", base);
-		g_free (base);
-	} else {
-		rcfile = g_strconcat (gnome_util_user_home (), "/.gnome/gtkhtml-bindings-custom", NULL);
-	}
-
-	if (g_file_exists (rcfile)) {
-
-		/* g_warning ("Loading keybindings -- %s", rcfile); */
-		gtk_rc_parse (rcfile);
-
-		name = g_strconcat ("gtkhtml-bindings-", klass->properties->keybindings_theme, NULL);
-		binding_set = gtk_binding_set_find (name);
-		/* g_warning ("Looking for %s set", name); */
-		g_assert (binding_set);
-		gtk_binding_set_add_path (binding_set,
-					  GTK_PATH_CLASS,
-					  gtk_type_name (GTK_OBJECT_CLASS (klass)->type),
-					  GTK_PATH_PRIO_GTK);
-		g_free (name);
-	}
-	/* else
-		g_warning (_("Couldn't find keybinding file -- %s"), rcfile); */
-	g_free (rcfile);
+	load_bindings_from_file (TRUE,  "keybindingsrc.emacs");
+	load_bindings_from_file (TRUE,  "keybindingsrc.ms");
+	load_bindings_from_file (FALSE, "gtkhtml-bindings-custom");
 
 	binding_set = gtk_binding_set_by_class (klass);
 
@@ -2363,7 +2656,7 @@ load_keybindings (GtkHTMLClass *klass)
 	gtk_binding_entry_add_signal (binding_set, GDK_ ## key, m, \
 				      "scroll", 3, \
 				      GTK_TYPE_ORIENTATION, GTK_ORIENTATION_ ## orient, \
-				      GTK_TYPE_ENUM, GTK_SCROLL_ ## sc, \
+				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_ ## sc, \
 				      GTK_TYPE_FLOAT, 0.0); \
 
 	BSCROLL (0, Up, VERTICAL, STEP_BACKWARD);
@@ -2405,10 +2698,13 @@ load_keybindings (GtkHTMLClass *klass)
 	BMOVE (0, Down,     DOWN,  ONE);
 	BMOVE (0, KP_Down,  DOWN,  ONE);
 
-	BMOVE (GDK_CONTROL_MASK, Left,  LEFT,  WORD);
 	BMOVE (GDK_CONTROL_MASK, KP_Left,  LEFT,  WORD);
-	BMOVE (GDK_CONTROL_MASK, h, RIGHT, WORD);
+	BMOVE (GDK_CONTROL_MASK, Left,     LEFT,  WORD);
+	BMOVE (GDK_MOD1_MASK,    Left,     LEFT,  WORD);
+
 	BMOVE (GDK_CONTROL_MASK, KP_Right, RIGHT, WORD);
+	BMOVE (GDK_CONTROL_MASK, Right,    RIGHT, WORD);
+	BMOVE (GDK_MOD1_MASK,    Right,    RIGHT, WORD);
 
 	BMOVE (0, Page_Up,       UP,   PAGE);
 	BMOVE (0, KP_Page_Up,    UP,   PAGE);
@@ -2428,6 +2724,11 @@ load_keybindings (GtkHTMLClass *klass)
 	BCOM (GDK_SHIFT_MASK, BackSpace, DELETE_BACK);
 	BCOM (0, Delete, DELETE);
 	BCOM (0, KP_Delete, DELETE);
+
+	BCOM (0, Tab, FOCUS_FORWARD);
+	BCOM (0, ISO_Left_Tab, FOCUS_FORWARD);
+	BCOM (GDK_SHIFT_MASK, Tab, FOCUS_BACKWARD);
+	BCOM (GDK_SHIFT_MASK, ISO_Left_Tab, FOCUS_BACKWARD);
 }
 
 void
@@ -2437,3 +2738,33 @@ gtk_html_set_iframe_parent (GtkHTML *html, GtkWidget *parent)
 
 	html->iframe_parent = parent;
 }
+
+void
+gtk_html_select_word (GtkHTML *html)
+{
+	HTMLEngine *e;
+
+	if (!html->allow_selection)
+		return;
+
+	e = html->engine;
+	if (html_engine_get_editable (e))
+		html_engine_select_word_editable (e);
+	else
+		html_engine_select_word (e);
+}
+
+void
+gtk_html_select_line (GtkHTML *html)
+{
+	HTMLEngine *e;
+
+	if (!html->allow_selection)
+		return;
+
+	e = html->engine;
+	if (html_engine_get_editable (e))
+		html_engine_select_line_editable (e);
+	else
+		html_engine_select_line (e);
+}
