@@ -23,8 +23,10 @@
 #define _GTKHTML_H_
 
 
-typedef struct _GtkHTML	GtkHTML;
-typedef struct _GtkHTMLClass	GtkHTMLClass;
+typedef struct _GtkHTML	            GtkHTML;
+typedef struct _GtkHTMLPrivate      GtkHTMLPrivate;
+typedef struct _GtkHTMLClass	    GtkHTMLClass;
+typedef struct _GtkHTMLInputLine    GtkHTMLInputLine;
 
 #include <sys/types.h>
 #include <gtk/gtk.h>
@@ -60,7 +62,7 @@ enum _GtkHTMLParagraphStyle {
 	GTK_HTML_PARAGRAPH_STYLE_PRE,
 	GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED,
 	GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN,
-	GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT,
+	GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT
 };
 typedef enum _GtkHTMLParagraphStyle GtkHTMLParagraphStyle;
 
@@ -75,7 +77,7 @@ enum _GtkHTMLCursorSkipType {
 	GTK_HTML_CURSOR_SKIP_ONE,
 	GTK_HTML_CURSOR_SKIP_WORD,
 	GTK_HTML_CURSOR_SKIP_PAGE,
-	GTK_HTML_CURSOR_SKIP_ALL,
+	GTK_HTML_CURSOR_SKIP_ALL
 };
 typedef enum _GtkHTMLCursorSkipType GtkHTMLCursorSkipType;
 
@@ -152,11 +154,18 @@ enum _GtkHTMLCommandType {
 	GTK_HTML_COMMAND_CAPITALIZE_WORD,
 	GTK_HTML_COMMAND_UPCASE_WORD,
 	GTK_HTML_COMMAND_DOWNCASE_WORD,
-#ifdef GTKHTML_HAVE_PSPELL
+
 	GTK_HTML_COMMAND_SPELL_SUGGEST,
 	GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD,
 	GTK_HTML_COMMAND_SPELL_SESSION_DICTIONARY_ADD,
-#endif
+
+	GTK_HTML_COMMAND_SEARCH,
+	GTK_HTML_COMMAND_SEARCH_INCREMENTAL_FORWARD,
+	GTK_HTML_COMMAND_SEARCH_INCREMENTAL_BACKWARD,
+	GTK_HTML_COMMAND_SEARCH_REGEX,
+
+	GTK_HTML_COMMAND_FOCUS_FORWARD,
+	GTK_HTML_COMMAND_FOCUS_BACKWARD
 };
 typedef enum _GtkHTMLCommandType GtkHTMLCommandType;
 
@@ -174,8 +183,10 @@ typedef HTMLEngineSaveReceiverFn GtkHTMLSaveReceiverFn;
 struct _GtkHTML {
 	GtkLayout layout;
 
-	GtkWidget  *iframe_parent;
-	HTMLEngine *engine;
+	GtkBindingSet      *editor_bindings;
+	GtkWidget          *iframe_parent;
+	GtkHTMLInputLine   *input_line;
+	HTMLEngine         *engine;
 
 	/* The URL of the link over which the pointer currently is.  NULL if
            the pointer is not over a link.  */
@@ -189,7 +200,7 @@ struct _GtkHTML {
 	gint selection_x1, selection_y1;
 
 	guint in_selection : 1;
-	guint button_pressed : 1;
+	guint button1_pressed : 1;
 	guint load_in_progress : 1;
 
 	guint debug : 1;
@@ -198,16 +209,8 @@ struct _GtkHTML {
 	guint hadj_connection;
 	guint vadj_connection;
 
-	guint idle_handler_id;
-	guint scroll_timeout_id;
-
-	GtkHTMLParagraphStyle paragraph_style;
-	guint paragraph_indentation;
-	GtkHTMLParagraphAlignment paragraph_alignment;
-
-	GtkHTMLFontStyle insertion_font_style;
-
 	gboolean binding_handled;
+	GtkHTMLPrivate *priv;
 };
 
 /* must be forward referenced *sigh* */
@@ -240,10 +243,7 @@ struct _GtkHTMLClass {
 	void (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
 	void (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
 	void (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
-
-#ifdef GTKHTML_HAVE_PSPELL
-	void (* spell_suggestion_request) (GtkHTML *html, PspellManager *spell_checker, gchar *word);
-#endif
+	void (* spell_suggestion_request) (GtkHTML *html, gpointer *spell_checker, gchar *word);
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
@@ -267,6 +267,8 @@ void  gtk_html_enable_debug  (GtkHTML  *html,
 /* Behavior.  */
 void  gtk_html_allow_selection  (GtkHTML  *html,
 				 gboolean  allow);
+void  gtk_html_select_word      (GtkHTML  *html);
+void  gtk_html_select_line      (GtkHTML  *html);
 int   gtk_html_request_paste    (GtkWidget *widget,
 				 gint32 time);
 /* Loading.  */
@@ -301,8 +303,18 @@ void      gtk_html_set_editable  (GtkHTML       *html,
 gboolean  gtk_html_get_editable  (const GtkHTML *html);
 
 /* Printing support.  */
-void  gtk_html_print  (GtkHTML           *html,
-		       GnomePrintContext *print_context);
+
+typedef void (*GtkHTMLPrintCallback) (GtkHTML *html, GnomePrintContext *print_context,
+				      gdouble x, gdouble y, gdouble width, gdouble height);
+
+void  gtk_html_print_with_header_footer  (GtkHTML              *html,
+					  GnomePrintContext    *print_context,
+					  gdouble               header_height,
+					  gdouble               footer_height,
+					  GtkHTMLPrintCallback  header_print,
+					  GtkHTMLPrintCallback  footer_print);
+void  gtk_html_print                     (GtkHTML              *html,
+					  GnomePrintContext    *print_context);
 
 /* Title.  */
 const gchar *gtk_html_get_title  (GtkHTML *html);
@@ -338,5 +350,6 @@ void  gtk_html_redo  (GtkHTML *html);
 /* misc utils */
 
 void  gtk_html_set_default_background_color (GtkHTML *html, GdkColor *c);
+gpointer gtk_html_get_image_by_imageid (GtkHTML *html, gchar *id);
 
 #endif /* _GTKHTML_H_ */
