@@ -249,7 +249,7 @@ check_point (HTMLObject *o,
 
 	if (x < o->x || x > o->x + o->width
 	    || y > o->y + o->descent || y < o->y - o->ascent)
-		return 0L;
+		return NULL;
 
 	x = x - o->x;
 	y = y - o->y + o->ascent;
@@ -265,6 +265,35 @@ check_point (HTMLObject *o,
 	return NULL;
 }
 
+static gint
+check_page_split (HTMLObject *self,
+		  gint y)
+{
+	HTMLClue *clue;
+	HTMLObject *p;
+	gint last_under = 0;
+
+	clue = HTML_CLUE (self);
+
+	y -= (self->y - self->ascent);
+
+	for (p = clue->head; p != NULL; p = p->next) {
+		gint y1, y2;
+
+		y1 = p->y - p->ascent;
+		y2 = p->y + p->descent;
+
+		if (y1 > y)
+			return last_under;
+
+		if (y >= y1 && y < y2)
+			return html_object_check_page_split (p, y - y1) + y1;
+		last_under = y2;
+	}
+
+	return y;
+}
+
 static void
 forall (HTMLObject *self,
 	HTMLObjectForallFunc func,
@@ -303,7 +332,8 @@ save (HTMLObject *self,
 
 static gboolean
 save_plain (HTMLObject *self,
-      HTMLEngineSaveState *state)
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
 {
 	HTMLObject *p;
 	HTMLClue *clue;
@@ -311,7 +341,7 @@ save_plain (HTMLObject *self,
 	clue = HTML_CLUE (self);
 
 	for (p = clue->head; p != NULL; p = p->next) {
-		if (! html_object_save_plain (p, state))
+		if (! html_object_save_plain (p, state, requested_width))
 		    return FALSE;
 	}
 
@@ -369,26 +399,71 @@ search (HTMLObject *obj, HTMLSearch *info)
 		cur  = html_search_pop (info);
 		cur = (info->forward) ? cur->next : cur->prev;
 		next = TRUE;
-	} else {
+	} else
 		cur = (info->forward) ? clue->head : clue->tail;
-	}
 
 	while (cur) {
 		html_search_push (info, cur);
-		if (html_object_search (cur, info)) {
+		if (html_object_search (cur, info))
 			return TRUE;
-		}
+
 		html_search_pop (info);
 		cur = (info->forward) ? cur->next : cur->prev;
 	}
 
-	if (next) {
+	if (next)
 		return html_search_next_parent (info);
-	}
 
 	return FALSE;
 }
 
+static void
+append_selection_string (HTMLObject *self,
+			 GString *buffer)
+{
+	HTMLObject *o = HTML_CLUE (self)->head;
+
+	while (o) {
+		html_object_append_selection_string (o, buffer);
+		o = o->next;
+	}
+}
+
+static HTMLObject *
+next (HTMLObject *self, HTMLObject *child)
+{
+	g_assert (self);
+	g_assert (child);
+	g_assert (child->parent == self);
+
+	return html_object_next_not_slave (child);
+}
+
+static HTMLObject *
+prev (HTMLObject *self, HTMLObject *child)
+{
+	g_assert (self);
+	g_assert (child);
+	g_assert (child->parent == self);
+
+	return html_object_prev_not_slave (child);
+}
+
+static HTMLObject *
+head (HTMLObject *self)
+{
+	return HTML_CLUE (self)->head;
+}
+
+static HTMLObject *
+tail (HTMLObject *self)
+{
+	HTMLObject *obj;
+
+	obj = HTML_CLUE (self)->tail;
+	return (obj && HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) ? html_object_prev_not_slave (obj) : obj;
+}
+
 
 void
 html_clue_type_init (void)
@@ -419,12 +494,18 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->calc_min_width = calc_min_width;
 	object_class->check_point = check_point;
+	object_class->check_page_split = check_page_split;
 	object_class->find_anchor = find_anchor;
 	object_class->forall = forall;
 	object_class->is_container = is_container;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
 	object_class->search = search;
+	object_class->append_selection_string = append_selection_string;
+	object_class->next = next;
+	object_class->prev = prev;
+	object_class->head = head;
+	object_class->tail = tail;
 
 	/* HTMLClue methods.  */
 	klass->get_left_clear = get_left_clear;
