@@ -37,6 +37,7 @@
 #include "htmltextslave.h"	/* FIXME */
 #include "htmlsearch.h"
 #include "htmlentity.h"
+#include "htmlengine-edit.h"
 
 
 HTMLClueFlowClass html_clueflow_class;
@@ -61,8 +62,8 @@ calc_padding (HTMLPainter *painter)
 	guint ascent, descent;
 
 	/* FIXME maybe this should depend on the style.  */
-	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
-	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
+	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
 
 	return ascent + descent;
 }
@@ -72,8 +73,8 @@ calc_indent_unit (HTMLPainter *painter)
 {
 	guint ascent, descent;
 
-	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
-	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
+	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
 
 	return (ascent + descent) * 3;
 }
@@ -83,8 +84,8 @@ calc_bullet_size (HTMLPainter *painter)
 {
 	guint ascent, descent;
 
-	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
-	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3);
+	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
 
 	return (ascent + descent) / 3;
 }
@@ -269,8 +270,6 @@ calc_min_width (HTMLObject *o,
 		cur = cur->next;
 	}
 
-	/* printf ("flow min width: %d\n", min_width + get_indent (HTML_CLUEFLOW (o), painter)); */
-
 	return min_width + get_indent (HTML_CLUEFLOW (o), painter);
 }
 
@@ -668,7 +667,7 @@ calc_size (HTMLObject *o,
 	if (o->width < o->max_width)
 		o->width = o->max_width;
 
-#if 1
+#if 0
 	if (o->width > rmargin - o->x)
 		o->width = rmargin - o->x;
 #endif
@@ -691,10 +690,11 @@ calc_preferred_width (HTMLObject *o,
 		if (!(obj->flags & HTML_OBJECT_FLAG_NEWLINE)) {
 			w += html_object_calc_preferred_width (obj, painter);
 		}
-		/* remove trailing space width on the end of line */
-		if (!obj->next || obj->flags & HTML_OBJECT_FLAG_NEWLINE) {
-			HTMLObject *eol = (obj->flags & HTML_OBJECT_FLAG_NEWLINE) ? obj->prev : obj;
 
+		if (obj->flags & HTML_OBJECT_FLAG_NEWLINE || !html_object_next_not_slave (obj)) {
+			HTMLObject *eol = (obj->flags & HTML_OBJECT_FLAG_NEWLINE) ? html_object_prev_not_slave (obj) : obj;
+
+			/* remove trailing space width on the end of line */
 			if (HTML_OBJECT_TYPE (eol) == HTML_TYPE_TEXTMASTER
 			    || HTML_OBJECT_TYPE (eol) == HTML_TYPE_LINKTEXTMASTER) {
 				w -= html_text_master_trail_space_width (HTML_TEXT_MASTER (eol), painter);
@@ -847,10 +847,9 @@ static void
 append_selection_string (HTMLObject *self,
 			 GString *buffer)
 {
-	if (! self->selected)
-		return;
-
-	g_string_append_c (buffer, '\n');
+        (*HTML_OBJECT_CLASS (parent_class)->append_selection_string) (self, buffer);
+	if (self->selected)
+		g_string_append_c (buffer, '\n');
 }
 
 
@@ -1100,69 +1099,135 @@ save (HTMLObject *self,
 	return write_post_tags (HTML_CLUEFLOW (self), state);
 }
 
-static gboolean
-save_plain (HTMLObject *self,
-	    HTMLEngineSaveState *state)
+static gint
+string_append_len (GString *out, guchar *s, gint length)
 {
-	HTMLClueFlow *clueflow;
+	gint len = length;
+	guchar c;
 
-	clueflow = HTML_CLUEFLOW (self);
+	while (len--) {
+		c = (*s == ENTITY_NBSP) ? ' ' : *s;
+		g_string_append_c (out, c);
+		s++;
+	}
+	return length;
+}
 
-	if (get_pre_padding (clueflow, 1) > 0)
-		if (! html_engine_save_output_string (state, "\n"))
-			return FALSE;
+#define CLUEFLOW_ITEM_MARKER "* "
+#define CLUEFLOW_ITEM_PAD    "  "
+#define CLUEFLOW_INDENT      "    "
 
-	if (clueflow->level > 0) {
-		gint i;
+static gint 
+plain_padding (HTMLClueFlow *flow, GString *out, gboolean firstline)
+{
+	gint pad;
+	gint i;
+	       
+	pad = flow->level * strlen (CLUEFLOW_INDENT) 
+		+ (is_item (flow) ? strlen (CLUEFLOW_ITEM_PAD) : 0); 
 
-		for (i = 0; i < (gint) clueflow->level; i++) {
-			if (! html_engine_save_output_string (state, "\t"))
-				return FALSE;
+	if (out) {
+		for (i = 0; i < (gint) flow->level; i++) {
+			g_string_append (out, CLUEFLOW_INDENT);
+		}
+
+		if (is_item (flow)) {
+			if (firstline) {
+				g_string_append (out, CLUEFLOW_ITEM_MARKER);
+			} else {
+				g_string_append (out, CLUEFLOW_ITEM_PAD);
+			}
 		}
 	}
 
-	/* Paragraph's content.  */
-	if (! HTML_OBJECT_CLASS (&html_clue_class)->save_plain (self, state))
-		return FALSE;
-
-	if (!html_engine_save_output_string (state, "\n"))
-		return FALSE;
-	
-	if (get_post_padding (clueflow, 1) > 0)
-		if (! html_engine_save_output_string (state, "\n"))
-			return FALSE;
-
-	return TRUE;
+	return pad;
 }
 
-
-static gint
-check_page_split (HTMLObject *self,
-		  gint y)
+static gboolean
+save_plain (HTMLObject *self,
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
 {
-	HTMLClue *clue;
-	HTMLObject *p;
-	gint last_base;
+	HTMLClueFlow *flow;
+	HTMLEngineSaveState *buffer_state;
+	GString *out = g_string_new ("");
+	size_t len;
+	gint pad;
+	gboolean firstline = TRUE;
 
-	clue = HTML_CLUE (self);
+	flow = HTML_CLUEFLOW (self);
 
-	last_base = 0;
-	for (p = clue->head; p != NULL; p = p->next) {
-		gint base;
+	pad = plain_padding (flow, NULL, FALSE);
+	buffer_state = html_engine_save_buffer_new (state->engine);
 
-		base = p->y + p->descent;
+	/* buffer the paragraph's content into the save buffer */
+	if (HTML_OBJECT_CLASS (&html_clue_class)->save_plain (self, 
+							      buffer_state, 
+							      requested_width - pad)) {
+		guchar *s, *space;
+		
+		if (get_pre_padding (flow, 1) > 0)
+			g_string_append (out, "\n");
 
-		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTMASTER
-		    || HTML_OBJECT_TYPE (p) == HTML_TYPE_LINKTEXTMASTER)
-			continue;
+		s = html_engine_save_buffer_peek_text (buffer_state);
 
-		if (base > y)
-			return last_base;
+		if (*s == 0)
+			g_string_append (out, "\n");
+		else while (*s) {
+			/* FIXME we should allow wrapping on PRE sections as well */
+			len = strcspn (s, "\n");
+			
+			switch (flow->style) {
+			case HTML_CLUEFLOW_STYLE_PRE:
+			case HTML_CLUEFLOW_STYLE_NOWRAP:
 
-		last_base = base;
+				plain_padding (flow, out, firstline);
+				s += string_append_len (out, s, len);
+
+				break;
+			default:
+
+				if (len > (requested_width - pad)) {
+					space = s + (requested_width - pad);
+					while (space > s && (*space != ' '
+							     || (*(space + 1) == ENTITY_NBSP)
+							     || (*(space - 1) == ENTITY_NBSP)))
+						/* space = unicode_previous_utf8 (s, space); */
+						space = space - 1;
+					
+					if (space != s)
+						len = space - s;
+				}
+
+				plain_padding (flow, out, firstline);
+				s += string_append_len (out, s, len);
+
+				while (*s == ' ' || *s == ENTITY_NBSP) 
+					s++;
+				
+				break;
+			}							
+
+			if (*s == '\n') 
+				s++;
+			
+			g_string_append_c (out, '\n');
+			firstline = FALSE;
+		}
+		
+		if (get_post_padding (flow, 1) > 0)
+			g_string_append (out, "\n");
+				
 	}
-
-	return y;
+	html_engine_save_buffer_free (buffer_state);
+		
+	if (!html_engine_save_output_string (state, out->str)) {
+		g_string_free (out, TRUE);
+		return FALSE;
+	}		
+	
+	g_string_free (out, TRUE);
+	return TRUE;
 }
 
 
@@ -1432,6 +1497,22 @@ search_next (HTMLObject *obj, HTMLSearch *info)
 	return FALSE;
 }
 
+static gboolean
+relayout (HTMLObject *self,
+	  HTMLEngine *engine,
+	  HTMLObject *child)
+{
+	gint mw;
+
+	mw = html_object_calc_min_width (self, engine->painter);
+	if (mw <= self->max_width)
+		return (*HTML_OBJECT_CLASS (parent_class)->relayout) (self, engine, child);
+	html_engine_calc_size (engine);
+	html_engine_draw (engine, 0, 0, engine->width, engine->height);
+
+	return TRUE;
+}
+
 
 void
 html_clueflow_type_init (void)
@@ -1463,11 +1544,12 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->draw_background = draw_background;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
-	object_class->check_page_split = check_page_split;
+	// object_class->check_page_split = check_page_split;
 	object_class->check_point = check_point;
 	object_class->append_selection_string = append_selection_string;
 	object_class->search = search;
 	object_class->search_next = search_next;
+	object_class->relayout = relayout;
 
 	klass->get_default_font_style = get_default_font_style;
 
@@ -1614,9 +1696,11 @@ html_clueflow_set_style (HTMLClueFlow *flow,
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
 
+	html_object_change_set (HTML_OBJECT (flow), HTML_CHANGE_ALL);
 	flow->style = style;
 
-	relayout_with_siblings (flow, engine);
+	html_engine_schedule_update (engine);
+	/* FIXME - make it more effective: relayout_with_siblings (flow, engine); */
 }
 
 HTMLClueFlowStyle
@@ -1760,33 +1844,51 @@ html_clueflow_remove_text_slaves (HTMLClueFlow *flow)
 
 /* spell checking */
 
-static gchar *
-get_text (HTMLClue *clue)
+#include "htmlinterval.h"
+
+static guint
+get_text_bytes (HTMLClue *clue, HTMLInterval *i)
 {
 	HTMLObject *obj;
-	guint len = 0;
-	gchar *text, *ct;
+	guint bytes;
 
-	obj = clue->head;
-	while (obj) {
-		if (html_object_is_text (obj))
-			len += HTML_TEXT (obj)->text_len;
-		else if (HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE)
-			len++;
+	g_assert (i);
+	g_assert (i->from);
+	g_assert (i->to);
+
+	bytes = 0;
+	obj = i->from;
+	while (1) {
+		bytes += html_interval_get_bytes (i, obj);
+		if (obj == i->to) break;
 		obj = obj->next;
+		g_assert (obj);
 	}
 
-	ct  = text = g_malloc (len+1);
-	text [len] = 0;
-	obj = clue->head;
+	return bytes;
+}
+
+static gchar *
+get_text (HTMLClue *clue, HTMLInterval *i)
+{
+	HTMLObject *obj;
+	guint cb, bytes = 0;
+	gchar *text, *ct;
+
+	bytes      = get_text_bytes (clue, i);
+	ct         = text = g_malloc (bytes + 1);
+	text [bytes] = 0;
+
+	obj = i->from;
 	while (obj) {
-		if (html_object_is_text (obj)) {
-			strcpy (ct, HTML_TEXT (obj)->text);
-			ct += HTML_TEXT (obj)->text_len;
-		} else if (HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE) {
-			*ct = ' ';
-			ct++;
-		}
+		cb = html_interval_get_bytes (i, obj);
+		if (html_object_is_text (obj))
+			strncpy (ct, HTML_TEXT (obj)->text + html_interval_get_start_index (i, obj), cb);
+		else
+			if (cb == 1) *ct = ' ';
+			else memset (ct, ' ', cb);
+		ct += cb;
+		if (obj == i->to) break;
 		obj = obj->next;
 	}
 
@@ -1796,48 +1898,54 @@ get_text (HTMLClue *clue)
 }
 
 static HTMLObject *
-next_obj (HTMLObject *obj, guint *off, gboolean *is_text)
+next_obj_and_clear (HTMLObject *obj, guint *off, gboolean *is_text, HTMLInterval *i)
 {
-	*off += (*is_text)
-		? HTML_TEXT (obj)->text_len
-		: ((HTML_OBJECT_TYPE (obj) != HTML_TYPE_TEXTSLAVE) ? 1 : 0);
+	*off += html_object_get_length (obj) - html_interval_get_start (i, obj);
 	obj = obj->next;
 	if (obj && (*is_text = html_object_is_text (obj)))
-		html_text_spell_errors_clear (HTML_TEXT (obj));
+		html_text_spell_errors_clear_interval (HTML_TEXT (obj), i);
 
 	return obj;
 }
 
 static HTMLObject *
-spell_check_word (HTMLObject *obj, const gchar *text, const gchar *word, guint *off)
+spell_check_word_mark (HTMLObject *obj, const gchar *text, const gchar *word, guint *off, HTMLInterval *i)
 {
-	guint w_off;
-	guint len = strlen (word);
+	guint w_off, w_index, ioff;
+	guint len = unicode_strlen (word, -1);
 	gboolean is_text;
 
-	/* printf ("[not in dictionary off: %d]\n", word - text); */
+	/* printf ("[not in dictionary word off: %d off: %d]\n", word - text, *off); */
 	is_text = html_object_is_text (obj);
-	w_off   = word - text;
-	while (obj && (!is_text || (is_text && *off + HTML_TEXT (obj)->text_len <= w_off)))
-		obj = next_obj (obj, off, &is_text);
+	w_index = word - text;
+	w_off   = unicode_index_to_offset (text, w_index);
+	while (obj && (!is_text || (is_text && *off + html_interval_get_length (i, obj) <= w_off)))
+		obj = next_obj_and_clear (obj, off, &is_text, i);
 
-	/* printf ("is_text: %d len: %d obj: %p\n", is_text, len, obj); */
+	/* printf ("is_text: %d len: %d obj: %p off: %d\n", is_text, len, obj, *off); */
 	if (obj && is_text) {
+		gchar *t;
 		guint tlen;
 		guint toff;
 
 		while (len) {
 			toff  = w_off - *off;
-			tlen  = MIN (HTML_TEXT (obj)->text_len - toff, len);
-			g_assert (!strncmp (text + w_off, HTML_TEXT (obj)->text + toff, tlen));
-			/* printf ("add spell error - word: %s beg: %s len: %d\n",
-			   word, HTML_TEXT (obj)->text + toff, tlen); */
+			ioff  = html_interval_get_start (i, obj);
+			tlen  = MIN (HTML_TEXT (obj)->text_len - toff - ioff, len);
+			t     = HTML_TEXT (obj)->text;
+			g_assert (!strncmp (text + w_index, t + unicode_offset_to_index (t, toff + ioff),
+					    unicode_offset_to_index (t, toff + ioff + tlen)
+					    - unicode_offset_to_index (t, toff + ioff)));
+			/* printf ("add spell error - word: %s off: %d beg: %s len: %d\n",
+			   word, *off, HTML_TEXT (obj)->text + toff, tlen); */
 			html_text_spell_errors_add (HTML_TEXT (obj),
-						    toff, tlen);
-			len   -= tlen;
-			w_off += tlen;
+						    ioff + toff, tlen);
+			len     -= tlen;
+			w_off   += tlen;
+			w_index += unicode_offset_to_index (text + w_index, tlen);
 			if (len)
-				do obj = next_obj (obj, off, &is_text); while (obj && !is_text);
+				do obj = next_obj_and_clear (obj, off, &is_text, i); while (obj && !is_text);
+			/* printf ("off: %d\n", *off); */
 			g_assert (!len || obj);
 		}
 	}
@@ -1845,32 +1953,61 @@ spell_check_word (HTMLObject *obj, const gchar *text, const gchar *word, guint *
 	return obj;
 }
 
+static gchar *
+begin_of_word (gchar *text, gchar *ct)
+{
+	unicode_char_t uc;
+
+	do
+		unicode_get_utf8 (ct, &uc);
+	while (!html_is_in_word (uc) && (ct = unicode_next_utf8 (ct)) && *ct);
+
+	return ct;
+}
+
+static gchar *
+end_of_word (gchar *ct)
+{
+	unicode_char_t uc;
+	gchar *cn;
+
+	while (*ct && (cn = unicode_get_utf8 (ct, &uc)) && html_is_in_word (uc))
+		ct = cn;
+
+	return ct;
+}
+
 void
-html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e)
+html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *i)
 {
 	HTMLObject *obj;
 	HTMLClue *clue;
 	guint off;
 	gchar *text, *ct, *word;
 
-	if (!e->spell_checker)
+	g_assert (HTML_OBJECT_TYPE (flow) == HTML_TYPE_CLUEFLOW);
+
+	/* if (i)
+	   printf ("html_clueflow_spell_check %p %p %d %d\n", i->from, i->to, i->from_offset, i->to_offset); */
+
+	clue = HTML_CLUE (flow);
+	if (!GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
+	    || !e->spell_checker || !clue || !clue->tail)
 		return;
 
 	off  = 0;
-	clue = HTML_CLUE (flow);
-	text = get_text (clue);
-	obj  = clue->head;
+	if (!i)
+		i = html_interval_new (clue->head, clue->tail, 0, html_object_get_length (clue->tail));
+	text = get_text (clue, i);
+	obj  = i->from;
 	if (obj && html_object_is_text (obj))
-		html_text_spell_errors_clear (HTML_TEXT (obj));
+		html_text_spell_errors_clear_interval (HTML_TEXT (obj), i);
 
 	if (text) {
 		ct = text;
 		while (*ct) {
-			/* find begin of word */
-			while (*ct && !isalpha (*ct)) ct++;
-			word = ct;
-			/* find end of word */
-			while (isalpha (*ct)) ct++;
+			word = ct = begin_of_word (text, ct);
+			ct        =   end_of_word (ct);
 
 			/* test if we have found word */
 			if (word != ct) {
@@ -1879,17 +2016,19 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e)
 
 				bak = *ct;
 				*ct = 0;
-				/* printf ("going to test word: \"%s\"\n", word); */
+				/* printf ("off %d going to test word: \"%s\"\n", off, word); */
 				result = pspell_manager_check (e->spell_checker, word);
 
 				if (result == 1) {
 					gboolean is_text = (obj) ? html_object_is_text (obj) : FALSE;
-					while (obj && (!is_text || (is_text && off + HTML_TEXT (obj)->text_len < ct - text)))
-						obj = next_obj (obj, &off, &is_text);
+					while (obj && (!is_text
+						       || (is_text && off + html_interval_get_length (i, obj)
+							   < unicode_index_to_offset (text, ct - text))))
+						obj = next_obj_and_clear (obj, &off, &is_text, i);
 				} else {
 					if (result == 0) {
 						if (obj)
-							obj = spell_check_word (obj, text, word, &off);
+							obj = spell_check_word_mark (obj, text, word, &off, i);
 					} else if (result == -1)
 						g_warning ("pspell error: %s\n",
 							   pspell_manager_error_message (e->spell_checker));
@@ -1898,7 +2037,8 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e)
 				}
 
 				*ct = bak;
-				if (ct) ct++;
+				if (*ct)
+					ct = unicode_next_utf8 (ct);
 			}
 		}
 	}
