@@ -85,6 +85,13 @@ cluev_next_aligned (HTMLObject *aclue)
 	return HTML_OBJECT (HTML_CLUEALIGNED (aclue)->next_aligned);
 }
 
+static gint
+get_lmargin (HTMLObject *o, HTMLPainter *painter)
+{
+	return HTML_CLUEV (o)->padding * html_painter_get_pixel_size (painter)
+		+ (o->parent ?  html_object_get_left_margin (o->parent, o->y) : 0);
+}
+
 static gboolean
 do_layout (HTMLObject *o,
 	   HTMLPainter *painter,
@@ -98,6 +105,9 @@ do_layout (HTMLObject *o,
 	gboolean changed;
 	gint old_width, old_ascent, old_descent;
 	gint new_x;
+	gint pixel_size;
+
+	pixel_size = html_painter_get_pixel_size (painter);
 
 	cluev = HTML_CLUEV (o);
 	clue = HTML_CLUE (o);
@@ -108,18 +118,13 @@ do_layout (HTMLObject *o,
 
 	changed = FALSE;
 
-	if (o->parent != NULL) {
-		lmargin = html_object_get_left_margin (o->parent, o->y);
-		lmargin += cluev->padding;
-	} else {
-		lmargin = cluev->padding;
-	}
+	lmargin = get_lmargin (o, painter);
 
 	/* If we have already called calc_size for the children, then just
 	   continue from the last object done in previous call. */
 	
 	if (clue->curr != NULL) {
-		o->ascent = cluev->padding;
+		o->ascent = cluev->padding * pixel_size;
 		
 		/* Get the current ascent not including curr */
 		obj = clue->head;
@@ -133,7 +138,7 @@ do_layout (HTMLObject *o,
 		remove_aligned_by_parent (cluev, clue->curr);
 	} else {
 		o->width = 0;
-		o->ascent = cluev->padding;
+		o->ascent = pixel_size * cluev->padding;
 		o->descent = 0;
 		clue->curr = clue->head;
 	}
@@ -146,8 +151,8 @@ do_layout (HTMLObject *o,
 		if (calc_size)
 			changed |= html_object_calc_size (clue->curr, painter);
 
-		if (o->width < clue->curr->width + 2 * cluev->padding)
-			o->width = clue->curr->width + 2 * cluev->padding;
+		if (o->width < clue->curr->width + 2 * pixel_size * cluev->padding)
+			o->width = clue->curr->width + 2 * pixel_size * cluev->padding;
 
 		o->ascent += clue->curr->ascent + clue->curr->descent;
 
@@ -157,7 +162,7 @@ do_layout (HTMLObject *o,
 		clue->curr = clue->curr->next;
 	}
 
-	o->ascent += cluev->padding;
+	o->ascent += pixel_size * cluev->padding;
 
 	/* Remember the last object so that we can start from here next time
 	   we are called. */
@@ -230,7 +235,7 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 
 	o->max_width = max_width;
 	for (obj = HTML_CLUE (o)->head; obj != NULL; obj = obj->next)
-		html_object_set_max_width (obj, painter, o->max_width);
+		html_object_set_max_width (obj, painter, max_width);
 }
 
 static void
@@ -423,33 +428,6 @@ relayout (HTMLObject *self,
 	return TRUE;
 }
 
-static gint
-check_page_split (HTMLObject *self,
-		  gint y)
-{
-	HTMLClue *clue;
-	HTMLObject *p;
-
-	clue = HTML_CLUE (self);
-
-	y -= (self->y - self->ascent);
-
-	for (p = clue->head; p != NULL; p = p->next) {
-		gint y1, y2;
-
-		y1 = p->y - p->ascent;
-		y2 = p->y + p->descent;
-
-		if (y1 > y)
-			return y;
-
-		if (y >= y1 && y < y2)
-			return html_object_check_page_split (p, y - y1) + y1;
-	}
-
-	return y;
-}
-
 static gint
 get_left_margin (HTMLObject *self, gint y)
 {
@@ -785,7 +763,6 @@ html_cluev_class_init (HTMLClueVClass *klass,
 	object_class->reset = reset;
 	object_class->draw = draw;
 	object_class->check_point = check_point;
-	object_class->check_page_split = check_page_split;
 	object_class->get_left_margin = get_left_margin;
 	object_class->get_right_margin = get_right_margin;
 
