@@ -24,7 +24,10 @@
 #ifndef HTMLCURSOR_H
 #define HTMLCURSOR_H
 
+typedef struct _HTMLCursor HTMLCursor;
+
 #include <glib.h>
+#include <unicode.h>
 
 #include "htmlobject.h"
 #include "htmlengine.h"
@@ -39,7 +42,6 @@ struct _HTMLCursor {
 
 	gint position;
 };
-typedef struct _HTMLCursor HTMLCursor;
 
 
 /* Lifecycle.  */
@@ -72,10 +74,6 @@ gboolean  html_cursor_jump_to                (HTMLCursor *cursor,
 					      HTMLObject *obj,
 					      guint       offset);
 
-/* Utility functions for moving around the tree in cursor order.  */
-HTMLObject *html_object_next_for_cursor  (HTMLObject *object);
-HTMLObject *html_object_prev_for_cursor  (HTMLObject *object);
-
 /* Internals.  */
 void  html_cursor_normalize     (HTMLCursor *cursor);
 
@@ -86,8 +84,8 @@ void  html_cursor_jump_to_position  (HTMLCursor *cursor,
 				     gint        position);
 
 /* Retrieving the character under/before the cursor.  */
-gchar  html_cursor_get_current_char  (const HTMLCursor *cursor);
-gchar  html_cursor_get_prev_char     (const HTMLCursor *cursor);
+unicode_char_t  html_cursor_get_current_char  (const HTMLCursor *cursor);
+unicode_char_t  html_cursor_get_prev_char     (const HTMLCursor *cursor);
 
 /* Comparison.  */
 gboolean  html_cursor_equal     (const HTMLCursor *a,
