@@ -36,7 +36,7 @@ html_engine_insert_image (HTMLEngine *e,
 			  gint16 height,
 			  gint8 percent,
 			  gint8 border,
-			  const GdkColor *border_color,
+			  HTMLColor *border_color,
 			  HTMLHAlignType halign,
 			  HTMLVAlignType valign,
 			  gint8 hspace,
