@@ -58,7 +58,7 @@ extend_selection (HTMLEngine *engine,
 
 	obj = old_point->object;
 
-	while (obj != new_point->object) {
+	while (obj && obj != new_point->object) {
 		if (forward) {
 			if (obj == mark->object)
 				start = mark->offset;
@@ -79,11 +79,11 @@ extend_selection (HTMLEngine *engine,
 			prev_clueflow = NULL;
 
 		if (forward)
-			obj = html_object_next_for_cursor (obj);
+			obj = html_object_next_leaf (obj);
 		else
-			obj = html_object_prev_for_cursor (obj);
+			obj = html_object_prev_leaf (obj);
 
-		if (obj->parent != prev_clueflow && prev_clueflow != NULL)
+		if (obj && obj->parent != prev_clueflow && prev_clueflow != NULL)
 			html_object_select_range (prev_clueflow, engine, 0, -1, TRUE);
 	}
 
@@ -141,9 +141,9 @@ reduce_selection (HTMLEngine *engine,
 			prev_clueflow = NULL;
 
 		if (forward)
-			obj = html_object_next_for_cursor (obj);
+			obj = html_object_next_leaf (obj);
 		else
-			obj = html_object_prev_for_cursor (obj);
+			obj = html_object_prev_leaf (obj);
 
 		if (obj->parent != prev_clueflow)
 			html_object_select_range (obj->parent, engine, 0, 0, TRUE);
@@ -198,7 +198,7 @@ update_selection (HTMLEngine *engine,
 		return;
 
 	if (html_cursor_equal (new_point, mark)) {
-		html_engine_unselect_all (engine, TRUE);
+		html_engine_unselect_all (engine);
 		engine->active_selection = TRUE;
 		return;
 	}
@@ -210,7 +210,7 @@ update_selection (HTMLEngine *engine,
 	if ((html_cursor_precedes (new_point, mark) && html_cursor_follows (old_point, mark))
 	    || (html_cursor_follows (new_point, mark) && html_cursor_precedes (old_point, mark)))
 	{
-		html_engine_unselect_all (engine, TRUE);
+		html_engine_unselect_all (engine);
 		old_point = mark;
 	}
 
@@ -323,6 +323,7 @@ html_engine_edit_selection_updater_schedule (HTMLEngineEditSelectionUpdater *upd
 
 	if (updater->idle_id != 0)
 		return;
+
 	updater->idle_id = gtk_idle_add (updater_idle_callback, updater);
 }
 
