@@ -22,6 +22,7 @@
 */
 
 
+#include <ctype.h>
 #include <glib.h>
 
 #include "htmlobject.h"
@@ -34,6 +35,7 @@
 #include "htmlengine-cutbuffer.h"
 #include "htmlengine-edit-cut.h"
 #include "htmlengine-edit-cursor.h"
+#include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-paste.h"
 #include "htmlengine-edit.h"
 
@@ -48,7 +50,7 @@ html_engine_undo (HTMLEngine *e)
 	g_return_if_fail (e->undo != NULL);
 	g_return_if_fail (e->editable);
 
-	html_engine_unselect_all (e, TRUE);
+	html_engine_unselect_all (e);
 
 	undo = e->undo;
 	html_undo_do_undo (undo, e);
@@ -63,7 +65,7 @@ html_engine_redo (HTMLEngine *e)
 	g_return_if_fail (HTML_IS_ENGINE (e));
 	g_return_if_fail (e->undo != NULL);
 
-	html_engine_unselect_all (e, TRUE);
+	html_engine_unselect_all (e);
 
 	undo = e->undo;
 	html_undo_do_redo (undo, e);
@@ -78,7 +80,7 @@ html_engine_set_mark (HTMLEngine *e)
 	g_return_if_fail (e->editable);
 
 	if (e->mark != NULL)
-		html_engine_unselect_all (e, TRUE);
+		html_engine_unselect_all (e);
 
 	e->mark = html_cursor_dup (e->cursor);
 	e->active_selection = TRUE;
@@ -132,10 +134,10 @@ html_engine_selection_pop (HTMLEngine *e)
 	if (selection) {
 		gint cursor, mark;
 
-		mark = GPOINTER_TO_INT (e->selection_stack->data);
-		e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
 		cursor = GPOINTER_TO_INT (e->selection_stack->data);
 		e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
+		mark = GPOINTER_TO_INT (e->selection_stack->data);
+		e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
 
 		html_cursor_jump_to_position (e->cursor, e, mark);
 		html_engine_set_mark (e);
@@ -174,3 +176,55 @@ html_engine_cut_and_paste (HTMLEngine *e, gchar *op_name, GFunc iterator, gpoint
 	g_list_foreach (e->cut_buffer, iterator, data);
 	html_engine_cut_and_paste_end (e);
 }
+
+#ifdef GTKHTML_HAVE_PSPELL
+void
+html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
+{
+	HTMLObject *parent;
+	HTMLInterval *i;
+
+	begin = html_cursor_dup (begin);
+	end   = html_cursor_dup (end);
+
+	g_assert (begin->object->parent == end->object->parent);
+
+	parent = begin->object->parent;
+
+	while (html_is_in_word (html_cursor_get_prev_char (begin)))
+		if (html_cursor_backward (begin, e));
+	while (html_is_in_word (html_cursor_get_current_char (end)))
+		if (html_cursor_forward (end, e));
+
+	i = html_interval_new_from_cursor (begin, end);
+	html_clueflow_spell_check (HTML_CLUEFLOW (parent), e, i);
+	html_interval_destroy (i);
+	html_cursor_destroy (begin);
+	html_cursor_destroy (end);
+}
+#endif
+
+gboolean
+html_is_in_word (unicode_char_t uc)
+{
+	/* printf ("test %d %c => %d\n", uc, uc, unicode_isalnum (uc) || uc == '\''); */
+	return unicode_isalnum (uc) || uc == '\'';
+}
+
+void
+html_engine_select_word_editable (HTMLEngine *e)
+{
+	while (html_is_in_word (html_cursor_get_prev_char (e->cursor)))
+		html_cursor_backward (e->cursor, e);
+	html_engine_set_mark (e);
+	while (html_is_in_word (html_cursor_get_current_char (e->cursor)))
+		html_cursor_forward (e->cursor, e);
+}
+
+void
+html_engine_select_line_editable (HTMLEngine *e)
+{
+	html_engine_beginning_of_line (e);
+	html_engine_set_mark (e);
+	html_engine_end_of_line (e);
+}
