@@ -22,27 +22,36 @@
 #define _HTMLENGINE_EDIT_H
 
 #include <glib.h>
+#include <unicode.h>
 
 #include "htmlengine.h"
 
 
 /* Undo/redo.  */
-void  html_engine_undo                (HTMLEngine *e);
-void  html_engine_redo                (HTMLEngine *e);
-void  html_engine_set_mark            (HTMLEngine *e);
+void  html_engine_undo                 (HTMLEngine *e);
+void  html_engine_redo                 (HTMLEngine *e);
+void  html_engine_set_mark             (HTMLEngine *e);
+void  html_engine_select_word_editable (HTMLEngine *e);
+void  html_engine_select_line_editable (HTMLEngine *e);
 
-void  html_engine_cut_buffer_push     (HTMLEngine *e);
-void  html_engine_cut_buffer_pop      (HTMLEngine *e);
+void  html_engine_cut_buffer_push      (HTMLEngine *e);
+void  html_engine_cut_buffer_pop       (HTMLEngine *e);
 
-void  html_engine_selection_push      (HTMLEngine *e);
-void  html_engine_selection_pop       (HTMLEngine *e);
+void  html_engine_selection_push       (HTMLEngine *e);
+void  html_engine_selection_pop        (HTMLEngine *e);
 
-void  html_engine_cut_and_paste_begin (HTMLEngine *e,
-				       gchar *op_name);
-void  html_engine_cut_and_paste_end   (HTMLEngine *e);
-void  html_engine_cut_and_paste       (HTMLEngine *e,
-				       gchar *op_name,
-				       GFunc iterator,
-				       gpointer data);
+void  html_engine_cut_and_paste_begin  (HTMLEngine *e,
+					gchar *op_name);
+void  html_engine_cut_and_paste_end    (HTMLEngine *e);
+void  html_engine_cut_and_paste        (HTMLEngine *e,
+					gchar *op_name,
+					GFunc iterator,
+					gpointer data);
+#ifdef GTKHTML_HAVE_PSPELL
+void  html_engine_spell_check_range    (HTMLEngine *e,
+					HTMLCursor *begin,
+					HTMLCursor *end);
+#endif
+gboolean html_is_in_word               (unicode_char_t uc);
 
 #endif /* _HTMLENGINE_EDIT_H */
