@@ -133,6 +133,7 @@ struct _HTMLEngine {
 	gboolean newPage;
  
 	HTMLStack *font_style_stack; /* Font style stack, elements are GtkHTMLFontStyles.  */
+	HTMLStack *font_face_stack;
 	HTMLStack *color_stack;	/* Color stack, elements are GdkColors.  */
 	HTMLStack *clueflow_style_stack; /* Clueflow style stack, elements are HTMLClueFlowStyles.  */
 
@@ -251,6 +252,9 @@ struct _HTMLEngine {
 	PspellConfig  *spell_config;
 	PspellManager *spell_checker;
 #endif
+	
+	/* image id storage */
+	GHashTable *imageid_table;
 };
 
 /* must be forward referenced *sigh* */
@@ -324,6 +328,10 @@ void  html_engine_queue_clear       (HTMLEngine *e,
 				     guint       width,
 				     guint       height);
 
+void  html_engine_set_painter       (HTMLEngine  *e,
+				     HTMLPainter *painter,
+				     gint max_width);
+
 /* Getting objects through pointer positions.  */
 HTMLObject  *html_engine_get_object_at  (HTMLEngine *e,
 					 gint        x,
@@ -349,10 +357,17 @@ void   html_engine_select_region         (HTMLEngine *e,
 					  gint        x1,
 					  gint        y1,
 					  gint        x2,
-					  gint        y2,
-					  gboolean    queue_draw);
-void   html_engine_unselect_all          (HTMLEngine *e,
-					  gboolean    queue_draw);
+					  gint        y2);
+
+void   html_engine_set_active_selection  (HTMLEngine *e,
+					  gboolean active,
+					  guint32 time);
+HTMLCursor *
+       html_engine_get_cursor            (HTMLEngine *e);
+void   html_engine_select_word           (HTMLEngine *e);
+void   html_engine_select_line           (HTMLEngine *e);
+void   html_engine_unselect_all          (HTMLEngine *e);
+
 void   html_engine_disable_selection     (HTMLEngine *e);
 gchar *html_engine_get_selection_string  (HTMLEngine *e);
 
@@ -369,13 +384,6 @@ void      html_engine_thaw    (HTMLEngine *engine);
 void      html_engine_load_empty                (HTMLEngine *engine);
 
 /* Search & Replace */
-gboolean  html_engine_search                    (HTMLEngine *e,
-						 const gchar *text,
-						 gboolean case_sensitive,
-						 gboolean forward,
-						 gboolean regular);
-gboolean  html_engine_search_next               (HTMLEngine *e);
-gboolean  html_engine_search_incremental        (HTMLEngine *e);
 
 void      html_engine_replace                   (HTMLEngine *e,
 						 const gchar *text,
