@@ -20,10 +20,13 @@
 */
 
 #include <string.h>
+#include <stdlib.h>
 #include <libart_lgpl/art_rect.h>
+#include <gdk/gdkx.h>
+#include <unicode.h>
+#include <gal/widgets/e-font.h>
 
 #include "htmlentity.h"
-
 #include "htmlgdkpainter.h"
 
 
@@ -42,14 +45,137 @@ finalize (GtkObject *object)
 	if (painter->gc != NULL)
 		gdk_gc_destroy (painter->gc);
 
-	html_gdk_font_manager_destroy (painter->font_manager);
-
 	if (painter->pixmap != NULL)
 		gdk_pixmap_unref (painter->pixmap);
 
 	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static gint
+get_size (gchar *font_name)
+{
+    gchar *s, *end;
+    gint n;
+
+    /* Search paramether */
+    for (s=font_name, n=7; n; n--,s++)
+	    s = strchr (s,'-');
+
+    if (s && *s != 0) {
+	    end = strchr (s, '-');
+	    if (end) {
+		    *end = 0;
+		    n = atoi (s);
+		    *end = '-';
+		    return n;
+	    } else
+		    return 0;
+    } else
+	    return 0;
+}
+
+static gboolean
+find_font (gchar *font_name, gdouble req_size, gint *font_size)
+{
+	gint n;
+	gdouble size, smaller, bigger;
+	gchar **list;
+	gboolean rv = FALSE;
+
+	/* list available sizes */
+	smaller = bigger = size = .0;
+	list = XListFonts (GDK_DISPLAY (), font_name, 0xffff, &n);
+	/* look for right one */
+	while (n) {
+		n--;
+		size = get_size (list [n]);
+		if ((gdouble) size == req_size) {
+			*font_size = size;
+			rv = TRUE;
+			break;
+		} else if (size < req_size && size > smaller)
+			smaller = size;
+		else if (size > req_size && (size < bigger || bigger == .0))
+			bigger = size;
+	}
+	XFreeFontNames (list);
+
+	/* if not found use closest one */
+	if (!rv && (bigger || smaller)) {
+		if (bigger == .0)
+			size = smaller;
+		if (smaller == .0)
+			size = bigger;
+		if (bigger != .0 && smaller != .0)
+			size = (bigger - req_size <= req_size - smaller) ? bigger : smaller;
+		*font_size = size;
+		rv = TRUE;
+	}
+
+	return rv;
+}
+
+static GdkFont *
+get_font_with_enc (const gchar *family, const gchar *weight, const gchar *slant, gdouble size, gchar *enc)
+{
+	GdkFont *font;
+	gchar *font_name;
+	gint font_size;
+
+	font      = NULL;
+	font_name = g_strdup_printf ("-*-%s-%s-%s-normal-*-*-*-*-*-*-*-%s",
+				     family, weight, slant, enc);
+	if (find_font (font_name, size, &font_size)) {
+		g_free (font_name);
+		font_name = g_strdup_printf ("-*-%s-%s-%s-normal-*-%d-*-*-*-*-*-%s",
+					     family, weight, slant, font_size, enc);
+		font = gdk_font_load (font_name);
+	}
+	g_free (font_name);
+
+	/* printf ("family: %s enc: %s ==> %p\n", family, enc, font); */
+
+	return font;
+}
+
+static GdkFont *
+get_font (const gchar *family, const gchar *weight, const gchar *slant, gdouble size)
+{
+	GdkFont *font;
+
+	/* prefer unicode fonts */
+	font = get_font_with_enc (family, weight, slant, size, "iso10646-1");
+	return font ? font : get_font_with_enc (family, weight, slant, size, "*-*");
+}
+
+static GdkFont *
+get_closest_font (const gchar *family, const gchar *weight, const gchar *slant, gdouble size)
+{
+	GdkFont *font;
+
+	font = get_font (family, weight, slant, size);
+	if (!font && *slant == 'i')
+		font = get_font (family, weight, "o", size);
+
+	return font;
+}
+
+static gpointer
+alloc_font (gchar *face, gdouble size, GtkHTMLFontStyle style)
+{
+	return (face) ? get_closest_font (face, (style & GTK_HTML_FONT_STYLE_BOLD) ? "bold" : "medium",
+					  (style & GTK_HTML_FONT_STYLE_ITALIC) ? "i" : "r", size) : gdk_font_load ("fixed");
+}
+
+static gpointer
+alloc_e_font (gchar *face, gdouble size, GtkHTMLFontStyle style)
+{
+	GdkFont *font;
+
+	font = alloc_font (face, size, style);
+	return font ? e_font_from_gdk_font (font) : NULL;
+}
+
 static void
 alloc_color (HTMLPainter *painter,
 	     GdkColor *color)
@@ -210,25 +336,6 @@ get_black (const HTMLPainter *painter)
 	return &gdk_painter->black;
 }
 
-static void
-set_font_style (HTMLPainter *painter,
-		GtkHTMLFontStyle style)
-{
-	HTMLGdkPainter *gdk_painter;
-
-	gdk_painter = HTML_GDK_PAINTER (painter);
-	gdk_painter->font_style = style;
-}
-
-static GtkHTMLFontStyle
-get_font_style (HTMLPainter *painter)
-{
-	HTMLGdkPainter *gdk_painter;
-
-	gdk_painter = HTML_GDK_PAINTER (painter);
-	return gdk_painter->font_style;
-}
-
 
 /* HTMLPainter drawing functions.  */
 
@@ -711,6 +818,70 @@ fill_rect (HTMLPainter *painter,
 			    width, height);
 }
 
+/* static gint
+utf8_gdk_text_width (GdkFont *font, const gchar *text, gint len)
+{
+	const gchar *p;
+	gint width, bytes;
+
+	if (!text || !font) return 0;
+
+	width = 0;
+	bytes = unicode_offset_to_index (text, len);
+	for (p = text; p && *p && p - text < bytes; p = unicode_next_utf8 (p)) {
+		unicode_char_t unival;
+		guchar c;
+		unicode_get_utf8 (p, &unival);
+		if (unival < 0) unival = '_';
+		if (unival > 255) unival = '_';
+		if (unival == 0xa0) unival = ' ';
+		c = unival;
+		width += gdk_text_width (font, &c, 1);
+	}
+
+	return width;
+}
+
+static void
+utf8_gdk_draw_text (GdkFont *font,
+		    GdkDrawable *drawable,
+		    GdkGC *gc,
+		    gint x, gint y,
+		    const gchar *text, gint len)
+{
+	const gchar *p;
+	guchar *b;
+	gint off, bytes;
+
+	if (!text || !font) return;
+
+	bytes = unicode_offset_to_index (text, len);
+	b = alloca (bytes);
+	off = 0;
+
+	for (p = text; p && *p && p - text < bytes; p = unicode_next_utf8 (p)) {
+		unicode_char_t unival;
+		unicode_get_utf8 (p, &unival);
+		if (unival < 0) unival = '_';
+		if (unival > 255) unival = '_';
+		if (unival == 0xa0) unival = ' ';
+		b [off++] = unival;
+	}
+
+	gdk_draw_text (drawable, font, gc, x, y, b, len);
+} */
+
+static EFontStyle
+e_style (GtkHTMLFontStyle style)
+{
+	EFontStyle rv = E_FONT_PLAIN;
+
+	if (style & GTK_HTML_FONT_STYLE_BOLD)   rv |= E_FONT_BOLD;
+	if (style & GTK_HTML_FONT_STYLE_ITALIC) rv |= E_FONT_ITALIC;
+
+	return rv;
+}
+
 #ifdef GTKHTML_HAVE_PSPELL
 static void
 draw_spell_error (HTMLPainter *painter,
@@ -719,25 +890,22 @@ draw_spell_error (HTMLPainter *painter,
 		  guint off, gint len)
 {
 	HTMLGdkPainter *gdk_painter;
-	GdkFont *gdk_font;
+	EFont *e_font;
 	GdkGCValues values;
 	guint x_off, width;
-	gchar *pt;
 	gchar dash [2];
 
-	pt = html_entity_prepare (text);
-
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
 	x -= gdk_painter->x1;
 	y -= gdk_painter->y1;
 
-	gdk_font = html_gdk_font_manager_get_font (gdk_painter->font_manager,
-						   gdk_painter->font_style);
+	e_font = html_painter_get_font  (painter, painter->font_face, painter->font_style);
+	x_off  = e_font_utf8_text_width (e_font, e_style (painter->font_style), text,
+					 unicode_offset_to_index (text, off)) + x;
+	width  = e_font_utf8_text_width (e_font, e_style (painter->font_style), text + unicode_offset_to_index (text, off),
+					 unicode_offset_to_index (text + unicode_offset_to_index (text, off), len)) + x_off;
 
-	x_off = gdk_text_width (gdk_font, text,       off) + x;
-	width = gdk_text_width (gdk_font, text + off, len) + x_off;
-	
 	gdk_gc_get_values (gdk_painter->gc, &values);
 	gdk_gc_set_fill (gdk_painter->gc, GDK_OPAQUE_STIPPLED);
 	dash [0] = 2;
@@ -753,8 +921,6 @@ draw_spell_error (HTMLPainter *painter,
 		       width, y + 1);
 	gdk_gc_set_line_attributes (gdk_painter->gc, values.line_width,
 				    values.line_style, values.cap_style, values.join_style);
-	g_free (pt);
-	gdk_font_unref (gdk_font);
 }
 #endif
 
@@ -765,49 +931,37 @@ draw_text (HTMLPainter *painter,
 	   gint len)
 {
 	HTMLGdkPainter *gdk_painter;
-	GdkFont *gdk_font;
-	gchar *pt;
+	EFont *e_font;
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
-	/* FIXME:  Ick!  This is pretty gross.  :-)  */
-
-	pt = html_entity_prepare (text);
-
 	if (len == -1)
-		len = strlen (pt);
+		len = unicode_strlen (text, -1);
 
 	x -= gdk_painter->x1;
 	y -= gdk_painter->y1;
 
-	gdk_font = html_gdk_font_manager_get_font (gdk_painter->font_manager,
-						   gdk_painter->font_style);
+	e_font = html_painter_get_font (painter, painter->font_face, painter->font_style);
+	e_font_draw_utf8_text (gdk_painter->pixmap, e_font, e_style (painter->font_style), gdk_painter->gc,
+			       x, y, text, unicode_offset_to_index (text, len));
 
-	gdk_draw_text (gdk_painter->pixmap,
-		       gdk_font,
-		       gdk_painter->gc,
-		       x, y,
-		       pt, len);
-
-	if (gdk_painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE
-				       | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
+	if (painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE
+				   | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
 		guint width;
 
-		width = gdk_text_width (gdk_font, pt, len);
+		width = e_font_utf8_text_width (e_font, e_style (painter->font_style),
+						text, unicode_offset_to_index (text, len));
 
-		if (gdk_painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE)
+		if (painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE)
 			gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, 
 				       x, y + 1, 
 				       x + width, y + 1);
 
-		if (gdk_painter->font_style & GTK_HTML_FONT_STYLE_STRIKEOUT)
+		if (painter->font_style & GTK_HTML_FONT_STYLE_STRIKEOUT)
 			gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, 
-				       x, y - gdk_font->ascent / 2, 
-				       x + width, y - gdk_font->ascent / 2);
+				       x, y - e_font_ascent (e_font) / 2, 
+				       x + width, y - e_font_ascent (e_font) / 2);
 	}
-
-	g_free (pt);
-	gdk_font_unref (gdk_font);
 }
 
 static void
@@ -829,61 +983,51 @@ draw_shade_line (HTMLPainter *painter,
 }
 
 static guint
-calc_ascent (HTMLPainter *painter,
-	     GtkHTMLFontStyle style)
+calc_ascent (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
 {
 	HTMLGdkPainter *gdk_painter;
-	GdkFont *gdk_font;
-	gint ascent;
+	EFont *e_font;
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
-	gdk_font = html_gdk_font_manager_get_font (gdk_painter->font_manager, style);
-	if (gdk_font == NULL)
+	e_font = html_painter_get_font (painter, face, style);
+	if (e_font == NULL)
 		return 0;
 
-	ascent = gdk_font->ascent;
-	gdk_font_unref (gdk_font);
-
-	return ascent;
+	return e_font_ascent (e_font);
 }
 
 static guint
-calc_descent (HTMLPainter *painter,
-	      GtkHTMLFontStyle style)
+calc_descent (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
 {
 	HTMLGdkPainter *gdk_painter;
-	GdkFont *gdk_font;
-	gint descent;
+	EFont *e_font;
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
-	gdk_font = html_gdk_font_manager_get_font (gdk_painter->font_manager, style);
-	if (gdk_font == NULL)
+	e_font = html_painter_get_font (painter, face, style);
+	if (e_font == NULL)
 		return 0;
 
-	descent = gdk_font->descent;
-	gdk_font_unref (gdk_font);
-
-	return descent;
+	return e_font_descent (e_font);
 }
 
 static guint
 calc_text_width (HTMLPainter *painter,
 		 const gchar *text,
 		 guint len,
-		 GtkHTMLFontStyle style)
+		 GtkHTMLFontStyle style,
+		 HTMLFontFace *face)
 {
 	HTMLGdkPainter *gdk_painter;
-	GdkFont *gdk_font;
+	EFont *e_font;
 	gint width;
-	gchar *pt = html_entity_prepare (text);
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
-	gdk_font = html_gdk_font_manager_get_font (gdk_painter->font_manager, style);
-	width = gdk_text_width (gdk_font, pt, len);
-	gdk_font_unref (gdk_font);
-	g_free (pt);
+	e_font = html_painter_get_font (painter, face, style);
+
+	width = e_font_utf8_text_width (e_font, e_style (painter->font_style),
+					text, unicode_offset_to_index (text, len));
 
 	return width;
 }
@@ -923,9 +1067,6 @@ init (GtkObject *object)
 	gdk_painter->set_background = FALSE;
 	gdk_painter->do_clear = FALSE;
 
-	gdk_painter->font_manager = html_gdk_font_manager_new ();
-	gdk_painter->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
-
 	init_color (& gdk_painter->background, 0xffff, 0xffff, 0xffff);
 	init_color (& gdk_painter->dark, 0, 0, 0);
 	init_color (& gdk_painter->light, 0, 0, 0);
@@ -942,10 +1083,11 @@ class_init (GtkObjectClass *object_class)
 
 	painter_class->begin = begin;
 	painter_class->end = end;
+	painter_class->alloc_font = alloc_e_font;
+	painter_class->ref_font   = (HTMLFontManagerRefFont)   e_font_ref;
+	painter_class->unref_font = (HTMLFontManagerUnrefFont) e_font_unref;
 	painter_class->alloc_color = alloc_color;
 	painter_class->free_color = free_color;
-	painter_class->set_font_style = set_font_style;
-	painter_class->get_font_style = get_font_style;
 	painter_class->calc_ascent = calc_ascent;
 	painter_class->calc_descent = calc_descent;
 	painter_class->calc_text_width = calc_text_width;
