@@ -23,7 +23,7 @@
 #define _HTMLGDKPAINTER_H
 
 #include "htmlpainter.h"
-#include "htmlgdkfontmanager.h"
+#include "htmlfontmanager.h"
 
 
 #define HTML_TYPE_GDK_PAINTER                 (html_gdk_painter_get_type ())
@@ -51,10 +51,6 @@ struct _HTMLGdkPainter {
 	gboolean do_clear;
 	gboolean alpha;
 
-	/* Font handling.  */
-	HTMLGdkFontManager *font_manager;
-	GtkHTMLFontStyle font_style;
-
 	/* Colors used for shading.  */
 	GdkColor dark;
 	GdkColor light;
