@@ -21,6 +21,8 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "htmliframe.h"
+#include "htmlengine-search.h"
+#include "htmlsearch.h"
 
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
@@ -70,7 +72,19 @@ iframe_size_changed (GtkAdjustment *adj, gpointer data)
 	
 	html_engine_schedule_update (parent->engine);
 }
-	
+
+static gboolean
+iframe_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
+{
+	HTMLIFrame *iframe = HTML_IFRAME (data);
+	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
+	gboolean ret_val;
+
+	ret_val = FALSE;
+	gtk_signal_emit_by_name (GTK_OBJECT (parent), "object_requested", eb, &ret_val);
+	return ret_val;
+}
+		
 HTMLObject *
 html_iframe_new (GtkWidget *parent, 
 		 char *src, 
@@ -112,7 +126,6 @@ calc_min_width (HTMLObject *o,
 	min_width = html_object_calc_min_width (html->engine->clue,
 						html->engine->painter);
 	html->engine->width = min_width;
-	html_engine_calc_size (html->engine);
 	min_width = html_engine_get_doc_width (html->engine);
     
 	return min_width;
@@ -135,33 +148,94 @@ reset (HTMLObject *o)
 	html_object_reset (GTK_HTML (iframe->html)->engine->clue);
 }
 
+/* FIXME rodo - draw + set_painter is not much clean now, needs refactoring */
+
+static void
+draw (HTMLObject *o,
+      HTMLPainter *p,
+      gint x, gint y,
+      gint width, gint height,
+      gint tx, gint ty)
+{
+	HTMLEmbedded *element = HTML_EMBEDDED(o);
+	HTMLIFrame *iframe;
+	ArtIRect paint;
+	gint new_x, new_y;
+
+	html_object_calc_intersection (o, &paint, x, y, width, height);
+	if (art_irect_empty (&paint))
+		return;
+
+	iframe = HTML_IFRAME (o);
+	if (GTK_OBJECT_TYPE (GTK_HTML (iframe->html)->engine->painter) != HTML_TYPE_GDK_PAINTER)
+		html_object_draw (GTK_HTML (iframe->html)->engine->clue, GTK_HTML (iframe->html)->engine->painter,
+				  x, y, width, height, tx, ty);
+
+	if (element->widget) {
+		new_x = GTK_LAYOUT (element->parent)->hadjustment->value + o->x + tx;
+		new_y = GTK_LAYOUT (element->parent)->vadjustment->value + o->y + ty - o->ascent;
+
+		if(new_x != element->abs_x || new_y != element->abs_y)
+			gtk_layout_move(GTK_LAYOUT(element->parent), element->widget,
+					new_x, new_y);
+		element->abs_x = new_x;
+		element->abs_y = new_y;
+	}
+}
+
+static void
+set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+{
+	HTMLIFrame *iframe;
+
+	iframe = HTML_IFRAME (o);
+	if (GTK_OBJECT_TYPE (GTK_HTML (iframe->html)->engine->painter) == HTML_TYPE_GDK_PAINTER)
+		iframe->gdk_painter = GTK_HTML (iframe->html)->engine->painter;
+	html_engine_set_painter (GTK_HTML (iframe->html)->engine,
+				 GTK_OBJECT_TYPE (painter) == HTML_TYPE_GDK_PAINTER ? iframe->gdk_painter : painter,
+				 max_width);
+}
+
+static void
+forall (HTMLObject *self,
+	HTMLObjectForallFunc func,
+	gpointer data)
+{
+	HTMLIFrame *iframe;
+
+	iframe = HTML_IFRAME (self);
+	(* func) (self, data);
+	html_object_forall (GTK_HTML (iframe->html)->engine->clue, func, data);
+}
+
+static gint
+check_page_split (HTMLObject *self, gint y)
+{
+	return html_object_check_page_split (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue, y);
+}
+
 static gboolean
 calc_size (HTMLObject *o,
 	   HTMLPainter *painter)
 {
 	HTMLIFrame *iframe;
-	guint pixel_size = 1;
+	HTMLEngine *e;
 	gint width, height;
 	gint old_width, old_ascent, old_descent;
-	GtkHTML *html;
 	
-	if (painter)
-		pixel_size = html_painter_get_pixel_size (painter);
-       
 	old_width = o->width;
 	old_ascent = o->ascent;
 	old_descent = o->descent;
 
 	iframe = HTML_IFRAME (o);
-	
-	html = GTK_HTML (iframe->html);
+	e      = GTK_HTML (iframe->html)->engine;
 
 	if ((iframe->width < 0) && (iframe->height < 0)) {
-		html->engine->width = o->max_width;
-		html_engine_calc_size (html->engine);
+		e->width = o->max_width;
+		html_engine_calc_size (e);
 
-		height = html_engine_get_doc_height (html->engine);
-		width = html_engine_get_doc_width (html->engine);
+		height = html_engine_get_doc_height (e);
+		width = html_engine_get_doc_width (e);
 
 		gtk_widget_set_usize (iframe->scroll, width, height);
 		gtk_widget_queue_resize (iframe->scroll);
@@ -170,13 +244,12 @@ calc_size (HTMLObject *o,
 						GTK_POLICY_NEVER,
 						GTK_POLICY_NEVER);
 
-		o->width = width * pixel_size;
-		o->ascent = height * pixel_size;
+		o->width = width;
+		o->ascent = height;
 		o->descent = 0;
-	} else {
+	} else
 		return (* HTML_OBJECT_CLASS (parent_class)->calc_size) 
 			(o, painter);
-	}
 
 	if (o->descent != old_descent
 	    || o->ascent != old_ascent
@@ -186,6 +259,35 @@ calc_size (HTMLObject *o,
 	return FALSE;
 }
 
+static gboolean
+search (HTMLObject *self, HTMLSearch *info)
+{
+	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
+
+	/* printf ("search\n"); */
+
+	/* search_next? */
+	if (info->stack && HTML_OBJECT (info->stack->data) == e->clue) {
+		/* printf ("next\n"); */
+		info->engine = GTK_HTML (GTK_HTML (HTML_IFRAME (self)->html)->iframe_parent)->engine;
+		html_search_pop (info);
+		html_engine_unselect_all (e);
+		return html_search_next_parent (info);
+	}
+
+	info->engine = e;
+	html_search_push (info, e->clue);
+	if (html_object_search (e->clue, info))
+		return TRUE;
+	html_search_pop (info);
+
+	info->engine = GTK_HTML (GTK_HTML (HTML_IFRAME (self)->html)->iframe_parent)->engine;
+	/* printf ("FALSE\n"); */
+
+	return FALSE;
+}
+
+
 void 
 html_iframe_init (HTMLIFrame *iframe,
 		  HTMLIFrameClass *klass,
@@ -212,6 +314,7 @@ html_iframe_init (HTMLIFrame *iframe,
 					GTK_POLICY_AUTOMATIC);
 	html = gtk_html_new ();
 	iframe->html = html;
+	/* GTK_HTML (html)->engine->clue->parent = HTML_OBJECT (iframe); */
 	gtk_html_set_iframe_parent (GTK_HTML (html), parent);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), html);
 	gtk_widget_show (html);
@@ -236,6 +339,9 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_connect (GTK_OBJECT (html), "size_changed",
 			    GTK_SIGNAL_FUNC (iframe_size_changed),
 			    (gpointer)iframe);	
+	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
+			    GTK_SIGNAL_FUNC (iframe_object_requested),
+			    (gpointer)iframe);	
 	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
@@ -280,7 +386,6 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
 			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
 	*/
-	
 }
 
 
@@ -308,7 +413,11 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 
 	object_class->calc_size = calc_size;
 	object_class->calc_min_width = calc_min_width;
+	object_class->set_painter = set_painter;
 	object_class->reset = reset;
+	object_class->draw = draw;
 	object_class->set_max_width = set_max_width;
-	
+	object_class->forall = forall;
+	object_class->check_page_split = check_page_split;
+	object_class->search = search;
 }
