@@ -47,10 +47,20 @@ HTMLObjectClass html_object_class;
 static void
 destroy (HTMLObject *self)
 {
-	if (self->redraw_pending)
+	if (self->redraw_pending) {
 		self->free_pending = TRUE;
-	else
+	} else {
+
+#if GTKHTML_MEM_DEBUG
+		self->next = 0xdeadbeef;
+		self->prev = 0xdeadbeef;
+#else
+		self->next = NULL;
+		self->prev = NULL;
+#endif
 		g_free (self);
+
+	}
 }
 
 static void
@@ -175,6 +185,11 @@ get_right_margin (HTMLObject *self, gint y)
 	return MAX (self->max_width, self->width);
 }
 
+static void
+set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+{
+}
+
 static void
 reset (HTMLObject *o)
 {
@@ -435,7 +450,8 @@ save (HTMLObject *self,
 
 static gboolean
 save_plain (HTMLObject *self,
-      HTMLEngineSaveState *state)
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
 {
 	return TRUE;
 }
@@ -444,7 +460,7 @@ static gint
 check_page_split (HTMLObject *self,
 		  gint y)
 {
-	return y;
+	return 0;
 }
 
 static gboolean
@@ -454,6 +470,30 @@ search (HTMLObject *self, HTMLSearch *info)
 	return FALSE;
 }
 
+static HTMLObject *
+next (HTMLObject *self, HTMLObject *child)
+{
+	return NULL;
+}
+
+static HTMLObject *
+prev (HTMLObject *self, HTMLObject *child)
+{
+	return NULL;
+}
+
+static HTMLObject *
+head (HTMLObject *self)
+{
+	return NULL;
+}
+
+static HTMLObject *
+tail (HTMLObject *self)
+{
+	return NULL;
+}
+
 
 /* Class initialization.  */
 
@@ -487,6 +527,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->set_max_width = set_max_width;
 	klass->get_left_margin = get_left_margin;
 	klass->get_right_margin = get_right_margin;
+	klass->set_painter = set_painter;
 	klass->reset = reset;
 	klass->calc_min_width = calc_min_width;
 	klass->calc_preferred_width = calc_preferred_width;
@@ -512,6 +553,10 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->search = search;
 	klass->search_next = search;
 	klass->get_length = get_length;
+	klass->next = next;
+	klass->prev = prev;
+	klass->head = head;
+	klass->tail = tail;
 }
 
 void
@@ -573,8 +618,7 @@ html_object_dup (HTMLObject *object)
 
 
 void
-html_object_set_parent (HTMLObject *o,
-			HTMLObject *parent)
+html_object_set_parent (HTMLObject *o, HTMLObject *parent)
 {
 	o->parent = parent;
 }
@@ -706,6 +750,33 @@ html_object_get_right_margin (HTMLObject *self, gint y)
 	return (* HO_CLASS (self)->get_right_margin) (self, y);
 }
 
+struct _hlpSetPainter {
+	HTMLPainter *painter;
+	gint max_width;
+};
+typedef struct _hlpSetPainter hlpSetPainter;
+
+static void
+set_painter_forall (HTMLObject *o, gpointer data)
+{
+	hlpSetPainter *sp;
+
+	sp = (hlpSetPainter *) data;
+	(* HO_CLASS (o)->set_painter) (o, sp->painter, sp->max_width);
+}
+
+void
+html_object_set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+{
+	hlpSetPainter *sp;
+
+	sp = g_new (hlpSetPainter, 1);
+	sp->painter = painter;
+	sp->max_width = max_width;
+	html_object_forall (o, set_painter_forall, sp);
+	g_free (sp);
+}
+
 void
 html_object_reset (HTMLObject *o)
 {
@@ -912,9 +983,10 @@ html_object_save (HTMLObject *self,
 
 gboolean
 html_object_save_plain (HTMLObject *self,
-			HTMLEngineSaveState *state)
+			HTMLEngineSaveState *state,
+			gint requested_width)
 {
-	return (* HO_CLASS (self)->save_plain) (self, state);
+	return (* HO_CLASS (self)->save_plain) (self, state, requested_width);
 }
 
 gint
@@ -986,3 +1058,151 @@ html_object_get_length (HTMLObject *self)
 {
 	return (* HO_CLASS (self)->get_length) (self);
 }
+
+HTMLObject *
+html_object_next_by_type (HTMLObject *self, HTMLType t)
+{
+	HTMLObject *next;
+
+	g_assert (self);
+
+	next = self->next;
+	while (next && HTML_OBJECT_TYPE (next) != t)
+		next = next->next;
+
+	return next;
+}
+
+HTMLObject *
+html_object_prev_by_type (HTMLObject *self, HTMLType t)
+{
+	HTMLObject *prev;
+
+	g_assert (self);
+
+	prev = self->prev;
+	while (prev && HTML_OBJECT_TYPE (prev) != t)
+		prev = prev->prev;
+
+	return prev;
+}
+
+/* Movement functions */
+
+HTMLObject *
+html_object_next (HTMLObject *self, HTMLObject *child)
+{
+	return (* HO_CLASS (self)->next) (self, child);
+}
+
+HTMLObject *
+html_object_prev (HTMLObject *self, HTMLObject *child)
+{
+	return (* HO_CLASS (self)->prev) (self, child);
+}
+
+HTMLObject *
+html_object_head (HTMLObject *self)
+{
+	return (* HO_CLASS (self)->head) (self);
+}
+
+HTMLObject *
+html_object_tail (HTMLObject *self)
+{
+	return (* HO_CLASS (self)->tail) (self);
+}
+
+gboolean
+html_object_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
+{
+	gint len;
+
+	g_assert (self);
+	g_assert (cursor->object == self);
+
+	len = html_object_get_length (self);
+	if (cursor->offset < len) {
+		cursor->offset ++;
+		cursor->position ++;
+		return TRUE;
+	} else
+		return FALSE;
+}
+
+gboolean
+html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
+{
+	g_assert (self);
+	g_assert (cursor->object == self);
+
+	if (cursor->offset > 1 || (!html_object_prev_not_slave (self) && cursor->offset > 0)) {
+		cursor->offset --;
+		cursor->position --;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+/* movement on leafs */
+
+/* go up in tree so long as we can get object in neighborhood given by function next_fn */
+
+static HTMLObject *
+next_object_uptree (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *, HTMLObject *))
+{
+	HTMLObject *next = NULL;
+
+	while (obj->parent && !(next = (*next_fn) (obj->parent, obj)))
+		obj = obj->parent;
+
+	return next;
+}
+
+/* go down in tree to leaf in way given by down_fn children */
+
+static HTMLObject *
+move_object_downtree (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *))
+{
+	HTMLObject *down;
+
+	while ((down = (*down_fn) (obj)))
+		obj = down;
+
+	return obj;
+}
+
+static HTMLObject *
+move_object (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *, HTMLObject *), HTMLObject * (*down_fn ) (HTMLObject *))
+{
+	obj = next_object_uptree (obj, next_fn);
+	if (obj)
+		obj = move_object_downtree (obj, down_fn);
+	return obj;
+}
+
+
+HTMLObject *
+html_object_next_leaf (HTMLObject *self)
+{
+	return move_object (self, html_object_next, html_object_head);
+}
+
+HTMLObject *
+html_object_prev_leaf (HTMLObject *self)
+{
+	return move_object (self, html_object_prev, html_object_tail);
+}
+
+guint
+html_object_get_bytes (HTMLObject *self)
+{
+	return html_object_is_text (self) ? html_text_get_bytes (HTML_TEXT (self)) : html_object_get_length (self);
+}
+
+guint
+html_object_get_index (HTMLObject *self, guint offset)
+{
+	return html_object_is_text (self) ? html_text_get_index (HTML_TEXT (self), offset) : offset;
+}
