@@ -40,6 +40,7 @@ finalize (GtkObject *object)
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
+	html_font_manager_finalize (&painter->font_manager);
 
 	/* FIXME ownership of the color set?  */
 
@@ -59,11 +60,13 @@ finalize (GtkObject *object)
 DEFINE_UNIMPLEMENTED (begin);
 DEFINE_UNIMPLEMENTED (end);
 
+DEFINE_UNIMPLEMENTED (alloc_font);
+DEFINE_UNIMPLEMENTED (  ref_font);
+DEFINE_UNIMPLEMENTED (unref_font);
+
 DEFINE_UNIMPLEMENTED (alloc_color);
 DEFINE_UNIMPLEMENTED (free_color);
 
-DEFINE_UNIMPLEMENTED (set_font_style);
-DEFINE_UNIMPLEMENTED (get_font_style);
 DEFINE_UNIMPLEMENTED (calc_ascent);
 DEFINE_UNIMPLEMENTED (calc_descent);
 DEFINE_UNIMPLEMENTED (calc_text_width);
@@ -90,13 +93,16 @@ DEFINE_UNIMPLEMENTED (draw_background);
 DEFINE_UNIMPLEMENTED (get_pixel_size);
 
 static void
-init (GtkObject *object)
+init (GtkObject *object, HTMLPainterClass *real_klass)
 {
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
-
 	painter->color_set = html_colorset_new (NULL);
+
+	html_font_manager_init (&painter->font_manager, real_klass->alloc_font, real_klass->ref_font, real_klass->unref_font);
+	painter->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
+	painter->font_face = NULL;
 }
 
 static void
@@ -111,11 +117,13 @@ class_init (GtkObjectClass *object_class)
 	class->begin = (gpointer) begin_unimplemented;
 	class->end = (gpointer) end_unimplemented;
 
+	class->alloc_font = (gpointer) alloc_font_unimplemented;
+	class->ref_font   = (gpointer)   ref_font_unimplemented;
+	class->unref_font = (gpointer) unref_font_unimplemented;
+
 	class->alloc_color = (gpointer) alloc_color_unimplemented;
 	class->free_color = (gpointer) free_color_unimplemented;
 
-	class->set_font_style = (gpointer) set_font_style_unimplemented;
-	class->get_font_style = (gpointer) get_font_style_unimplemented;
 	class->calc_ascent = (gpointer) calc_ascent_unimplemented;
 	class->calc_descent = (gpointer) calc_descent_unimplemented;
 	class->calc_text_width = (gpointer) calc_text_width_unimplemented;
@@ -231,7 +239,7 @@ html_painter_set_font_style (HTMLPainter *painter,
 	g_return_if_fail (HTML_IS_PAINTER (painter));
 	g_return_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	(* HP_CLASS (painter)->set_font_style) (painter, font_style);
+	painter->font_style = font_style;
 }
 
 GtkHTMLFontStyle
@@ -240,43 +248,66 @@ html_painter_get_font_style (HTMLPainter *painter)
 	g_return_val_if_fail (painter != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), GTK_HTML_FONT_STYLE_DEFAULT);
 
-	return (* HP_CLASS (painter)->get_font_style) (painter);
+	return painter->font_style;
+}
+
+void
+html_painter_set_font_face (HTMLPainter *painter,
+			    HTMLFontFace *face)
+{
+	g_return_if_fail (painter != NULL);
+	g_return_if_fail (HTML_IS_PAINTER (painter));
+
+	if (!painter->font_face || !face || strcmp (painter->font_face, face)) {
+		if (painter->font_face)
+			g_free (painter->font_face);
+		painter->font_face = g_strdup (face);
+	}
+}
+
+gpointer
+html_painter_get_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style);
 }
 
 guint
 html_painter_calc_ascent (HTMLPainter *painter,
-			  GtkHTMLFontStyle font_style)
+			  GtkHTMLFontStyle font_style,
+			  HTMLFontFace *face)
 {
 	g_return_val_if_fail (painter != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
 	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
 
-	return (* HP_CLASS (painter)->calc_ascent) (painter, font_style);
+	return (* HP_CLASS (painter)->calc_ascent) (painter, font_style, face);
 }
 
 guint
 html_painter_calc_descent (HTMLPainter *painter,
-			   GtkHTMLFontStyle font_style)
+			   GtkHTMLFontStyle font_style,
+			   HTMLFontFace *face)
 {
 	g_return_val_if_fail (painter != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
 	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
 
-	return (* HP_CLASS (painter)->calc_descent) (painter, font_style);
+	return (* HP_CLASS (painter)->calc_descent) (painter, font_style, face);
 }
 
 guint
 html_painter_calc_text_width (HTMLPainter *painter,
 			      const gchar *text,
 			      guint len,
-			      GtkHTMLFontStyle font_style)
+			      GtkHTMLFontStyle font_style,
+			      HTMLFontFace *face)
 {
 	g_return_val_if_fail (painter != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
 	g_return_val_if_fail (text != NULL, 0);
 	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
 
-	return (* HP_CLASS (painter)->calc_text_width) (painter, text, len, font_style);
+	return (* HP_CLASS (painter)->calc_text_width) (painter, text, len, font_style, face);
 }
 
 
