@@ -30,6 +30,7 @@ typedef struct _HTMLPainter HTMLPainter;
 #include <gtk/gtk.h>
 
 #include "gtkhtmlfontstyle.h"
+#include "htmlfontmanager.h"
 #include "htmlcolorset.h"
 
 
@@ -50,24 +51,29 @@ typedef enum {
 struct _HTMLPainter {
 	GtkObject base;
 
-	HTMLColorSet *color_set;
+	HTMLFontManager     font_manager;
+	HTMLColorSet       *color_set;
+	HTMLFontFace       *font_face;
+	GtkHTMLFontStyle    font_style;
 };
 
 struct _HTMLPainterClass {
-	GtkObjectClass base;
+	GtkObjectClass   base;
 
 	void (* begin) (HTMLPainter *painter, int x1, int y1, int x2, int y2);
 	void (* end) (HTMLPainter *painter);
 
-	void (* alloc_color) (HTMLPainter *painter, GdkColor *color);
-	void (* free_color) (HTMLPainter *painter, GdkColor *color);
+	HTMLFontManagerAllocFont  alloc_font;
+	HTMLFontManagerRefFont      ref_font;
+	HTMLFontManagerUnrefFont  unref_font;
 
-	void (* set_font_style) (HTMLPainter *p, GtkHTMLFontStyle  f);
-	GtkHTMLFontStyle (* get_font_style) (HTMLPainter *p);
-	guint (* calc_ascent) (HTMLPainter *p, GtkHTMLFontStyle f);
-	guint (* calc_descent) (HTMLPainter *p, GtkHTMLFontStyle f);
+	void (* alloc_color) (HTMLPainter *painter, GdkColor *color);
+	void (* free_color)   (HTMLPainter *painter, GdkColor *color);
+
+	guint (* calc_ascent) (HTMLPainter *p, GtkHTMLFontStyle f, HTMLFontFace *face);
+	guint (* calc_descent) (HTMLPainter *p, GtkHTMLFontStyle f, HTMLFontFace *face);
 	guint (* calc_text_width) (HTMLPainter *p, const gchar *text, guint len,
-				   GtkHTMLFontStyle font_style);
+				   GtkHTMLFontStyle font_style, HTMLFontFace *face);
 
 	void (* set_pen) (HTMLPainter *painter, const GdkColor *color);
 	const GdkColor * (* get_black) (const HTMLPainter *painter);
@@ -126,17 +132,27 @@ const GdkColor *html_painter_get_default_highlight_foreground_color  (HTMLPainte
 const GdkColor *html_painter_get_black                               (const HTMLPainter *painter);
 
 /* Font handling.  */
+HTMLFontFace     *html_painter_find_font_face   (HTMLPainter *p,
+						 const gchar *families);
 void              html_painter_set_font_style   (HTMLPainter      *p,
 						 GtkHTMLFontStyle  f);
 GtkHTMLFontStyle  html_painter_get_font_style   (HTMLPainter      *p);
+void              html_painter_set_font_face    (HTMLPainter      *p,
+						 HTMLFontFace     *f);
+gpointer          html_painter_get_font         (HTMLPainter *painter,
+						 HTMLFontFace *face,
+						 GtkHTMLFontStyle style);
 guint             html_painter_calc_ascent      (HTMLPainter      *p,
-						 GtkHTMLFontStyle  f);
+						 GtkHTMLFontStyle  f,
+						 HTMLFontFace     *face);
 guint             html_painter_calc_descent     (HTMLPainter      *p,
-						 GtkHTMLFontStyle  f);
+						 GtkHTMLFontStyle  f,
+						 HTMLFontFace     *face);
 guint             html_painter_calc_text_width  (HTMLPainter      *p,
 						 const gchar      *text,
 						 guint             len,
-						 GtkHTMLFontStyle  font_style);
+						 GtkHTMLFontStyle  font_style,
+						 HTMLFontFace     *face);
 
 /* The actual paint operations.  */
 void  html_painter_set_pen               (HTMLPainter    *painter,
