@@ -20,7 +20,9 @@
 */
 
 #include <gnome.h>
+#include <ctype.h>
 #include <libgnomeprint/gnome-print.h>
+#include <unicode.h>
 
 #include "htmlprinter.h"
 
@@ -103,10 +105,17 @@ get_bmargin (HTMLPrinter *printer)
 #endif
 
 
-static void
-engine_coordinates_to_gnome_print (HTMLPrinter *printer,
-				   gint engine_x, gint engine_y,
-				   gdouble *print_x_return, gdouble *print_y_return)
+
+gdouble
+html_printer_scale_to_gnome_print (gint x)
+{
+	return SCALE_ENGINE_TO_GNOME_PRINT (x);
+}
+
+void
+html_printer_coordinates_to_gnome_print (HTMLPrinter *printer,
+					 gint engine_x, gint engine_y,
+					 gdouble *print_x_return, gdouble *print_y_return)
 {
 	gdouble print_x, print_y;
 
@@ -149,8 +158,6 @@ finalize (GtkObject *object)
 		gtk_object_unref (GTK_OBJECT (printer->print_context));
 	}
 
-	html_print_font_manager_destroy (printer->font_manager);
-
 	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -174,7 +181,7 @@ begin (HTMLPainter *painter,
 
 	gnome_print_gsave (pc);
 
-	engine_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
+	html_printer_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
 	printer_x2 = printer_x1 + SCALE_ENGINE_TO_GNOME_PRINT (x2);
 	printer_y2 = printer_y1 - SCALE_ENGINE_TO_GNOME_PRINT (y2);
 
@@ -250,49 +257,45 @@ get_black (const HTMLPainter *painter)
 }
 
 static void
-set_font_style (HTMLPainter *painter,
-		GtkHTMLFontStyle font_style)
+prepare_rectangle (HTMLPainter *painter, gint _x, gint _y, gint w, gint h)
 {
-	HTMLPrinter *printer;
-	GnomeFont *font;
+	GnomePrintContext *context = HTML_PRINTER (painter)->print_context;
+	gdouble x;
+	gdouble y;
+	gdouble width;
+	gdouble height;
 
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	width = SCALE_ENGINE_TO_GNOME_PRINT (w);
+	height = SCALE_ENGINE_TO_GNOME_PRINT (h);
 
-	font = html_print_font_manager_get_font (printer->font_manager, font_style);
-	gnome_print_setfont (printer->print_context, font);
+	html_printer_coordinates_to_gnome_print (HTML_PRINTER (painter), _x, _y, &x, &y);
 
-	printer->font_style = font_style;
-}
-
-static GtkHTMLFontStyle
-get_font_style (HTMLPainter *painter)
-{
-	HTMLPrinter *printer;
-
-	printer = HTML_PRINTER (painter);
-
-	return printer->font_style;
+	gnome_print_newpath (context);
+	gnome_print_moveto  (context, x, y);
+	gnome_print_lineto  (context, x + width, y);
+	gnome_print_lineto  (context, x + width, y - height);
+	gnome_print_lineto  (context, x, y - height);
+	gnome_print_lineto  (context, x, y);
 }
 
 static void
-do_rectangle (GnomePrintContext *context,
-	      gdouble x, gdouble y,
-	      gdouble width, gdouble height,
-	      gdouble line_width)
+do_rectangle (HTMLPainter *painter, gint x, gint y, gint w, gint h, gint lw)
 {
-	gnome_print_setlinewidth (context, line_width);
-
-	gnome_print_newpath (context);
-	gnome_print_moveto (context, x, y);
-	gnome_print_lineto (context, x + width, y);
-	gnome_print_lineto (context, x + width, y - height);
-	gnome_print_lineto (context, x, y - height);
-	gnome_print_lineto (context, x, y);
+	GnomePrintContext *context = HTML_PRINTER (painter)->print_context;
 
+	gnome_print_setlinewidth (context, SCALE_ENGINE_TO_GNOME_PRINT (lw) * PIXEL_SIZE);
+	prepare_rectangle (painter, x, y, w, h);
 	gnome_print_stroke (context);
 }
 
+static void
+set_clip_rectangle (HTMLPainter *painter,
+		    gint x, gint y,
+		    gint width, gint height)
+{
+	prepare_rectangle (painter, x, y, width, height);
+	gnome_print_clip (HTML_PRINTER (painter)->print_context);
+}
 
 /* HTMLPainter drawing functions.  */
 
@@ -308,8 +311,8 @@ draw_line (HTMLPainter *painter,
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->print_context != NULL);
 
-	engine_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
-	engine_coordinates_to_gnome_print (printer, x2, y2, &printer_x2, &printer_y2);
+	html_printer_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
+	html_printer_coordinates_to_gnome_print (printer, x2, y2, &printer_x2, &printer_y2);
 
 	gnome_print_setlinewidth (printer->print_context, PIXEL_SIZE);
 
@@ -325,21 +328,7 @@ draw_rect (HTMLPainter *painter,
 	   gint x, gint y,
 	   gint width, gint height)
 {
-	HTMLPrinter *printer;
-	double printer_x, printer_y;
-	double printer_width, printer_height;
-
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
-
-	printer_width = SCALE_ENGINE_TO_GNOME_PRINT (width);
-	printer_height = SCALE_ENGINE_TO_GNOME_PRINT (height);
-
-	engine_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
-
-	do_rectangle (printer->print_context,
-		      printer_x, printer_y, printer_width, printer_height,
-		      PIXEL_SIZE);
+	do_rectangle (painter, x, y, width, height, 1);
 }
 
 static void
@@ -349,21 +338,7 @@ draw_panel (HTMLPainter *painter,
 	    GtkHTMLEtchStyle inset,
 	    gint bordersize)
 {
-	HTMLPrinter *printer;
-	double printer_x, printer_y;
-	double printer_width, printer_height;
-
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
-
-	printer_width = SCALE_ENGINE_TO_GNOME_PRINT (width);
-	printer_height = SCALE_ENGINE_TO_GNOME_PRINT (height);
-
-	engine_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
-
-	do_rectangle (printer->print_context,
-		      printer_x, printer_y, printer_width, printer_height,
-		      SCALE_ENGINE_TO_GNOME_PRINT (bordersize) * PIXEL_SIZE);
+	do_rectangle (painter, x, y, width, height, bordersize);
 }
 
 static void
@@ -385,7 +360,7 @@ draw_background (HTMLPainter *painter,
 
 	width = SCALE_ENGINE_TO_GNOME_PRINT  (pix_width);
 	height = SCALE_ENGINE_TO_GNOME_PRINT (pix_height);
-	engine_coordinates_to_gnome_print (printer, ix, iy, &x, &y);
+	html_printer_coordinates_to_gnome_print (printer, ix, iy, &x, &y);
 
 	if (color) {
 		gnome_print_setrgbcolor (pc, color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
@@ -422,7 +397,7 @@ draw_pixmap (HTMLPainter *painter,
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
 
-	engine_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
+	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
 
 	print_scale_width = SCALE_ENGINE_TO_GNOME_PRINT (scale_width);
 	print_scale_height = SCALE_ENGINE_TO_GNOME_PRINT (scale_height);
@@ -486,7 +461,7 @@ fill_rect (HTMLPainter *painter,
 	printer_width = SCALE_ENGINE_TO_GNOME_PRINT (width);
 	printer_height = SCALE_ENGINE_TO_GNOME_PRINT (height);
 
-	engine_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
+	html_printer_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
 
 	gnome_print_newpath (printer->print_context);
 	gnome_print_moveto (printer->print_context, printer_x, printer_y);
@@ -504,30 +479,32 @@ draw_text (HTMLPainter *painter,
 	   const gchar *text,
 	   gint len)
 {
+	GnomeFont *font;
 	HTMLPrinter *printer;
-	GtkHTMLFontStyle font_style;
 	gdouble print_x, print_y;
 	gchar *text_tmp;
+	gint bytes;
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->print_context != NULL);
 
-	engine_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
+	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
 
 	gnome_print_newpath (printer->print_context);
 	gnome_print_moveto (printer->print_context, print_x, print_y);
 
 	/* Oh boy, this sucks so much.  The GnomePrint API could be improved to
            avoid this.  */
-	text_tmp = alloca (len + 1);
-	memcpy (text_tmp, text, len);
-	text_tmp[len] = '\0';
+	bytes = unicode_offset_to_index (text, len);
+	text_tmp = alloca (bytes + 1);
+	memcpy (text_tmp, text, bytes);
+	text_tmp [bytes] = '\0';
 
+	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
+	gnome_print_setfont (printer->print_context, font);
 	gnome_print_show (printer->print_context, text_tmp);
 
-	font_style = printer->font_style;
-	if (font_style & (GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
-		GnomeFont *font;
+	if (painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
 		double text_width;
 		double ascender, descender;
 		double y;
@@ -538,10 +515,8 @@ draw_text (HTMLPainter *painter,
 		gnome_print_setlinewidth (printer->print_context, 1.0);
 		gnome_print_setlinecap (printer->print_context, GDK_CAP_BUTT);
 
-		font = html_print_font_manager_get_font (printer->font_manager, font_style);
-
 		text_width = gnome_font_get_width_string_n (font, text, len);
-		if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
+		if (painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
 			descender = gnome_font_get_descender (font);
 			y = print_y + gnome_font_get_underline_position (font);
 
@@ -553,7 +528,7 @@ draw_text (HTMLPainter *painter,
 			gnome_print_stroke (printer->print_context);
 		}
 
-		if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
+		if (painter->font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
 			ascender = gnome_font_get_ascender (font);
 			y = print_y + ascender / 2.0;
 			gnome_print_newpath (printer->print_context);
@@ -583,7 +558,8 @@ draw_shade_line (HTMLPainter *painter,
 
 static guint
 calc_ascent (HTMLPainter *painter,
-	     GtkHTMLFontStyle style)
+	     GtkHTMLFontStyle style,
+	     HTMLFontFace *face)
 {
 	HTMLPrinter *printer;
 	GnomeFont *font;
@@ -592,7 +568,7 @@ calc_ascent (HTMLPainter *painter,
 	printer = HTML_PRINTER (painter);
 	g_return_val_if_fail (printer->print_context != NULL, 0);
 
-	font = html_print_font_manager_get_font (printer->font_manager, style);
+	font = html_painter_get_font (painter, face, style);
 	g_return_val_if_fail (font != NULL, 0);
 
 	ascender = gnome_font_get_ascender (font) * SPACING_FACTOR;
@@ -601,7 +577,8 @@ calc_ascent (HTMLPainter *painter,
 
 static guint
 calc_descent (HTMLPainter *painter,
-	      GtkHTMLFontStyle style)
+	      GtkHTMLFontStyle style,
+	      HTMLFontFace *face)
 {
 	HTMLPrinter *printer;
 	GnomeFont *font;
@@ -610,7 +587,7 @@ calc_descent (HTMLPainter *painter,
 	printer = HTML_PRINTER (painter);
 	g_return_val_if_fail (printer->print_context != NULL, 0);
 
-	font = html_print_font_manager_get_font (printer->font_manager, style);
+	font = html_painter_get_font (painter, face, style);
 	g_return_val_if_fail (font != NULL, 0);
 
 	descender = gnome_font_get_descender (font) * SPACING_FACTOR;
@@ -621,7 +598,8 @@ static guint
 calc_text_width (HTMLPainter *painter,
 		 const gchar *text,
 		 guint len,
-		 GtkHTMLFontStyle style)
+		 GtkHTMLFontStyle style,
+		 HTMLFontFace *face)
 {
 	HTMLPrinter *printer;
 	GnomeFont *font;
@@ -630,10 +608,11 @@ calc_text_width (HTMLPainter *painter,
 	printer = HTML_PRINTER (painter);
 	g_return_val_if_fail (printer->print_context != NULL, 0);
 
-	font = html_print_font_manager_get_font (printer->font_manager, style);
+	font = html_painter_get_font (painter, face, style);
 	g_return_val_if_fail (font != NULL, 0);
 
 	width = gnome_font_get_width_string_n (font, text, len);
+
 	return SCALE_GNOME_PRINT_TO_ENGINE (width);
 }
 
@@ -643,19 +622,38 @@ get_pixel_size (HTMLPainter *painter)
 	return SCALE_GNOME_PRINT_TO_ENGINE (PIXEL_SIZE);
 }
 
+static gpointer
+alloc_font (gchar *face, gdouble size, GtkHTMLFontStyle style)
+{
+	GnomeFontWeight weight;
+	gboolean italic;
+	gchar *s = face;
+
+	weight = (style & GTK_HTML_FONT_STYLE_BOLD) ? GNOME_FONT_BOLD : GNOME_FONT_BOOK;
+	italic = (style & GTK_HTML_FONT_STYLE_ITALIC);
+
+	/* gnome-print is case sensitive - need to be fixed */
+	if (s && *s) {
+		/* capitalize */
+		*s = toupper (*s);
+		s++;
+		while (*s) {
+			*s = tolower (*s);
+			s++;
+		}
+	}
+
+	return gnome_font_new_closest ((face) ? face : "Courier", weight, italic, size);
+}
+
 
 static void
 init (GtkObject *object)
 {
 	HTMLPrinter *printer;
 
-	printer = HTML_PRINTER (object);
-
+	printer                = HTML_PRINTER (object);
 	printer->print_context = NULL;
-
-	printer->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
-
-	printer->font_manager = html_print_font_manager_new ();
 }
 
 static void
@@ -669,10 +667,11 @@ class_init (GtkObjectClass *object_class)
 
 	painter_class->begin = begin;
 	painter_class->end = end;
+	painter_class->alloc_font = alloc_font;
+	painter_class->ref_font   = (HTMLFontManagerRefFont)   gtk_object_ref;
+	painter_class->unref_font = (HTMLFontManagerUnrefFont) gtk_object_unref;	
 	painter_class->alloc_color = alloc_color;
 	painter_class->free_color = free_color;
-	painter_class->set_font_style = set_font_style;
-	painter_class->get_font_style = get_font_style;
 	painter_class->calc_ascent = calc_ascent;
 	painter_class->calc_descent = calc_descent;
 	painter_class->calc_text_width = calc_text_width;
@@ -688,6 +687,7 @@ class_init (GtkObjectClass *object_class)
 	painter_class->draw_shade_line = draw_shade_line;
 	painter_class->draw_background = draw_background;
 	painter_class->get_pixel_size = get_pixel_size;
+	painter_class->set_clip_rectangle = set_clip_rectangle;
 
 	parent_class = gtk_type_class (html_painter_get_type ());
 }
