@@ -53,7 +53,21 @@
 HTMLTableClass html_table_class;
 static HTMLObjectClass *parent_class = NULL;
 
+static void alloc_cell (HTMLTable *table, gint r, gint c);
+static void set_cell   (HTMLTable *table, gint r, gint c, HTMLTableCell *cell);
+static void do_cspan   (HTMLTable *table, gint row, gint col, HTMLTableCell *cell);
+static void do_rspan   (HTMLTable *table, gint row);
+
 
+
+static gboolean
+invalid_cell (HTMLTable *table, gint r, gint c)
+{
+	return (table->cells[r][c] == NULL
+		|| (c < table->totalCols - 1 && table->cells[r][c] == table->cells[r][c+1])
+		|| (r < table->totalRows - 1 && table->cells[r][c] == table->cells[r+1][c]));
+}
+
 /* HTMLObject methods.  */
 
 static void
@@ -139,42 +153,59 @@ forall (HTMLObject *self,
 
 
 static void
-add_columns (HTMLTable *table, gint num)
+previous_rows_do_cspan (HTMLTable *table, gint c)
 {
-	HTMLTableCell **newCells;
-	gint r;
-
-	for (r = 0; r < table->allocRows; r++) {
-		newCells = g_new (HTMLTableCell *, table->totalCols + num);
-		memcpy (newCells, table->cells[r], table->totalCols * sizeof (HTMLTableCell *));
-		memset (newCells + table->totalCols, 0, num * sizeof (HTMLTableCell *));
-		g_free (table->cells[r]);
-		table->cells[r] = newCells;
-	}
-
-	table->totalCols += num;
-	
+	gint i;
+	if (c)
+		for (i=0; i < table->totalRows - 1; i++)
+			if (table->cells [i][c - 1])
+				do_cspan (table, i, c, table->cells [i][c - 1]);
 }
 
 static void
-add_rows (HTMLTable *table, gint num)
+expand_columns (HTMLTable *table, gint num)
 {
-	HTMLTableCell ***newRows = g_new (HTMLTableCell **, table->allocRows + num);
 	gint r;
 
-	memcpy (newRows, table->cells, table->allocRows * sizeof (HTMLTableCell **));
+	for (r = 0; r < table->allocRows; r++) {
+		table->cells [r] = g_renew (HTMLTableCell *, table->cells [r], table->totalCols + num);
+		memset (table->cells [r] + table->totalCols, 0, num * sizeof (HTMLTableCell *));
+	}
+	table->totalCols += num;
+}
 
-	g_free (table->cells);
-	table->cells = newRows;
+static void
+inc_columns (HTMLTable *table, gint num)
+{
+	expand_columns (table, num);
+	previous_rows_do_cspan (table, table->totalCols - num);
+}
+
+static void
+expand_rows (HTMLTable *table, gint num)
+{
+	gint r;
+
+	table->cells = g_renew (HTMLTableCell **, table->cells, table->allocRows + num);
 
 	for (r = table->allocRows; r < table->allocRows + num; r++) {
-		table->cells[r] = g_new (HTMLTableCell *, table->totalCols);
-		memset (table->cells[r], 0, table->totalCols * sizeof (HTMLTableCell *));
+		table->cells [r] = g_new (HTMLTableCell *, table->totalCols);
+		memset (table->cells [r], 0, table->totalCols * sizeof (HTMLTableCell *));
 	}
 
 	table->allocRows += num;
 }
 
+static void
+inc_rows (HTMLTable *table, gint num)
+{
+	if (table->totalRows + num > table->allocRows)
+		expand_rows (table, num + MAX (10, table->allocRows >> 2));
+	table->totalRows += num;
+	if (table->totalRows - num > 0)
+		do_rspan (table, table->totalRows - num);
+}
+
 static void
 calc_col_info (HTMLTable *table,
 	       HTMLPainter *painter)
@@ -430,9 +461,7 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 						continue;
 				}
 
-				minWidth = (COLUMN_OPT (table, c + 1)
-					    - COLUMN_OPT (table, c + 1 - cell->cspan));
-
+				minWidth  = (COLUMN_OPT (table, c + 1) - COLUMN_OPT (table, cell->col));
 				prefWidth = (cell->fixed_width
 					     + pixel_size * (table->padding * 2
 							     + table->spacing
@@ -451,16 +480,14 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 						COLUMN_OPT (table, c1) += addSize;
 				}
 				else {
-					gint c_b = c + 1 - cell->cspan;
-
 					/* Some end-conditions are required here to prevent looping */
-					if (c_b < c_start)
+					if (cell->col < c_start)
 						continue;
-					if ((c_b == c_start) && (c == c_end))
+					if ((cell->col == c_start) && (c == c_end))
 						continue;
 
 					/* Scale the columns covered by 'cell' first */
-					scale_columns (table, painter, c_b, c, addSize);
+					scale_columns (table, painter, cell->col, c, addSize);
 				}
 			}
 		}
@@ -542,14 +569,12 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 				/* Only cells with a colspan which fits
 				   within c_begin .. c_start */
 				if (cell->cspan > 1) {
-					gint c_b = c + 1 - cell->cspan;
-
 					if (colspan == 0)
 						continue;
 
-					if (c_b < c_start)
+					if (cell->col < c_start)
 						continue;
-					if ((c_b == c_start) && (c == c_end))
+					if ((cell->col == c_start) && (c == c_end))
 						continue;
 				}
 				else {
@@ -558,7 +583,7 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 				}
 
 				minWidth = COLUMN_OPT (table, c + 1) - 
-					COLUMN_OPT (table, c + 1 - cell->cspan);
+					COLUMN_OPT (table, cell->col);
 
 				/* The cast to `gdouble' is to avoid overflow (eg. when
 				   printing).  */
@@ -634,7 +659,7 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 
 			if (cell == 0)
 				continue;
-			if (r < table->totalRows && table->cells[r + 1][c] == cell)
+			if (r < table->totalRows - 1 && table->cells[r + 1][c] == cell)
 				continue;
 
 			if (HTML_OBJECT(cell)->flags
@@ -670,7 +695,7 @@ scale_columns (HTMLTable *table, HTMLPainter *painter,
 								 + borderExtra));
 			}
 			
-			prefCellWidth = prefCellWidth / cell->cspan;
+			prefCellWidth = prefCellWidth / MIN (cell->cspan, table->totalCols - cell->col);
 
 			if (prefCellWidth > prefColumnWidth [c])
 				prefColumnWidth [c] = prefCellWidth;
@@ -746,7 +771,7 @@ calc_column_widths (HTMLTable *table,
 		    HTMLPainter *painter)
 {
 	gint r, c, i;
-	gint indx, borderExtra = (table->border == 0) ? 0 : 1;
+	gint borderExtra = (table->border == 0) ? 0 : 1;
 	gint pixel_size;
 
 	pixel_size = html_painter_get_pixel_size (painter);
@@ -756,10 +781,10 @@ calc_column_widths (HTMLTable *table,
 		COLUMN_TYPE (table, i) = COLUMN_TYPE_VARIABLE;
 
 	g_array_set_size (table->columnPos, table->totalCols + 1);
-	COLUMN_POS (table, 0) = table->border + table->spacing;
+	COLUMN_POS (table, 0) = pixel_size * (table->border + table->spacing);
 	
 	g_array_set_size (table->columnPrefPos, table->totalCols + 1);
-	COLUMN_PREF_POS (table, 0) = table->border + table->spacing;
+	COLUMN_PREF_POS (table, 0) = pixel_size * (table->border + table->spacing);
 
 	g_array_set_size (table->colSpan, table->totalCols + 1);
 	for (i = 0; i < table->colSpan->len; i++)
@@ -783,11 +808,8 @@ calc_column_widths (HTMLTable *table,
 			    table->cells[r + 1][c] == cell)
 				continue;
 
-			if ((indx = c - cell->cspan + 1) < 0)
-				indx = 0;
-
 			/* calculate minimum pos */
-			colPos = (COLUMN_POS (table, indx)
+			colPos = (COLUMN_POS (table, cell->col)
 				  + html_object_calc_min_width (HTML_OBJECT (cell), painter)
 				  + pixel_size * (table->padding * 2
 						  + table->spacing
@@ -804,7 +826,7 @@ calc_column_widths (HTMLTable *table,
 			if (HTML_OBJECT (cell)->percent > 0) {
 				/* The cast to `gdouble' is to avoid overflow (eg. when
 				   printing).  */
-				colPos = (COLUMN_PREF_POS (table, indx)
+				colPos = (COLUMN_PREF_POS (table, cell->col)
 					  + ((gdouble) HTML_OBJECT (table)->max_width
 					     * HTML_OBJECT (cell)->percent / 100)
 					  + pixel_size * (table->padding * 2
@@ -816,7 +838,7 @@ calc_column_widths (HTMLTable *table,
 			}
 			else if (HTML_OBJECT (cell)->flags
 				 & HTML_OBJECT_FLAG_FIXEDWIDTH) {
-				colPos = (COLUMN_PREF_POS (table, indx)
+				colPos = (COLUMN_PREF_POS (table, cell->col)
 					  + HTML_OBJECT (cell)->width
 					  + pixel_size * (table->padding * 2
 							  + table->spacing
@@ -828,7 +850,7 @@ calc_column_widths (HTMLTable *table,
 			else {
 				colPos = html_object_calc_preferred_width (HTML_OBJECT (cell),
 									   painter);
-				colPos += (COLUMN_PREF_POS (table, indx)
+				colPos += (COLUMN_PREF_POS (table, cell->col)
 					   + pixel_size * (table->padding * 2
 							   + table->spacing
 							   + borderExtra));
@@ -855,14 +877,14 @@ calc_row_heights (HTMLTable *table,
 		  HTMLPainter *painter)
 {
 	gint r, c;
-	gint rowPos, indx, borderExtra = table->border ? 1: 0;
+	gint rowPos, borderExtra = table->border ? 1: 0;
 	HTMLTableCell *cell;
 	gint pixel_size;
 
 	pixel_size = html_painter_get_pixel_size (painter);
 
 	g_array_set_size (table->rowHeights, table->totalRows + 1);
-	ROW_HEIGHT (table, 0) = table->border + table->spacing;
+	ROW_HEIGHT (table, 0) = pixel_size *(table->border + table->spacing);
 
 	for (r = 0; r < table->totalRows; r++) {
 		ROW_HEIGHT (table, r + 1) = 0;
@@ -876,10 +898,7 @@ calc_row_heights (HTMLTable *table,
 			    table->cells[r + 1][c] == cell)
 				continue;
 
-			if ((indx = r - cell->rspan + 1) < 0)
-				indx = 0;
-
-			rowPos = (ROW_HEIGHT (table, indx) + 
+			rowPos = (ROW_HEIGHT (table, cell->row) + 
 				  HTML_OBJECT (cell)->ascent +
 				  HTML_OBJECT (cell)->descent
 				  + (pixel_size * (table->spacing
@@ -904,7 +923,7 @@ optimize_cell_width (HTMLTable *table,
 	addSize = 0;
 	if (width > COLUMN_POS (table, table->totalCols)) {
 		/* We have some space to spare */
-		addSize = width - COLUMN_POS (table, table->totalCols);
+		addSize = width - COLUMN_POS (table, table->totalCols) + html_painter_get_pixel_size (painter) * table->border;
 
 		if ((HTML_OBJECT (table)->percent <= 0)
 		    && (!(HTML_OBJECT (table)->flags
@@ -924,35 +943,66 @@ optimize_cell_width (HTMLTable *table,
 }
 
 static void
-set_cells (HTMLTable *table, gint r, gint c, HTMLTableCell *cell)
+do_cspan (HTMLTable *table, gint row, gint col, HTMLTableCell *cell)
 {
-	gint endRow = r + cell->rspan;
-	gint endCol = c + cell->cspan;
-	gint tc;
+	gint i;
 
-	g_return_if_fail (HTML_OBJECT (cell)->parent == NULL);
+	g_assert (cell);
+	g_assert (cell->col <= col);
 
-	HTML_OBJECT (cell)->parent = HTML_OBJECT (table);
-	
-	if (endCol > table->totalCols)
-		add_columns (table, endCol - table->totalCols);
+	for (i=col - cell->col; i < cell->cspan && cell->col + i < table->totalCols; i++)
+		set_cell (table, row, cell->col + i, cell);
+}
 
-	if (endRow >= table->allocRows)
-		add_rows (table, endRow - table->allocRows + 10);
+static void
+prev_col_do_cspan (HTMLTable *table, gint row)
+{
+	g_assert (row >= 0);
 
-	if (endRow > table->totalRows)
-		table->totalRows = endRow;
-
-	for (; r < endRow; r++) {
-		for (tc = c; tc < endCol ; tc++) {
-			if (table->cells[r][tc])
-				html_table_cell_unlink (table->cells[r][tc]);
-			table->cells[r][tc] = cell;
-			html_table_cell_link (cell);
-		}
+	/* add previous column cell which has cspan > 1 */
+	while (table->col < table->totalCols && table->cells [row][table->col] != 0) {
+		alloc_cell (table, row, table->col + table->cells [row][table->col]->cspan);
+		do_cspan (table, row, table->col + 1, table->cells [row][table->col]);
+		table->col += (table->cells [row][table->col])->cspan;
 	}
 }
 
+static void
+do_rspan (HTMLTable *table, gint row)
+{
+	gint i;
+
+	g_assert (row > 0);
+
+	for (i=0; i<table->totalCols; i++)
+		if (table->cells [row - 1][i]
+		    && (table->cells [row - 1][i])->row + (table->cells [row - 1][i])->rspan
+		    > row) {
+			set_cell (table, table->row, i, table->cells [table->row - 1][i]);
+			do_cspan (table, table->row, i + 1, table->cells [table->row -1][i]);
+		}
+}
+
+static void
+set_cell (HTMLTable *table, gint r, gint c, HTMLTableCell *cell)
+{
+	if (!table->cells [r][c]) {
+		table->cells [r][c] = cell;
+		html_table_cell_link (cell);
+		HTML_OBJECT (cell)->parent = HTML_OBJECT (table);
+	}
+}
+
+static void
+alloc_cell (HTMLTable *table, gint r, gint c)
+{
+	if (c >= table->totalCols)
+		inc_columns (table, c + 1 - table->totalCols);
+
+	if (r >= table->totalRows)
+		inc_rows (table, r + 1 - table->totalRows);
+}
+
 
 /* HTMLObject methods.  */
 
@@ -960,7 +1010,7 @@ static gboolean
 calc_size (HTMLObject *o,
 	   HTMLPainter *painter)
 {
-	gint indx, w;
+	gint w;
 	gint old_width, old_ascent, old_descent;
 	gint pixel_size;
 	gint r, c;
@@ -996,7 +1046,7 @@ calc_size (HTMLObject *o,
 
 	if (o->percent == 0 && ! (o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH)) {
 		o->width = COLUMN_OPT (table, table->totalCols) + pixel_size * table->border;
-		available_width = o->max_width;
+		available_width = o->max_width - 2 * pixel_size * table->border;
 	} else {
 		if (o->percent != 0) {
 			/* The cast to `gdouble' is to avoid overflow (eg. when
@@ -1005,11 +1055,11 @@ calc_size (HTMLObject *o,
 		} else /* if (o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH) */ {
 			o->width = table->specified_width * pixel_size;
 		}
-		available_width = o->width;
+		available_width = o->width - 2 * pixel_size * table->border;
 	}
 		
 	/* Attempt to get sensible cell widths */
-	optimize_cell_width (table, painter, available_width);
+	optimize_cell_width (table, painter, available_width < 0 ? 0 : available_width);
 
 	for (r = 0; r < table->totalRows; r++) {
 		for (c = 0; c < table->totalCols; c++) {
@@ -1022,10 +1072,7 @@ calc_size (HTMLObject *o,
 			    cell == table->cells [r + 1][c])
 				continue;
 
-			if ((indx = c - cell->cspan + 1) < 0)
-				indx = 0;
-
-			w = (COLUMN_OPT (table, c + 1) - COLUMN_OPT (table, indx)
+			w = (COLUMN_OPT (table, c + 1) - COLUMN_OPT (table, cell->col)
 			     - pixel_size * (table->spacing + table->padding * 2));
 
 			html_object_set_max_width (HTML_OBJECT (cell), painter, w);
@@ -1061,16 +1108,10 @@ calc_size (HTMLObject *o,
 			    cell == table->cells[r + 1][c])
 				continue;
 
-			if ((indx = c - cell->cspan + 1) < 0)
-				indx = 0;
-
-			HTML_OBJECT (cell)->x = COLUMN_OPT (table, indx);
+			HTML_OBJECT (cell)->x = COLUMN_OPT (table, cell->col);
 			HTML_OBJECT (cell)->y = HTML_OBJECT (table)->ascent - HTML_OBJECT (cell)->descent;
-			if ((indx = r - cell->rspan + 1) < 0) 
-				indx = 0;
-			
-			cellHeight = (ROW_HEIGHT (table, r + 1) - ROW_HEIGHT (table, indx)
-				      - pixel_size * (table->spacing));
+			cellHeight            = (ROW_HEIGHT (table, r + 1) - ROW_HEIGHT (table, cell->row)
+						 - pixel_size * (table->spacing));
 			html_object_set_max_ascent (HTML_OBJECT (cell), painter, cellHeight);
 		}
 
@@ -1094,6 +1135,47 @@ calc_size (HTMLObject *o,
 	return FALSE;
 }
 
+#define NEW_INDEX(l,h) ((l+h) >> 1);
+#define ARR(i) g_array_index (a, gint, i-1)
+
+static gint
+bin_search_eq_or_lower_index (GArray *a, gint range, gint val, gint offset)
+{
+	gint l,h;
+	gint i;
+
+	l = offset ? offset + 1 : 1;
+	h = range;
+	i = NEW_INDEX (l, h);
+
+	while (l < h && val != ARR (i)) {
+		if (val < ARR (i))
+			h = i - 1;
+		else
+			l = i + 1;
+		i = NEW_INDEX (l, h);
+	}
+
+	if (i && val == ARR (i))
+		return i - 1;
+
+	if (val < ARR (l) && l > 1)
+		return l - 2;
+	else
+		return l - 1;
+}
+
+static void
+get_bounds (HTMLTable *table, gint x, gint y, gint width, gint height, gint *sc, gint *ec, gint *sr, gint *er)
+{
+	*sr = bin_search_eq_or_lower_index (table->rowHeights, table->totalRows + 1, y, 0);
+	*er = MIN (bin_search_eq_or_lower_index (table->rowHeights, table->totalRows + 1, y + height, *sr) + 1,
+		   table->totalRows);
+	*sc = bin_search_eq_or_lower_index (table->columnOpt, table->totalCols + 1, x, 0);
+	*ec = MIN (bin_search_eq_or_lower_index (table->columnOpt, table->totalCols + 1, x + width, *sc) + 1,
+		   table->totalCols);
+}
+
 static void
 draw (HTMLObject *o,
       HTMLPainter *p, 
@@ -1103,9 +1185,8 @@ draw (HTMLObject *o,
 {
 	HTMLTableCell *cell;
 	HTMLTable *table = HTML_TABLE (o);
-	gint cindx, rindx;
 	gint pixel_size;
-	gint r, c;
+	gint r, c, start_row, end_row, start_col, end_col;
 	ArtIRect paint;
 
 	html_object_calc_intersection (o, &paint, x, y, width, height);
@@ -1114,20 +1195,20 @@ draw (HTMLObject *o,
 
 	pixel_size = html_painter_get_pixel_size (p);
 	
-
 	tx += o->x;
 	ty += o->y - o->ascent;
 
 	/* Draw the cells */
-	for (r = 0; r < table->totalRows; r++) {
-		for (c = 0; c < table->totalCols; c++) {
+	get_bounds (table, x, y, width, height, &start_col, &end_col, &start_row, &end_row);
+	for (r = start_row; r < end_row; r++) {
+		for (c = start_col; c < end_col; c++) {
 			cell = table->cells[r][c];
 
 			if (cell == NULL)
 				continue;
-			if (c < table->totalCols - 1 && cell == table->cells[r][c + 1])
+			if (c < end_col - 1 && cell == table->cells [r][c + 1])
 				continue;
-			if (r < table->totalRows - 1 && table->cells[r + 1][c] == cell)
+			if (r < end_row - 1 && table->cells [r + 1][c] == cell)
 				continue;
 
 			html_object_draw (HTML_OBJECT (cell), p, 
@@ -1154,30 +1235,25 @@ draw (HTMLObject *o,
 					 pixel_size * table->border);
 		
 		/* Draw borders around each cell */
-		for (r = 0; r < table->totalRows; r++) {
-			for (c = 0; c < table->totalCols; c++) {
+		for (r = start_row; r < end_row; r++) {
+			for (c = start_col; c < end_col; c++) {
 				if ((cell = table->cells[r][c]) == 0)
 					continue;
-				if (c < table->totalCols - 1 &&
+				if (c < end_col - 1 &&
 				    cell == table->cells[r][c + 1])
 					continue;
-				if (r < table->totalRows - 1 &&
+				if (r < end_row - 1 &&
 				    table->cells[r + 1][c] == cell)
 					continue;
 
-				if ((cindx = c - cell->cspan + 1) < 0)
-					cindx = 0;
-				if ((rindx = r - cell->rspan + 1) < 0)
-					rindx = 0;
-				
 				html_painter_draw_panel (p,
-							 tx + COLUMN_OPT (table, cindx),
-							 ty + ROW_HEIGHT (table, rindx) + capOffset,
+							 tx + COLUMN_OPT (table, cell->col),
+							 ty + ROW_HEIGHT (table, cell->row) + capOffset,
 							 (COLUMN_OPT (table, c + 1)
-							  - COLUMN_OPT (table, cindx)
+							  - COLUMN_OPT (table, cell->col)
 							  - pixel_size * table->spacing),
 							 (ROW_HEIGHT (table, r + 1)
-							  - ROW_HEIGHT (table, rindx)
+							  - ROW_HEIGHT (table, cell->row)
 							  - pixel_size * table->spacing),
 							 GTK_HTML_ETCH_IN, 1);
 						      
@@ -1286,7 +1362,7 @@ check_point (HTMLObject *self,
 	HTMLTableCell *cell;
 	HTMLObject *obj;
 	HTMLTable *table;
-	unsigned int r, c;
+	gint r, c, start_row, end_row, start_col, end_col;
 
 	if (x < self->x || x > self->x + self->width
 	    || y > self->y + self->descent || y < self->y - self->ascent)
@@ -1297,15 +1373,16 @@ check_point (HTMLObject *self,
 	x -= self->x;
 	y -= self->y - self->ascent;
 
-	for (r = 0; r < table->totalRows; r++) {
+	get_bounds (table, x, y, 1, 1, &start_col, &end_col, &start_row, &end_row);
+	for (r = start_row; r < end_row; r++) {
 		for (c = 0; c < table->totalCols; c++) {
 			cell = table->cells[r][c];
 			if (cell == NULL)
 				continue;
 
-			if (c < table->totalCols - 1 && cell == table->cells[r][c+1])
+			if (c < end_col - 1 && cell == table->cells[r][c+1])
 				continue;
-			if (r < table->totalRows - 1 && table->cells[r+1][c] == cell)
+			if (r < end_row - 1 && table->cells[r+1][c] == cell)
 				continue;
 
 			obj = html_object_check_point (HTML_OBJECT (cell),
@@ -1409,6 +1486,119 @@ fit_line (HTMLObject *o,
 	return HTML_FIT_COMPLETE;
 }
 
+static void
+append_selection_string (HTMLObject *self,
+			 GString *buffer)
+{
+	HTMLTable *table;
+	HTMLTableCell *cell;
+	gboolean tab;
+	gint r, c, len, rlen, tabs;
+
+	table = HTML_TABLE (self);
+	for (r = 0; r < table->totalRows; r++) {
+		tab = FALSE;
+		tabs = 0;
+		rlen = buffer->len;
+		for (c = 0; c < table->totalCols; c++) {
+			if ((cell = table->cells[r][c]) == 0)
+				continue;
+			if (c < table->totalCols - 1 &&
+			    cell == table->cells[r][c + 1])
+				continue;
+			if (r < table->totalRows - 1 &&
+			    table->cells[r + 1][c] == cell)
+				continue;
+			if (tab) {
+				g_string_append_c (buffer, '\t');
+				tabs++;
+			}
+			len = buffer->len;
+			html_object_append_selection_string (HTML_OBJECT (cell), buffer);
+			/* remove last \n from added text */
+			if (buffer->len != len && buffer->str [buffer->len-1] == '\n')
+				g_string_truncate (buffer, buffer->len - 1);
+			tab = TRUE;
+		}
+		if (rlen + tabs == buffer->len)
+			g_string_truncate (buffer, rlen);
+		else if (r + 1 < table->totalRows)
+			g_string_append_c (buffer, '\n');
+	}
+}
+
+static HTMLObject *
+head (HTMLObject *self)
+{
+	HTMLTable *table;
+	gint r, c;
+
+	table = HTML_TABLE (self);
+	for (r = 0; r < table->totalRows; r++)
+		for (c = 0; c < table->totalCols; c++) {
+			if (invalid_cell (table, r, c))
+				continue;
+			return HTML_OBJECT (table->cells [r][c]);
+		}
+	return NULL;
+}
+
+static HTMLObject *
+tail (HTMLObject *self)
+{
+	HTMLTable *table;
+	gint r, c;
+
+	table = HTML_TABLE (self);
+	for (r = table->totalRows - 1; r >= 0; r--)
+		for (c = table->totalCols - 1; c >= 0; c--) {
+			if (invalid_cell (table, r, c))
+				continue;
+			return HTML_OBJECT (table->cells [r][c]);
+		}
+	return NULL;
+}
+
+static HTMLObject *
+next (HTMLObject *self, HTMLObject *child)
+{
+	HTMLTable *table;
+	gint r, c;
+
+	table = HTML_TABLE (self);
+	r = HTML_TABLE_CELL (child)->row;
+	c = HTML_TABLE_CELL (child)->col;
+	for (c++; r < table->totalRows; r++) {
+		for (; c < table->totalCols; c++) {
+			if (invalid_cell (table, r, c))
+				continue;
+			return HTML_OBJECT (table->cells [r][c]);
+		}
+		c = 0;
+	}
+	return NULL;
+}
+
+static HTMLObject *
+prev (HTMLObject *self, HTMLObject *child)
+{
+	HTMLTable *table;
+	gint r, c;
+
+	table = HTML_TABLE (self);
+	r = HTML_TABLE_CELL (child)->row;
+	c = HTML_TABLE_CELL (child)->col;
+	for (c--; r >= 0; r--) {
+		for (; c >=0; c--) {
+			if (invalid_cell (table, r, c))
+				continue;
+			return HTML_OBJECT (table->cells [r][c]);
+		}
+		c = table->totalCols-1;
+	}
+	return NULL;
+}
+
 
 void
 html_table_type_init (void)
@@ -1441,6 +1631,11 @@ html_table_class_init (HTMLTableClass *klass,
 	object_class->forall = forall;
 	object_class->search = search;
 	object_class->fit_line = fit_line;
+	object_class->append_selection_string = append_selection_string;
+	object_class->head = head;
+	object_class->tail = tail;
+	object_class->next = next;
+	object_class->prev = prev;
 
 	parent_class = &html_object_class;
 }
@@ -1512,14 +1707,21 @@ html_table_new (gint width, gint percent,
 }
 
 
+
 void
 html_table_add_cell (HTMLTable *table, HTMLTableCell *cell)
 {
-	while (table->col < table->totalCols && 
-	       table->cells[table->row][table->col] != 0)
+	alloc_cell (table, table->row, table->col);
+	prev_col_do_cspan (table, table->row);
+
+	/* look for first free cell */
+	while (table->cells [table->row][table->col] && table->col < table->totalCols)
 		table->col++;
 
-	set_cells (table, table->row, table->col, cell);
+	alloc_cell (table, table->row, table->col);
+	set_cell (table, table->row, table->col, cell);
+	html_table_cell_set_position (cell, table->row, table->col);
+	do_cspan(table, table->row, table->col, cell);
 }
 
 void
@@ -1531,22 +1733,18 @@ html_table_start_row (HTMLTable *table)
 void
 html_table_end_row (HTMLTable *table)
 {
-	while (table->col < table->totalCols && 
-	       table->cells [table->row][table->col] != 0)
-		table->col++;
-	if (table->col)
-		table->row++;
+	gint i;
+
+	for (i=0; i < table->totalCols; i++)
+		if (table->cells [table->row][table->col]) {
+			table->row++;
+			break;
+		}
 }
 
 void
 html_table_end_table (HTMLTable *table)
 {
-	/* FIXME this should not be needed, as the functions are already called
-           by `calc_size()'.  */
-#if 0
-	calc_col_info (table);
-	calc_column_widths (table);
-#endif
 }
 
 gint
