@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <unicode.h>
 
 #include "htmltext.h"
 #include "htmlclueflow.h"
@@ -38,6 +39,8 @@ static HTMLObjectClass *parent_class = NULL;
 #ifdef GTKHTML_HAVE_PSPELL
 static SpellError * spell_error_new     (guint off, guint len);
 static void         spell_error_destroy (SpellError *se);
+static void         move_spell_errors   (GList *spell_errors, guint offset, gint delta);
+static GList *      remove_spell_errors (GList *spell_errors, guint offset, guint len);
 #endif
 
 
@@ -130,12 +133,15 @@ copy_helper (HTMLText *src,
 	GList *cur;
 
 	if (len < 0)
-		len = strlen (src->text);
+		len = unicode_strlen (src->text, -1);
 
-	dest->text = g_strndup (src->text + offset, len);
+	dest->text = g_strndup (html_text_get_text (src, offset),
+				unicode_offset_to_index (src->text, offset + len)
+				- unicode_offset_to_index (src->text, offset));
 	dest->text_len = len;
 
 	dest->font_style = src->font_style;
+	dest->face  = src->face;
 	dest->color = src->color;
 	html_color_ref (dest->color);
 
@@ -147,6 +153,9 @@ copy_helper (HTMLText *src,
 		cur->data = spell_error_new (se->off, se->len);
 		cur = cur->next;
 	}
+	dest->spell_errors = remove_spell_errors (dest->spell_errors, 0, offset);
+	dest->spell_errors = remove_spell_errors (dest->spell_errors, offset + len, src->text_len - offset - len);
+	move_spell_errors (dest->spell_errors, 0, -offset);
 #endif
 }
 
@@ -174,9 +183,9 @@ calc_size (HTMLObject *self,
 	text = HTML_TEXT (self);
 	font_style = html_text_get_font_style (text);
 
-	new_ascent = html_painter_calc_ascent (painter, font_style);
-	new_descent = html_painter_calc_descent (painter, font_style);
-	new_width = html_painter_calc_text_width (painter, text->text, text->text_len, font_style);
+	new_ascent = html_painter_calc_ascent (painter, font_style, text->face);
+	new_descent = html_painter_calc_descent (painter, font_style, text->face);
+	new_width = html_painter_calc_text_width (painter, text->text, text->text_len, font_style, text->face);
 
 	changed = FALSE;
 
@@ -210,7 +219,7 @@ calc_preferred_width (HTMLObject *self,
 
 	return html_painter_calc_text_width (painter,
 					     text->text, text->text_len,
-					     font_style);
+					     font_style, text->face);
 }
 
 static gint
@@ -245,16 +254,19 @@ html_text_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		return forward_get_nb_width (text, painter, begin);
 
 	/* if begins/ends with ' ' the width is 0 */
-	if ((begin && t [0] == ' ') || (!begin && t [text->text_len - 1] == ' '))
+	if ((begin && html_text_get_char (text, 0) == ' ')
+	    || (!begin && html_text_get_char (text, text->text_len - 1) == ' '))
 		return 0;
 
 	/* find end/begin of nb text */
-	t = (begin) ? strchr (t, ' ') : strrchr (t, ' ');
+	t = (begin) ? unicode_strchr (t, ' ') : strrchr (t, ' '); /* unicode_strrchr (t, ' '); */
 	if (!t)
 		return html_object_calc_preferred_width (HTML_OBJECT (text), painter);
 	return html_painter_calc_text_width (painter, (begin) ? text->text : t + 1,
-					     (begin) ? t - text->text : text->text_len - (t - text->text + 1),
-					     html_text_get_font_style (text));
+					     (begin)
+					     ? unicode_index_to_offset (text->text, t - text->text)
+					     : text->text_len - unicode_index_to_offset (text->text, t - text->text) + 1,
+					     html_text_get_font_style (text), text->face);
 }
 
 static gint
@@ -271,7 +283,7 @@ calc_min_width (HTMLObject *self,
 	font_style = html_text_get_font_style (text);
 	t          = text->text;
 
-	if (text->text_len == 0 || t [0] != ' ') {
+	if (text->text_len == 0 || html_text_get_char (text, 0) != ' ') {
 		obj = html_object_prev_not_slave (self);
 		w = (obj && html_object_is_text (obj)) ? html_text_get_nb_width (HTML_TEXT (obj), painter, FALSE) : 0;
 	}
@@ -280,8 +292,9 @@ calc_min_width (HTMLObject *self,
 		do {
 			space = strchr (t, ' ');
 			if (!space)
-				space = text->text + text->text_len;
-			w += html_painter_calc_text_width (painter, t, space - t, font_style);
+				space = text->text + unicode_offset_to_index (text->text, text->text_len);
+			w += html_painter_calc_text_width (painter, t, unicode_index_to_offset (t, space - t),
+							   font_style, text->face);
 			t = (*space) ? space + 1 : space;
 			if (!(*t))
 				break;
@@ -290,7 +303,7 @@ calc_min_width (HTMLObject *self,
 			w = 0;
 		} while (1);
 
-	if (text->text_len == 0 || text->text [text->text_len - 1] != ' ') {
+	if (text->text_len == 0 || html_text_get_char (text, text->text_len - 1) != ' ') {
 		obj = html_object_next_not_slave (self);
 		w += (obj && html_object_is_text (obj)) ? html_text_get_nb_width (HTML_TEXT (obj), painter, TRUE) : 0;
 	}
@@ -357,17 +370,41 @@ save (HTMLObject *self,
 	return TRUE;
 }
 
+static guchar *
+encode_text_to_8bit (gchar *utf8)
+{
+	guchar *raw;
+	guint i, len;
+	unicode_char_t uc;
+
+	len = unicode_strlen (utf8, -1);
+	raw = g_new (gchar, len + 1);
+	raw [len] = 0;
+
+	for (i = 0; i < len; i++) {
+		utf8 = unicode_get_utf8 (utf8, &uc);
+		/* raw [i] = uc == ENTITY_NBSP ? ' ' : uc & 0xff; */
+		raw [i] = uc & 0xff;
+	}
+
+	return raw;
+}
+
 static gboolean
 save_plain (HTMLObject *self,
-      HTMLEngineSaveState *state)
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
 {
 	HTMLText *text;
+	guchar *raw;
+	gboolean rv;
 	text = HTML_TEXT (self);
 
-	if (! html_engine_save_output_string (state, "%s", text->text))
-		return FALSE;
+	raw = encode_text_to_8bit (text->text);
+	rv  = html_engine_save_output_string (state, "%s", raw);
+	g_free (raw);
 	
-	return TRUE;
+	return rv;
 }
 
 
@@ -398,9 +435,7 @@ extract_text (HTMLText *text,
 		len = text->text_len - offset;
 
 	new = g_malloc (HTML_OBJECT (text)->klass->object_size);
-
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (HTML_OBJECT (text), HTML_OBJECT (new));
-
 	copy_helper (HTML_TEXT (text), HTML_TEXT (new), offset, len);
 
 	return new;
@@ -414,38 +449,107 @@ get_length (HTMLObject *self)
 
 /* #define DEBUG_NBSP */
 
-static void
-convert_nbsp (guchar *s, guint len)
+static gchar *
+convert_nbsp (gchar *s)
 {
 	/* state of automata:
 	   0..Text
 	   1..Sequence <space>&nbsp;...&nbsp;
 	*/
-	guint state=0;
+	gint delta;
+	guint state, pass;
+	unicode_char_t uc;
+	gchar *p, *op;
+	guchar *rv = NULL, *rp = NULL;
+
+	pass  = 0;
+	delta = 0;
 
 #ifdef DEBUG_NBSP
-	printf ("convert_nbsp: ");
+	printf ("convert_nbsp: %s --> ", s);
 #endif
-	while (len) {
-		if (*s == ENTITY_NBSP || *s == ' ') {
-			*s = state ? ENTITY_NBSP : ' ';
-			state = 1;
-		} else
-			state = 0;
+	for (pass=0; pass < 2; pass++) {
+		op = p = s;
+		state = 0;
+		while (*p && (p = unicode_get_utf8 (p, &uc))) {
+			if (uc == ENTITY_NBSP || uc == ' ') {
+				if (pass) {
+					if (state) {
 #ifdef DEBUG_NBSP
-		if (*s == ENTITY_NBSP)
-			printf ("&nbsp;");
-		else
-			printf ("%c", *s);
+						printf ("&nbsp;");
 #endif
-		len--;
-		s++;
+						*rp = 0xc2; rp ++;
+						*rp = 0xa0; rp ++;
+					} else {
+#ifdef DEBUG_NBSP
+						printf (" ");
+#endif
+						*rp = ' '; rp++;
+					}
+				} else {
+					if (uc == ENTITY_NBSP && !state) delta--;
+					if (uc == ' ' && state) delta++;
+				}
+				state = 1;
+			} else {
+				state = 0;
+				if (pass) {
+#ifdef DEBUG_NBSP
+					printf ("*");
+#endif
+					strncpy (rp, op, p - op);
+					rp += p - op;
+				}
+			}
+			op = p;
+		}
+		if (!pass) {
+			rp = rv = g_malloc (strlen (s) + delta + 1);
+			rv [strlen (s) + delta] = 0;
+		}
 	}
 #ifdef DEBUG_NBSP
 	printf ("\n");
 #endif
+	return rv;
 }
 
+#ifdef GTKHTML_HAVE_PSPELL
+static void 
+move_spell_errors (GList *spell_errors, guint offset, gint delta) 
+{ 
+	SpellError *se; 
+
+	if (!delta) return;
+
+	while (spell_errors) { 
+		se = (SpellError *) spell_errors->data; 
+		if (se->off >= offset) 
+			se->off += delta; 
+ 		spell_errors = spell_errors->next; 
+  	} 
+} 
+
+static GList *
+remove_spell_errors (GList *spell_errors, guint offset, guint len)
+{
+	SpellError *se; 
+	GList *cur, *cnext;
+
+	cur = spell_errors;
+	while (cur) { 
+		cnext = cur->next;
+		se = (SpellError *) cur->data; 
+		if (offset <= se->off && se->off <= offset + len) {
+			spell_errors = g_list_remove_link (spell_errors, cur);
+			spell_error_destroy (se);
+		}
+ 		cur = cnext;
+  	} 
+	return spell_errors;
+}
+#endif
+
 static guint
 insert_text (HTMLText *text,
 	     HTMLEngine *engine,
@@ -456,6 +560,7 @@ insert_text (HTMLText *text,
 	gchar *new_buffer;
 	guint old_len;
 	guint new_len;
+	guint l1, l2, ls;
 
 	old_len = text->text_len;
 	if (offset > old_len) {
@@ -466,26 +571,32 @@ insert_text (HTMLText *text,
 		offset = old_len;
 	}
 
-	new_len    = old_len + len;
-	new_buffer = g_malloc (new_len + 1);
+	new_len  = old_len + len;
+	l1         = unicode_offset_to_index (text->text, offset);
+	l2         = unicode_offset_to_index (text->text, text->text_len) - l1;
+	ls         = unicode_offset_to_index (s, len);
+	new_buffer = g_malloc (l1 + l2 + ls + 1);
 
 	/* concatenate strings */
-	memcpy (new_buffer,                text->text,        offset);
-	memcpy (new_buffer + offset,       s,                 len);
-	memcpy (new_buffer + offset + len, text->text+offset, old_len - offset);
-	new_buffer[new_len] = '\0';
-
-	/* do <space>&nbsp;...&nbsp; hack */
-	convert_nbsp (new_buffer, new_len);
+	memcpy (new_buffer,           text->text,      l1);
+	memcpy (new_buffer + l1,      s,               ls);
+	memcpy (new_buffer + l1 + ls, text->text + l1, l2);
+	new_buffer [l1 + l2 + ls] = '\0';
 
+#ifdef GTKHTML_HAVE_PSPELL
+	/* spell checking update */
+	move_spell_errors (text->spell_errors, offset, len);
+#endif
 	/* set new values */
 	g_free (text->text);
-	text->text = new_buffer;
+	text->text = convert_nbsp (new_buffer);
 	text->text_len = new_len;
+	g_free (new_buffer);
 
 	/* update */
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 	if (HTML_OBJECT (text)->parent != NULL) {
+		html_object_change_set (HTML_OBJECT (text)->parent, HTML_CHANGE_ALL);
 		if (! html_object_relayout (HTML_OBJECT (text)->parent,
 					    engine,
 					    HTML_OBJECT (text))) 
@@ -504,12 +615,13 @@ remove_text (HTMLText *text,
 	gchar *new_buffer;
 	guint old_len;
 	guint new_len;
+	guint l1, l2, lw;
 
 	/* The following code is very stupid and quite inefficient, but it is
            just for interactive editing so most likely people won't even
            notice.  */
 
-	old_len = strlen (text->text);
+	old_len = text->text_len;
 
 	if (offset > old_len) {
 		g_warning ("Cursor offset out of range for HTMLText::remove_text().");
@@ -521,18 +633,25 @@ remove_text (HTMLText *text,
 
 	new_len = old_len - len;
 
+	l1 = unicode_offset_to_index (text->text, offset);
+	l2 = unicode_offset_to_index (text->text, offset + len);
+	lw = unicode_offset_to_index (text->text, text->text_len);
+
 	/* concat strings */
-	new_buffer = g_malloc (new_len + 1);
-	memcpy (new_buffer,          text->text,                offset);
-	memcpy (new_buffer + offset, text->text + offset + len, old_len - offset - len + 1);
-
-	/* do <space>&nbsp;...&nbsp; hack */
-	convert_nbsp (new_buffer, new_len);
+	new_buffer = g_malloc (lw - (l2 - l1) + 1);
+	memcpy (new_buffer,      text->text,      l1);
+	memcpy (new_buffer + l1, text->text + l2, lw - l2 + 1);
 
+#ifdef GTKHTML_HAVE_PSPELL
+	/* spell checking update */
+	text->spell_errors = remove_spell_errors (text->spell_errors, offset, len);
+	move_spell_errors (text->spell_errors, offset + len, -len);
+#endif
 	/* set new values */
 	g_free (text->text);
-	text->text = new_buffer;
+	text->text = convert_nbsp (new_buffer);
 	text->text_len = new_len;
+	g_free (new_buffer);
 
 	/* update */
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
@@ -570,11 +689,36 @@ get_cursor_base (HTMLObject *self,
 		font_style = html_text_get_font_style (HTML_TEXT (self));
 		*x += html_painter_calc_text_width (painter,
 						    HTML_TEXT (self)->text,
-						    offset, font_style);
+						    offset, font_style, HTML_TEXT (self)->face);
 	}
 }
 
 
+
+#ifdef GTKHTML_HAVE_PSPELL
+static void
+split_spell_errors (HTMLText *self, HTMLText *new, guint offset)
+{
+	GList *cur;
+	SpellError *se;
+
+	cur = self->spell_errors;
+	while (cur) {
+		se = (SpellError *) cur->data;
+		if (se->off >= offset) {
+			if (cur->prev)
+				cur->prev->next = NULL;
+			else
+				self->spell_errors = NULL;
+			new->spell_errors = cur;
+			move_spell_errors (new->spell_errors, offset, -offset);
+			break;
+		}
+		cur = cur->next;
+	}
+}
+#endif
+
 static HTMLText *
 split (HTMLText *self,
        guint offset)
@@ -583,13 +727,17 @@ split (HTMLText *self,
 
 	new = g_malloc (HTML_OBJECT (self)->klass->object_size);
 	html_text_init (new, HTML_TEXT_CLASS (HTML_OBJECT (self)->klass),
-			self->text + offset, -1, self->font_style, self->color);
+			html_text_get_text (self, offset), -1, self->font_style, self->color);
 
-	self->text = g_realloc (self->text, offset + 1);
-	self->text[offset] = '\0';
+	self->text = g_realloc (self->text, unicode_offset_to_index (self->text, offset) + 1);
+	self->text [unicode_offset_to_index (self->text, offset)] = '\0';
 	self->text_len = offset;
 	html_object_change_set (HTML_OBJECT (self), HTML_CHANGE_MIN_WIDTH);
 
+#ifdef GTKHTML_HAVE_PSPELL
+	split_spell_errors (self, new, offset);
+#endif
+
 	return new;
 }
 
@@ -759,17 +907,15 @@ html_text_init (HTMLText *text_object,
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
 
 	if (len == -1) {
-		text_object->text_len = strlen (text);
+		text_object->text_len = unicode_strlen (text, -1);
 		text_object->text = g_strdup (text);
 	} else {
 		text_object->text_len = len;
-		text_object->text = g_strndup (text, len);
+		text_object->text = g_strndup (text, unicode_offset_to_index (text, len));
 	}
 
-	/* do <space>&nbsp;...&nbsp; hack */
-	convert_nbsp (text_object->text, text_object->text_len);
-
 	text_object->font_style = font_style;
+	text_object->face = NULL;
 	html_color_ref (color);
 	text_object->color = color;
 #ifdef GTKHTML_HAVE_PSPELL
@@ -924,6 +1070,14 @@ html_text_set_color (HTMLText *text,
 	return (* HT_CLASS (text)->set_color) (text, engine, color);
 }
 
+void
+html_text_set_font_face (HTMLText *text, HTMLFontFace *face)
+{
+	if (text->face)
+		g_free (text->face);
+	text->face = g_strdup (face);
+}
+
 void
 html_text_set_text (HTMLText *text, const gchar *new_text)
 {
@@ -936,6 +1090,7 @@ html_text_set_text (HTMLText *text, const gchar *new_text)
 
 /* spell checking */
 
+#include "htmlinterval.h"
 
 static SpellError *
 spell_error_new (guint off, guint len)
@@ -962,10 +1117,87 @@ html_text_spell_errors_clear (HTMLText *text)
 	text->spell_errors = NULL;
 }
 
+void
+html_text_spell_errors_clear_interval (HTMLText *text, HTMLInterval *i)
+{
+	GList *cur, *cnext;
+	SpellError *se;
+	guint offset, len;
+
+	offset = html_interval_get_start  (i, HTML_OBJECT (text));
+	len    = html_interval_get_length (i, HTML_OBJECT (text));
+	cur    = text->spell_errors;
+
+	/* printf ("html_text_spell_errors_clear_interval %s %d %d\n", text->text, offset, len); */
+
+	while (cur) {
+		cnext = cur->next;
+		se    = (SpellError *) cur->data;
+		/* test overlap */
+		if (MAX (offset, se->off) <= MIN (se->off + se->len, offset + len)) {
+			text->spell_errors = g_list_remove_link (text->spell_errors, cur);
+			g_list_free (cur);
+		}
+		cur = cnext;
+	}
+}
+
+static gint
+se_cmp (gconstpointer a, gconstpointer b)
+{
+	guint o1, o2;
+
+	o1 = ((SpellError *) a)->off;
+	o2 = ((SpellError *) b)->off;
+
+	if (o1 < o2)  return -1;
+	if (o1 == o2) return 0;
+	return 1;
+}
+
 void
 html_text_spell_errors_add (HTMLText *text, guint off, guint len)
 {
-	text->spell_errors = g_list_append (text->spell_errors, spell_error_new (off, len));
+	/* GList *cur;
+	SpellError *se;
+	cur = */
+
+	text->spell_errors = g_list_insert_sorted (text->spell_errors, spell_error_new (off, len), se_cmp);
+
+	/* printf ("---------------------------------------\n");
+	while (cur) {
+		se = (SpellError *) cur->data;
+		printf ("off: %d len: %d\n", se->off, se->len);
+		cur = cur->next;
+	}
+	printf ("---------------------------------------\n"); */
 }
 
 #endif
+
+guint
+html_text_get_bytes (HTMLText *text)
+{
+	return strlen (text->text);
+}
+
+guint
+html_text_get_index (HTMLText *text, guint offset)
+{
+	return unicode_offset_to_index (text->text, offset);
+}
+
+unicode_char_t
+html_text_get_char (HTMLText *text, guint offset)
+{
+	unicode_char_t uc;
+
+	unicode_get_utf8 (text->text + html_text_get_index (text, offset), &uc);
+	return uc;
+}
+
+gchar *
+html_text_get_text (HTMLText *text, guint offset)
+{
+	return text->text + html_text_get_index (text, offset);
+}
