@@ -67,6 +67,7 @@ static void bug_cb (GtkWidget *widget, gpointer data);
 static void slow_cb (GtkWidget *widget, gpointer data);
 static void stop_cb (GtkWidget *widget, gpointer data);
 static void dump_cb (GtkWidget *widget, gpointer data);
+static void dump_simple_cb (GtkWidget *widget, gpointer data);
 static void forward_cb (GtkWidget *widget, gpointer data);
 static void back_cb (GtkWidget *widget, gpointer data);
 static void home_cb (GtkWidget *widget, gpointer data);
@@ -145,6 +146,8 @@ static GnomeUIInfo debug_menu[] = {
 	  bug_cb, NULL, NULL, 0, 0, 0, 0},
 	{ GNOME_APP_UI_ITEM, "Dump Object tree", "Dump Object tree to stdout",
 	  dump_cb, NULL, NULL, 0, 0, 0, 0},
+	{ GNOME_APP_UI_ITEM, "Dump Object tree (simple)", "Dump Simple Object tree to stdout",
+	  dump_simple_cb, NULL, NULL, 0, 0, 0, 0},
 	GNOMEUIINFO_TOGGLEITEM("Slow loading", "Load documents slowly", slow_cb, NULL),
 	{ GNOME_APP_UI_ITEM, "Force resize", "Force a resize event",
 	  resize_cb, NULL, NULL, 0, 0, 0 },
@@ -283,6 +286,28 @@ create_toolbars (GtkWidget *app)
 
 }
 
+static gint page_num;
+static GnomeFont *font;
+
+static void
+print_footer (GtkHTML *html, GnomePrintContext *context,
+	      gdouble x, gdouble y, gdouble width, gdouble height)
+{
+	gchar *text = g_strdup_printf ("- %d -", page_num);
+	gdouble tw = gnome_font_get_width_string (font, "text");
+
+	if (font) {
+		gnome_print_newpath     (context);
+		gnome_print_setrgbcolor (context, .0, .0, .0);
+		gnome_print_moveto      (context, x + (width - tw)/2, y - (height + gnome_font_get_ascender (font))/2);
+		gnome_print_setfont     (context, font);
+		gnome_print_show        (context, text);
+	}
+
+	g_free (text);
+	page_num++;
+}
+
 static void
 print_preview_cb (GtkWidget *widget,
 		  gpointer data)
@@ -296,7 +321,10 @@ print_preview_cb (GtkWidget *widget,
 
 	print_context = gnome_print_master_get_context (print_master);
 
-	gtk_html_print (html, print_context);
+	page_num = 1;
+	font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, FALSE, 12);
+	gtk_html_print_with_header_footer (html, print_context, .0, .03, NULL, print_footer);
+	if (font) gtk_object_unref (GTK_OBJECT (font));
 
 	preview = GTK_WIDGET (gnome_print_master_preview_new (print_master, "HTML Print Preview"));
 	gtk_widget_show (preview);
@@ -313,6 +341,15 @@ dump_cb (GtkWidget *widget, gpointer data)
 	gtk_html_debug_dump_tree (html->engine->clue, 0);
 }
 
+static void
+dump_simple_cb (GtkWidget *widget, gpointer data)
+{
+	g_print ("Simple Object Tree\n");
+	g_print ("-----------\n");
+
+	gtk_html_debug_dump_tree_simple (html->engine->clue, 0);
+}
+
 static void
 resize_cb (GtkWidget *widget, gpointer data)
 {
@@ -622,14 +659,13 @@ object_timeout(GtkHTMLEmbedded *eb)
 {
 	GtkWidget *w;
 
-	w = gnome_color_picker_new();
+	w = gtk_check_button_new();
 	gtk_widget_show(w);
 
-	printf("inserting custom widget after a delay ...\n");
-
-	gtk_html_embedded_set_descent(eb, rand()%8);
-	gtk_container_add(GTK_CONTAINER(eb), w);
-	gtk_widget_unref(GTK_WIDGET (eb));
+	/* printf("inserting custom widget after a delay ...\n");
+	gtk_html_embedded_set_descent(eb, rand()%8); */
+	gtk_container_add (GTK_CONTAINER(eb), w);
+	gtk_widget_unref (GTK_WIDGET (eb));
 
 	return FALSE;
 }
@@ -637,13 +673,14 @@ object_timeout(GtkHTMLEmbedded *eb)
 static gboolean
 object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 {
-	printf("object requested, wiaint a bit before creating it ...\n");
+	/* printf("object requested, wiaint a bit before creating it ...\n"); */
 
 	if (strcmp (eb->classid, "mine:NULL") == 0)
 		return FALSE;
 
 	gtk_widget_ref (GTK_WIDGET (eb));
-	gtk_timeout_add(rand() % 5000 + 1000, (GtkFunction) object_timeout, eb);
+	/* gtk_timeout_add(rand() % 5000 + 1000, (GtkFunction) object_timeout, eb); */
+	object_timeout (eb);
 	return TRUE;
 }
 
@@ -1111,7 +1148,7 @@ main (gint argc, gchar *argv[])
 #endif
 	gtk_widget_realize (GTK_WIDGET (html));
 
-	gtk_window_set_default_size (GTK_WINDOW (app), 500, 400);
+	gtk_window_set_default_size (GTK_WINDOW (app), 540, 400);
 	gtk_window_set_focus (GTK_WINDOW (app), GTK_WIDGET (html));
 
 	gtk_widget_show_all (app);
@@ -1128,7 +1165,7 @@ main (gint argc, gchar *argv[])
 
 #ifdef MEMDEBUG
 
-	// gtk_widget_unref (html_widget);
+	/* gtk_widget_unref (html_widget); */
 	free (p);
 #endif
 
